@@ -163,7 +163,7 @@ bool bgp_adj_out_lookup(struct peer *peer, struct bgp_dest *dest,
 
 
 void bgp_adj_in_set(struct bgp_dest *dest, struct peer *peer, struct attr *attr,
-		    uint32_t addpath_id)
+		    uint32_t addpath_id, struct bgp_labels *labels)
 {
 	struct bgp_adj_in *adj;
 
@@ -173,6 +173,10 @@ void bgp_adj_in_set(struct bgp_dest *dest, struct peer *peer, struct attr *attr,
 				bgp_attr_unintern(&adj->attr);
 				adj->attr = bgp_attr_intern(attr);
 			}
+			if (!bgp_labels_cmp(adj->labels, labels)) {
+				bgp_labels_unintern(&adj->labels);
+				adj->labels = bgp_labels_intern(labels);
+			}
 			return;
 		}
 	}
@@ -181,13 +185,18 @@ void bgp_adj_in_set(struct bgp_dest *dest, struct peer *peer, struct attr *attr,
 	adj->attr = bgp_attr_intern(attr);
 	adj->uptime = monotime(NULL);
 	adj->addpath_rx_id = addpath_id;
+	adj->labels = bgp_labels_intern(labels);
 	BGP_ADJ_IN_ADD(dest, adj);
+	peer->stat_pfx_adj_rib_in++;
 	bgp_dest_lock_node(dest);
 }
 
 void bgp_adj_in_remove(struct bgp_dest **dest, struct bgp_adj_in *bai)
 {
 	bgp_attr_unintern(&bai->attr);
+	bgp_labels_unintern(&bai->labels);
+	if (bai->peer)
+		bai->peer->stat_pfx_adj_rib_in--;
 	BGP_ADJ_IN_DEL(*dest, bai);
 	*dest = bgp_dest_unlock_node(*dest);
 	peer_unlock(bai->peer); /* adj_in peer reference */
