@@ -318,12 +318,14 @@ struct attr {
 
 	/* AIGP Metric */
 	uint64_t aigp_metric;
+
+	/* Next-hop characteristics */
+	struct bgp_nhc *nhc;
 };
 
 /* rmap_change_flags definition */
 #define BATTR_RMAP_IPV4_NHOP_CHANGED (1 << 0)
 #define BATTR_RMAP_NEXTHOP_PEER_ADDRESS (1 << 1)
-#define BATTR_REFLECTED (1 << 2)
 #define BATTR_RMAP_NEXTHOP_UNCHANGED (1 << 3)
 #define BATTR_RMAP_IPV6_GLOBAL_NHOP_CHANGED (1 << 4)
 #define BATTR_RMAP_IPV6_LL_NHOP_CHANGED (1 << 5)
@@ -392,12 +394,12 @@ extern bgp_size_t bgp_packet_attribute(struct bgp *bgp, struct peer *peer, struc
 				       struct prefix *p, afi_t afi, safi_t safi, struct peer *from,
 				       struct prefix_rd *prd, mpls_label_t *label,
 				       uint8_t num_labels, bool addpath_capable,
-				       uint32_t addpath_tx_id);
+				       uint32_t addpath_tx_id, struct bgp_path_info *bpi);
 extern void bgp_dump_routes_attr(struct stream *s, struct bgp_path_info *bpi,
 				 const struct prefix *p);
 extern bool attrhash_cmp(const void *arg1, const void *arg2);
 extern unsigned int attrhash_key_make(const void *p);
-extern void attr_show_all(struct vty *vty);
+extern void attr_show_all(struct vty *vty, bool summary);
 extern unsigned long int attr_count(void);
 extern unsigned long int attr_unknown_count(void);
 extern void bgp_path_attribute_discard_vty(struct vty *vty, struct peer *peer,
@@ -515,7 +517,7 @@ static inline void bgp_attr_set_ecommunity(struct attr *attr,
 {
 	attr->ecommunity = ecomm;
 
-	if (ecomm)
+	if (ecomm && ecomm->size)
 		SET_FLAG(attr->flag, ATTR_FLAG_BIT(BGP_ATTR_EXT_COMMUNITIES));
 	else
 		UNSET_FLAG(attr->flag, ATTR_FLAG_BIT(BGP_ATTR_EXT_COMMUNITIES));
@@ -566,7 +568,7 @@ static inline void bgp_attr_set_ipv6_ecommunity(struct attr *attr,
 {
 	attr->ipv6_ecommunity = ipv6_ecomm;
 
-	if (ipv6_ecomm)
+	if (ipv6_ecomm && ipv6_ecomm->size)
 		SET_FLAG(attr->flag,
 			 ATTR_FLAG_BIT(BGP_ATTR_IPV6_EXT_COMMUNITIES));
 	else
@@ -585,6 +587,21 @@ static inline void bgp_attr_set_transit(struct attr *attr,
 	attr->transit = transit;
 }
 
+static inline struct bgp_nhc *bgp_attr_get_nhc(const struct attr *attr)
+{
+	return attr->nhc;
+}
+
+static inline void bgp_attr_set_nhc(struct attr *attr, struct bgp_nhc *bnc)
+{
+	attr->nhc = bnc;
+
+	if (bnc)
+		SET_FLAG(attr->flag, ATTR_FLAG_BIT(BGP_ATTR_NHC));
+	else
+		UNSET_FLAG(attr->flag, ATTR_FLAG_BIT(BGP_ATTR_NHC));
+}
+
 #define AIGP_TRANSMIT_ALLOWED(peer)                                                                \
 	(CHECK_FLAG((peer)->flags, PEER_FLAG_AIGP) || ((peer)->sub_sort == BGP_PEER_EBGP_OAD) ||   \
 	 ((peer)->sort != BGP_PEER_EBGP))
@@ -604,10 +621,17 @@ static inline uint64_t bgp_aigp_metric_total(struct bgp_path_info *bpi)
 {
 	uint64_t aigp = bgp_attr_get_aigp_metric(bpi->attr);
 
-	if (bpi->nexthop)
-		return aigp + bpi->nexthop->metric;
-	else
+	/* Don't increment if it's locally sourced */
+	if (bpi->peer == bpi->peer->bgp->peer_self)
 		return aigp;
+
+	return bpi->extra ? (aigp + bpi->extra->igpmetric) : aigp;
+}
+
+static inline void bgp_attr_set_med(struct attr *attr, uint32_t med)
+{
+	attr->med = med;
+	SET_FLAG(attr->flag, ATTR_FLAG_BIT(BGP_ATTR_MULTI_EXIT_DISC));
 }
 
 static inline struct cluster_list *bgp_attr_get_cluster(const struct attr *attr)
