@@ -11,8 +11,8 @@
 #include "queue.h"
 #include "filter.h"
 #include "stream.h"
-#include "jhash.h"
 #include "frrstr.h"
+#include "frregex_real.h"
 
 #include "bgpd/bgpd.h"
 #include "bgpd/bgp_community.h"
@@ -507,7 +507,7 @@ static char *community_str_get(struct community *com, int i)
 
 /* Internal function to perform regular expression match for
  * a single community. */
-static bool community_regexp_include(regex_t *reg, struct community *com, int i)
+static bool community_regexp_include(struct frregex *reg, struct community *com, int i)
 {
 	char *str;
 	int rv;
@@ -520,7 +520,7 @@ static bool community_regexp_include(regex_t *reg, struct community *com, int i)
 		str = community_str_get(com, i);
 
 	/* Regular expression match.  */
-	rv = regexec(reg, str, 0, NULL, 0);
+	rv = regexec(&reg->real, str, 0, NULL, 0);
 
 	XFREE(MTYPE_COMMUNITY_STR, str);
 
@@ -529,29 +529,43 @@ static bool community_regexp_include(regex_t *reg, struct community *com, int i)
 
 /* Internal function to perform regular expression match for community
    attribute.  */
-static bool community_regexp_match(struct community *com, regex_t *reg)
+static bool community_regexp_match(struct community *com, struct frregex *reg)
 {
 	const char *str;
 	char *regstr;
 	int rv;
+	bool translate_alias = !!bgp_ca_alias_hash->count;
 
 	/* When there is no communities attribute it is treated as empty
 	   string.  */
 	if (com == NULL || com->size == 0)
-		str = "";
-	else
-		str = community_str(com, false, true);
+		return false;
 
-	regstr = bgp_alias2community_str(str);
+	str = community_str(com, false, translate_alias);
+
+	/* If at least one community alias is configured, then let's
+	 * do the work, otherwise we don't need to spend time on splitting
+	 * stuff and creating a new string.
+	 */
+	regstr = translate_alias ? bgp_alias2community_str(str) : (char *)str;
 
 	/* Regular expression match.  */
-	rv = regexec(reg, regstr, 0, NULL, 0);
+	rv = regexec(&reg->real, regstr, 0, NULL, 0);
 
-	XFREE(MTYPE_TMP, regstr);
+	/* This is allocated by frrstr_join(), and needs to be freed
+	 * only if it was created.
+	 */
+	if (translate_alias)
+		XFREE(MTYPE_TMP, regstr);
 
 	return rv == 0;
 }
 
+static char *ecommunity_str_get(struct ecommunity *ecom, int i)
+{
+	return ecommunity_ecom2str_one(ecom, ECOMMUNITY_FORMAT_DISPLAY, i);
+}
+
 static char *lcommunity_str_get(struct lcommunity *lcom, int i)
 {
 	struct lcommunity_val lcomval;
@@ -580,8 +594,7 @@ static char *lcommunity_str_get(struct lcommunity *lcom, int i)
 
 /* Internal function to perform regular expression match for
  * a single community. */
-static bool lcommunity_regexp_include(regex_t *reg, struct lcommunity *lcom,
-				      int i)
+static bool lcommunity_regexp_include(struct frregex *reg, struct lcommunity *lcom, int i)
 {
 	char *str;
 
@@ -593,7 +606,7 @@ static bool lcommunity_regexp_include(regex_t *reg, struct lcommunity *lcom,
 		str = lcommunity_str_get(lcom, i);
 
 	/* Regular expression match.  */
-	if (regexec(reg, str, 0, NULL, 0) == 0) {
+	if (regexec(&reg->real, str, 0, NULL, 0) == 0) {
 		XFREE(MTYPE_LCOMMUNITY_STR, str);
 		return true;
 	}
@@ -603,31 +616,63 @@ static bool lcommunity_regexp_include(regex_t *reg, struct lcommunity *lcom,
 	return false;
 }
 
-static bool lcommunity_regexp_match(struct lcommunity *com, regex_t *reg)
+/* Internal function to perform regular expression match for a single ecommunity. */
+static bool ecommunity_regexp_include(struct frregex *reg, struct ecommunity *ecom, int i)
+{
+	char *str;
+
+	/* When there is no communities attribute it is treated as empty string.
+	 */
+	if (ecom == NULL || ecom->size == 0)
+		str = XSTRDUP(MTYPE_ECOMMUNITY_STR, "");
+	else
+		str = ecommunity_str_get(ecom, i);
+
+	/* Regular expression match.  */
+	if (regexec(&reg->real, str, 0, NULL, 0) == 0) {
+		XFREE(MTYPE_ECOMMUNITY_STR, str);
+		return true;
+	}
+
+	XFREE(MTYPE_ECOMMUNITY_STR, str);
+	/* No match.  */
+	return false;
+}
+
+static bool lcommunity_regexp_match(struct lcommunity *com, struct frregex *reg)
 {
 	const char *str;
 	char *regstr;
 	int rv;
+	bool translate_alias = !!bgp_ca_alias_hash->count;
 
 	/* When there is no communities attribute it is treated as empty
 	   string.  */
 	if (com == NULL || com->size == 0)
-		str = "";
-	else
-		str = lcommunity_str(com, false, true);
+		return false;
 
-	regstr = bgp_alias2community_str(str);
+	str = lcommunity_str(com, false, translate_alias);
+
+	/* If at least one community alias is configured, then let's
+	 * do the work, otherwise we don't need to spend time on splitting
+	 * stuff and creating a new string.
+	 */
+	regstr = translate_alias ? bgp_alias2community_str(str) : (char *)str;
 
 	/* Regular expression match.  */
-	rv = regexec(reg, regstr, 0, NULL, 0);
+	rv = regexec(&reg->real, regstr, 0, NULL, 0);
 
-	XFREE(MTYPE_TMP, regstr);
+	/* This is allocated by frrstr_join(), and needs to be freed
+	 * only if it was created.
+	 */
+	if (translate_alias)
+		XFREE(MTYPE_TMP, regstr);
 
 	return rv == 0;
 }
 
 
-static bool ecommunity_regexp_match(struct ecommunity *ecom, regex_t *reg)
+static bool ecommunity_regexp_match(struct ecommunity *ecom, struct frregex *reg)
 {
 	const char *str;
 
@@ -639,7 +684,7 @@ static bool ecommunity_regexp_match(struct ecommunity *ecom, regex_t *reg)
 		str = ecommunity_str(ecom);
 
 	/* Regular expression match.  */
-	if (regexec(reg, str, 0, NULL, 0) == 0)
+	if (regexec(&reg->real, str, 0, NULL, 0) == 0)
 		return true;
 
 	/* No match.  */
@@ -680,6 +725,24 @@ bool lcommunity_list_match(struct lcommunity *lcom, struct community_list *list)
 	return false;
 }
 
+/* Perform exact matching. In case of expanded extended-community-list, do
+ * same thing as ecommunity_list_match().
+ */
+bool ecommunity_list_exact_match(struct ecommunity *ecom, struct community_list *list)
+{
+	struct community_entry *entry;
+
+	for (entry = list->head; entry; entry = entry->next) {
+		if (entry->style == EXTCOMMUNITY_LIST_STANDARD) {
+			if (ecommunity_cmp(ecom, entry->u.lcom))
+				return entry->direct == COMMUNITY_PERMIT;
+		} else if (entry->style == EXTCOMMUNITY_LIST_EXPANDED) {
+			if (ecommunity_regexp_match(ecom, entry->reg))
+				return entry->direct == COMMUNITY_PERMIT;
+		}
+	}
+	return false;
+}
 
 /* Perform exact matching.  In case of expanded large-community-list, do
  * same thing as lcommunity_list_match().
@@ -850,7 +913,7 @@ int community_list_set(struct community_list_handler *ch, const char *name,
 	struct community_entry *entry = NULL;
 	struct community_list *list;
 	struct community *com = NULL;
-	regex_t *regex = NULL;
+	struct frregex *regex = NULL;
 	int64_t seqnum = COMMUNITY_SEQ_NUMBER_AUTO;
 
 	if (seq)
@@ -964,6 +1027,27 @@ bool lcommunity_list_any_match(struct lcommunity *lcom,
 	return false;
 }
 
+bool ecommunity_list_any_match(struct ecommunity *ecom, struct community_list *list)
+{
+	struct community_entry *entry;
+	uint8_t *ptr;
+	uint32_t i;
+
+	for (i = 0; i < ecom->size; i++) {
+		ptr = ecom->val + (i * ecom->unit_size);
+
+		for (entry = list->head; entry; entry = entry->next) {
+			if ((entry->style == EXTCOMMUNITY_LIST_STANDARD) &&
+			    ecommunity_include_one(entry->u.ecom, ptr))
+				return entry->direct == COMMUNITY_PERMIT;
+			if ((entry->style == EXTCOMMUNITY_LIST_EXPANDED) &&
+			    ecommunity_regexp_include(entry->reg, ecom, i))
+				return entry->direct == COMMUNITY_PERMIT;
+		}
+	}
+	return false;
+}
+
 /* Delete all permitted large communities in the list from com.  */
 struct lcommunity *lcommunity_list_match_delete(struct lcommunity *lcom,
 						struct community_list *list)
@@ -1057,7 +1141,7 @@ bool lcommunity_list_valid(const char *community, int style)
 	char **splits, **communities;
 	char *endptr;
 	int num, num_communities;
-	regex_t *regres;
+	struct frregex *regres;
 	int invalid = 0;
 
 	frrstr_split(community, " ", &communities, &num_communities);
@@ -1113,7 +1197,7 @@ int lcommunity_list_set(struct community_list_handler *ch, const char *name,
 	struct community_entry *entry = NULL;
 	struct community_list *list;
 	struct lcommunity *lcom = NULL;
-	regex_t *regex = NULL;
+	struct frregex *regex = NULL;
 	int64_t seqnum = COMMUNITY_SEQ_NUMBER_AUTO;
 
 	if (seq)
@@ -1178,7 +1262,7 @@ void lcommunity_list_unset(struct community_list_handler *ch, const char *name,
 	struct community_entry *entry = NULL;
 	struct community_list *list;
 	struct lcommunity *lcom = NULL;
-	regex_t *regex = NULL;
+	struct frregex *regex = NULL;
 
 	/* Lookup community list.  */
 	list = community_list_lookup(ch, name, 0, LARGE_COMMUNITY_LIST_MASTER);
@@ -1226,7 +1310,7 @@ int extcommunity_list_set(struct community_list_handler *ch, const char *name,
 	struct community_entry *entry = NULL;
 	struct community_list *list;
 	struct ecommunity *ecom = NULL;
-	regex_t *regex = NULL;
+	struct frregex *regex = NULL;
 	int64_t seqnum = COMMUNITY_SEQ_NUMBER_AUTO;
 
 	if (seq)
