@@ -1,21 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* Community attribute related functions.
  * Copyright (C) 1998, 2001 Kunihiro Ishiguro
- *
- * This file is part of GNU Zebra.
- *
- * GNU Zebra is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * GNU Zebra is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -28,6 +13,7 @@
 
 #include "bgpd/bgp_memory.h"
 #include "bgpd/bgp_community.h"
+#include "bgpd/bgp_community_alias.h"
 
 /* Hash of community attribute. */
 static struct hash *comhash;
@@ -56,14 +42,10 @@ void community_free(struct community **com)
 }
 
 /* Add one community value to the community. */
-static void community_add_val(struct community *com, uint32_t val)
+void community_add_val(struct community *com, uint32_t val)
 {
 	com->size++;
-	if (com->val)
-		com->val = XREALLOC(MTYPE_COMMUNITY_VAL, com->val,
-				    com_length(com));
-	else
-		com->val = XMALLOC(MTYPE_COMMUNITY_VAL, com_length(com));
+	com->val = XREALLOC(MTYPE_COMMUNITY_VAL, com->val, com_length(com));
 
 	val = htonl(val);
 	memcpy(com_lastval(com), &val, sizeof(uint32_t));
@@ -186,7 +168,6 @@ struct community *community_uniq_sort(struct community *com)
 
    For Well-known communities value, below keyword is used.
 
-   0x0             "internet"
    0xFFFF0000      "graceful-shutdown"
    0xFFFF0001      "accept-own"
    0xFFFF0002      "route-filter-translated-v4"
@@ -203,7 +184,8 @@ struct community *community_uniq_sort(struct community *com)
    0xFFFFFF04      "no-peer"
 
    For other values, "AS:VAL" format is used.  */
-static void set_community_string(struct community *com, bool make_json)
+static void set_community_string(struct community *com, bool make_json,
+				 bool translate_alias)
 {
 	int i;
 	char *str;
@@ -246,9 +228,6 @@ static void set_community_string(struct community *com, bool make_json)
 		comval = ntohl(comval);
 
 		switch (comval) {
-		case COMMUNITY_INTERNET:
-			len += strlen(" internet");
-			break;
 		case COMMUNITY_GSHUT:
 			len += strlen(" graceful-shutdown");
 			break;
@@ -292,7 +271,7 @@ static void set_community_string(struct community *com, bool make_json)
 			len += strlen(" no-peer");
 			break;
 		default:
-			len += strlen(" 65536:65535");
+			len = BUFSIZ;
 			break;
 		}
 	}
@@ -312,15 +291,6 @@ static void set_community_string(struct community *com, bool make_json)
 			strlcat(str, " ", len);
 
 		switch (comval) {
-		case COMMUNITY_INTERNET:
-			strlcat(str, "internet", len);
-			if (make_json) {
-				json_string =
-					json_object_new_string("internet");
-				json_object_array_add(json_community_list,
-						      json_string);
-			}
-			break;
 		case COMMUNITY_GSHUT:
 			strlcat(str, "graceful-shutdown", len);
 			if (make_json) {
@@ -446,13 +416,16 @@ static void set_community_string(struct community *com, bool make_json)
 			}
 			break;
 		default:
-			as = (comval >> 16) & 0xFFFF;
-			val = comval & 0xFFFF;
+			as = CHECK_FLAG((comval >> 16), 0xFFFF);
+			val = CHECK_FLAG(comval, 0xFFFF);
 			char buf[32];
 			snprintf(buf, sizeof(buf), "%u:%d", as, val);
-			strlcat(str, buf, len);
+			const char *com2alias =
+				translate_alias ? bgp_community2alias(buf) : buf;
+
+			strlcat(str, com2alias, len);
 			if (make_json) {
-				json_string = json_object_new_string(buf);
+				json_string = json_object_new_string(com2alias);
 				json_object_array_add(json_community_list,
 						      json_string);
 			}
@@ -488,7 +461,7 @@ struct community *community_intern(struct community *com)
 
 	/* Make string.  */
 	if (!find->str)
-		set_community_string(find, false);
+		set_community_string(find, false, true);
 
 	return find;
 }
@@ -498,6 +471,9 @@ void community_unintern(struct community **com)
 {
 	struct community *ret;
 
+	if (!*com)
+		return;
+
 	if ((*com)->refcnt)
 		(*com)->refcnt--;
 
@@ -545,8 +521,8 @@ struct community *community_dup(struct community *com)
 	return new;
 }
 
-/* Retrun string representation of communities attribute. */
-char *community_str(struct community *com, bool make_json)
+/* Return string representation of communities attribute. */
+char *community_str(struct community *com, bool make_json, bool translate_alias)
 {
 	if (!com)
 		return NULL;
@@ -555,7 +531,7 @@ char *community_str(struct community *com, bool make_json)
 		XFREE(MTYPE_COMMUNITY_STR, com->str);
 
 	if (!com->str)
-		set_community_string(com, make_json);
+		set_community_string(com, make_json, translate_alias);
 	return com->str;
 }
 
@@ -595,8 +571,6 @@ bool community_match(const struct community *com1, const struct community *com2)
 		return false;
 }
 
-/* If two aspath have same value then return 1 else return 0. This
-   function is used by hash package. */
 bool community_cmp(const struct community *com1, const struct community *com2)
 {
 	if (com1 == NULL && com2 == NULL)
@@ -615,13 +589,8 @@ bool community_cmp(const struct community *com1, const struct community *com2)
 struct community *community_merge(struct community *com1,
 				  struct community *com2)
 {
-	if (com1->val)
-		com1->val =
-			XREALLOC(MTYPE_COMMUNITY_VAL, com1->val,
-				 (com1->size + com2->size) * COMMUNITY_SIZE);
-	else
-		com1->val = XMALLOC(MTYPE_COMMUNITY_VAL,
-				    (com1->size + com2->size) * COMMUNITY_SIZE);
+	com1->val = XREALLOC(MTYPE_COMMUNITY_VAL, com1->val,
+			     (com1->size + com2->size) * COMMUNITY_SIZE);
 
 	memcpy(com1->val + com1->size, com2->val, com2->size * COMMUNITY_SIZE);
 	com1->size += com2->size;
@@ -690,12 +659,6 @@ community_gettoken(const char *buf, enum community_token *token, uint32_t *val)
 
 	/* Well known community string check. */
 	if (isalpha((unsigned char)*p)) {
-		if (strncmp(p, "internet", strlen("internet")) == 0) {
-			*val = COMMUNITY_INTERNET;
-			*token = community_token_no_export;
-			p += strlen("internet");
-			return p;
-		}
 		if (strncmp(p, "graceful-shutdown", strlen("graceful-shutdown"))
 		    == 0) {
 			*val = COMMUNITY_GSHUT;
@@ -916,10 +879,16 @@ void community_init(void)
 			    "BGP Community Hash");
 }
 
+static void community_hash_free(void *data)
+{
+	struct community *com = data;
+
+	community_free(&com);
+}
+
 void community_finish(void)
 {
-	hash_free(comhash);
-	comhash = NULL;
+	hash_clean_and_free(&comhash, community_hash_free);
 }
 
 static struct community *bgp_aggr_community_lookup(
@@ -929,7 +898,7 @@ static struct community *bgp_aggr_community_lookup(
 	return hash_lookup(aggregate->community_hash, community);
 }
 
-static void *bgp_aggr_communty_hash_alloc(void *p)
+static void *bgp_aggr_community_hash_alloc(void *p)
 {
 	struct community *ref = (struct community *)p;
 	struct community *community = NULL;
@@ -986,7 +955,7 @@ void bgp_compute_aggregate_community_hash(struct bgp_aggregate *aggregate,
 		/* Insert community into hash.
 		 */
 		aggr_community = hash_get(aggregate->community_hash, community,
-					  bgp_aggr_communty_hash_alloc);
+					  bgp_aggr_community_hash_alloc);
 	}
 
 	/* Increment reference counter.
