@@ -1,21 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * BGP-specific error messages.
  * Copyright (C) 2018 Cumulus Networks, Inc.
  *               Don Slice
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef __BGP_ERRORS_H__
@@ -34,6 +21,7 @@ enum bgp_log_refs {
 	EC_BGP_ATTR_PMSI_TYPE,
 	EC_BGP_ATTR_PMSI_LEN,
 	EC_BGP_ATTR_NH_SEND_LEN,
+	EC_BGP_ATTR_AIGP,
 	EC_BGP_PEER_GROUP,
 	EC_BGP_PEER_DELETE,
 	EC_BGP_TABLE_CHUNK,
