@@ -94,10 +94,8 @@ int bgp_peer_reg_with_nht(struct peer *peer)
 		connected = 1;
 
 	return bgp_find_or_add_nexthop(peer->bgp, peer->bgp,
-				       family2afi(
-					       peer->connection->su.sa.sa_family),
-				       SAFI_UNICAST, NULL, peer, connected,
-				       NULL);
+				       family2afi(peer->connection->su.sa.sa_family), SAFI_UNICAST,
+				       NULL, peer, connected, NULL, NULL);
 }
 
 static void peer_xfer_stats(struct peer *peer_dst, struct peer *peer_src)
@@ -173,18 +171,22 @@ static struct peer *peer_xfer_conn(struct peer *from_peer)
 	 */
 	bgp_keepalives_off(keeper);
 
-	EVENT_OFF(going_away->t_routeadv);
-	EVENT_OFF(going_away->t_connect);
-	EVENT_OFF(going_away->t_delayopen);
-	EVENT_OFF(going_away->t_connect_check_r);
-	EVENT_OFF(going_away->t_connect_check_w);
-	EVENT_OFF(going_away->t_stop_with_notify);
-	EVENT_OFF(keeper->t_routeadv);
-	EVENT_OFF(keeper->t_connect);
-	EVENT_OFF(keeper->t_delayopen);
-	EVENT_OFF(keeper->t_connect_check_r);
-	EVENT_OFF(keeper->t_connect_check_w);
-	EVENT_OFF(keeper->t_process_packet);
+	event_cancel(&going_away->t_routeadv);
+	event_cancel(&going_away->t_connect);
+	event_cancel(&going_away->t_delayopen);
+	event_cancel(&going_away->t_connect_check_r);
+	event_cancel(&going_away->t_connect_check_w);
+	event_cancel(&going_away->t_stop_with_notify);
+	event_cancel(&keeper->t_routeadv);
+	event_cancel(&keeper->t_connect);
+	event_cancel(&keeper->t_delayopen);
+	event_cancel(&keeper->t_connect_check_r);
+	event_cancel(&keeper->t_connect_check_w);
+
+	frr_with_mutex (&bm->peer_connection_mtx) {
+		if (peer_connection_fifo_member(&bm->connection_fifo, keeper))
+			peer_connection_fifo_del(&bm->connection_fifo, keeper);
+	}
 
 	/*
 	 * At this point in time, it is possible that there are packets pending
@@ -305,8 +307,13 @@ static struct peer *peer_xfer_conn(struct peer *from_peer)
 
 	bgp_reads_on(keeper);
 	bgp_writes_on(keeper);
-	event_add_event(bm->master, bgp_process_packet, keeper, 0,
-			&keeper->t_process_packet);
+
+	frr_with_mutex (&bm->peer_connection_mtx) {
+		if (!peer_connection_fifo_member(&bm->connection_fifo, keeper)) {
+			peer_connection_fifo_add_tail(&bm->connection_fifo, keeper);
+		}
+	}
+	event_add_event(bm->master, bgp_process_packet, NULL, 0, &bm->e_process_packet);
 
 	return (peer);
 }
@@ -325,25 +332,25 @@ void bgp_timer_set(struct peer_connection *connection)
 		/* First entry point of peer's finite state machine.  In Idle
 		   status start timer is on unless peer is shutdown or peer is
 		   inactive.  All other timer must be turned off */
-		if (BGP_PEER_START_SUPPRESSED(peer) || !peer_active(connection) ||
+		if (BGP_PEER_START_SUPPRESSED(peer) || peer_active(connection) != BGP_PEER_ACTIVE ||
 		    peer->bgp->vrf_id == VRF_UNKNOWN) {
-			EVENT_OFF(connection->t_start);
+			event_cancel(&connection->t_start);
 		} else {
 			BGP_TIMER_ON(connection->t_start, bgp_start_timer,
 				     peer->v_start);
 		}
-		EVENT_OFF(connection->t_connect);
-		EVENT_OFF(connection->t_holdtime);
+		event_cancel(&connection->t_connect);
+		event_cancel(&connection->t_holdtime);
 		bgp_keepalives_off(connection);
-		EVENT_OFF(connection->t_routeadv);
-		EVENT_OFF(connection->t_delayopen);
+		event_cancel(&connection->t_routeadv);
+		event_cancel(&connection->t_delayopen);
 		break;
 
 	case Connect:
 		/* After start timer is expired, the peer moves to Connect
 		   status.  Make sure start timer is off and connect timer is
 		   on. */
-		EVENT_OFF(connection->t_start);
+		event_cancel(&connection->t_start);
 		if (CHECK_FLAG(peer->flags, PEER_FLAG_TIMER_DELAYOPEN))
 			BGP_TIMER_ON(connection->t_connect, bgp_connect_timer,
 				     (peer->v_delayopen + peer->v_connect));
@@ -351,19 +358,19 @@ void bgp_timer_set(struct peer_connection *connection)
 			BGP_TIMER_ON(connection->t_connect, bgp_connect_timer,
 				     peer->v_connect);
 
-		EVENT_OFF(connection->t_holdtime);
+		event_cancel(&connection->t_holdtime);
 		bgp_keepalives_off(connection);
-		EVENT_OFF(connection->t_routeadv);
+		event_cancel(&connection->t_routeadv);
 		break;
 
 	case Active:
 		/* Active is waiting connection from remote peer.  And if
 		   connect timer is expired, change status to Connect. */
-		EVENT_OFF(connection->t_start);
+		event_cancel(&connection->t_start);
 		/* If peer is passive mode, do not set connect timer. */
 		if (CHECK_FLAG(peer->flags, PEER_FLAG_PASSIVE)
 		    || CHECK_FLAG(peer->sflags, PEER_STATUS_NSF_WAIT)) {
-			EVENT_OFF(connection->t_connect);
+			event_cancel(&connection->t_connect);
 		} else {
 			if (CHECK_FLAG(peer->flags, PEER_FLAG_TIMER_DELAYOPEN))
 				BGP_TIMER_ON(connection->t_connect,
@@ -374,30 +381,30 @@ void bgp_timer_set(struct peer_connection *connection)
 				BGP_TIMER_ON(connection->t_connect,
 					     bgp_connect_timer, peer->v_connect);
 		}
-		EVENT_OFF(connection->t_holdtime);
+		event_cancel(&connection->t_holdtime);
 		bgp_keepalives_off(connection);
-		EVENT_OFF(connection->t_routeadv);
+		event_cancel(&connection->t_routeadv);
 		break;
 
 	case OpenSent:
 		/* OpenSent status. */
-		EVENT_OFF(connection->t_start);
-		EVENT_OFF(connection->t_connect);
+		event_cancel(&connection->t_start);
+		event_cancel(&connection->t_connect);
 		if (peer->v_holdtime != 0) {
 			BGP_TIMER_ON(connection->t_holdtime, bgp_holdtime_timer,
 				     peer->v_holdtime);
 		} else {
-			EVENT_OFF(connection->t_holdtime);
+			event_cancel(&connection->t_holdtime);
 		}
 		bgp_keepalives_off(connection);
-		EVENT_OFF(connection->t_routeadv);
-		EVENT_OFF(connection->t_delayopen);
+		event_cancel(&connection->t_routeadv);
+		event_cancel(&connection->t_delayopen);
 		break;
 
 	case OpenConfirm:
 		/* OpenConfirm status. */
-		EVENT_OFF(connection->t_start);
-		EVENT_OFF(connection->t_connect);
+		event_cancel(&connection->t_start);
+		event_cancel(&connection->t_connect);
 
 		/*
 		 * If the negotiated Hold Time value is zero, then the Hold Time
@@ -405,7 +412,7 @@ void bgp_timer_set(struct peer_connection *connection)
 		 * Additionally if a different hold timer has been negotiated
 		 * than we must stop then start the timer again
 		 */
-		EVENT_OFF(connection->t_holdtime);
+		event_cancel(&connection->t_holdtime);
 		if (peer->v_holdtime == 0)
 			bgp_keepalives_off(connection);
 		else {
@@ -413,16 +420,16 @@ void bgp_timer_set(struct peer_connection *connection)
 				     peer->v_holdtime);
 			bgp_keepalives_on(connection);
 		}
-		EVENT_OFF(connection->t_routeadv);
-		EVENT_OFF(connection->t_delayopen);
+		event_cancel(&connection->t_routeadv);
+		event_cancel(&connection->t_delayopen);
 		break;
 
 	case Established:
 		/* In Established status start and connect timer is turned
 		   off. */
-		EVENT_OFF(connection->t_start);
-		EVENT_OFF(connection->t_connect);
-		EVENT_OFF(connection->t_delayopen);
+		event_cancel(&connection->t_start);
+		event_cancel(&connection->t_connect);
+		event_cancel(&connection->t_delayopen);
 
 		/*
 		 * Same as OpenConfirm, if holdtime is zero then both holdtime
@@ -430,7 +437,7 @@ void bgp_timer_set(struct peer_connection *connection)
 		 * Additionally if a different hold timer has been negotiated
 		 * then we must stop then start the timer again
 		 */
-		EVENT_OFF(connection->t_holdtime);
+		event_cancel(&connection->t_holdtime);
 		if (peer->v_holdtime == 0)
 			bgp_keepalives_off(connection);
 		else {
@@ -440,22 +447,22 @@ void bgp_timer_set(struct peer_connection *connection)
 		}
 		break;
 	case Deleted:
-		EVENT_OFF(peer->connection->t_gr_restart);
-		EVENT_OFF(peer->connection->t_gr_stale);
+		event_cancel(&peer->connection->t_gr_restart);
+		event_cancel(&peer->connection->t_gr_stale);
 
 		FOREACH_AFI_SAFI (afi, safi)
-			EVENT_OFF(peer->t_llgr_stale[afi][safi]);
+			event_cancel(&peer->t_llgr_stale[afi][safi]);
 
-		EVENT_OFF(peer->connection->t_pmax_restart);
-		EVENT_OFF(peer->t_refresh_stalepath);
+		event_cancel(&peer->connection->t_pmax_restart);
+		event_cancel(&peer->t_refresh_stalepath);
 		fallthrough;
 	case Clearing:
-		EVENT_OFF(connection->t_start);
-		EVENT_OFF(connection->t_connect);
-		EVENT_OFF(connection->t_holdtime);
+		event_cancel(&connection->t_start);
+		event_cancel(&connection->t_connect);
+		event_cancel(&connection->t_holdtime);
 		bgp_keepalives_off(connection);
-		EVENT_OFF(connection->t_routeadv);
-		EVENT_OFF(connection->t_delayopen);
+		event_cancel(&connection->t_routeadv);
+		event_cancel(&connection->t_delayopen);
 		break;
 	case BGP_STATUS_MAX:
 		flog_err(EC_LIB_DEVELOPMENT,
@@ -472,7 +479,8 @@ static void bgp_start_timer(struct event *thread)
 	struct peer *peer = connection->peer;
 
 	if (bgp_debug_neighbor_events(peer))
-		zlog_debug("%s [FSM] Timer (start timer expire).", peer->host);
+		zlog_debug("%s [FSM] Timer (start timer expire for %s).", peer->host,
+			   bgp_peer_get_connection_direction(connection));
 
 	EVENT_VAL(thread) = BGP_Start;
 	bgp_event(thread); /* bgp_event unlocks peer */
@@ -485,14 +493,14 @@ static void bgp_connect_timer(struct event *thread)
 	struct peer *peer = connection->peer;
 
 	/* stop the DelayOpenTimer if it is running */
-	EVENT_OFF(connection->t_delayopen);
+	event_cancel(&connection->t_delayopen);
 
 	assert(!connection->t_write);
 	assert(!connection->t_read);
 
 	if (bgp_debug_neighbor_events(peer))
-		zlog_debug("%s [FSM] Timer (connect timer (%us) expire)", peer->host,
-			   peer->v_connect);
+		zlog_debug("%s [FSM] Timer (connect timer (%us) expire for %s)", peer->host,
+			   peer->v_connect, bgp_peer_get_connection_direction(connection));
 
 	if (CHECK_FLAG(peer->sflags, PEER_STATUS_ACCEPT_PEER))
 		bgp_stop(connection);
@@ -512,8 +520,8 @@ static void bgp_holdtime_timer(struct event *thread)
 	struct peer *peer = connection->peer;
 
 	if (bgp_debug_neighbor_events(peer))
-		zlog_debug("%s [FSM] Timer (holdtime timer expire)",
-			   peer->host);
+		zlog_debug("%s [FSM] Timer (holdtime timer expire for %s)", peer->host,
+			   bgp_peer_get_connection_direction(connection));
 
 	/*
 	 * Given that we do not have any expectation of ordering
@@ -528,9 +536,11 @@ static void bgp_holdtime_timer(struct event *thread)
 	frr_with_mutex (&connection->io_mtx) {
 		inq_count = atomic_load_explicit(&connection->ibuf->count, memory_order_relaxed);
 	}
-	if (inq_count)
+	if (inq_count) {
 		BGP_TIMER_ON(connection->t_holdtime, bgp_holdtime_timer,
 			     peer->v_holdtime);
+		return;
+	}
 
 	EVENT_VAL(thread) = Hold_Timer_expired;
 	bgp_event(thread); /* bgp_event unlocks peer */
@@ -542,7 +552,8 @@ void bgp_routeadv_timer(struct event *thread)
 	struct peer *peer = connection->peer;
 
 	if (bgp_debug_neighbor_events(peer))
-		zlog_debug("%s [FSM] Timer (routeadv timer expire)", peer->host);
+		zlog_debug("%s [FSM] Timer (routeadv timer expire for %s)", peer->host,
+			   bgp_peer_get_connection_direction(connection));
 
 	peer->synctime = monotime(NULL);
 
@@ -561,8 +572,8 @@ void bgp_delayopen_timer(struct event *thread)
 	struct peer *peer = connection->peer;
 
 	if (bgp_debug_neighbor_events(peer))
-		zlog_debug("%s [FSM] Timer (DelayOpentimer expire)",
-			   peer->host);
+		zlog_debug("%s [FSM] Timer (DelayOpentimer expire for %s)", peer->host,
+			   bgp_peer_get_connection_direction(connection));
 
 	EVENT_VAL(thread) = DelayOpen_timer_expired;
 	bgp_event(thread); /* bgp_event unlocks peer */
@@ -623,13 +634,13 @@ static void bgp_graceful_restart_timer_off(struct peer_connection *connection,
 			return;
 
 	UNSET_FLAG(peer->sflags, PEER_STATUS_NSF_WAIT);
-	EVENT_OFF(connection->t_gr_stale);
+	event_cancel(&connection->t_gr_stale);
 
 	if (peer_dynamic_neighbor(peer) &&
 	    !(CHECK_FLAG(peer->flags, PEER_FLAG_DELETE))) {
 		if (bgp_debug_neighbor_events(peer))
-			zlog_debug("%s (dynamic neighbor) deleted (%s)",
-				   peer->host, __func__);
+			zlog_debug("%s (dynamic neighbor) deleted (%s) for %s", __func__,
+				   peer->host, bgp_peer_get_connection_direction(connection));
 		peer_delete(peer);
 	}
 
@@ -654,8 +665,9 @@ static void bgp_llgr_stale_timer_expire(struct event *thread)
 	 * stale routes from the neighbor that it is retaining.
 	 */
 	if (bgp_debug_neighbor_events(peer))
-		zlog_debug("%pBP Long-lived stale timer (%s) expired", peer,
-			   get_afi_safi_str(afi, safi, false));
+		zlog_debug("%pBP Long-lived stale timer (%s) expired for %s", peer,
+			   get_afi_safi_str(afi, safi, false),
+			   bgp_peer_get_connection_direction(peer->connection));
 
 	UNSET_FLAG(peer->af_sflags[afi][safi], PEER_STATUS_LLGR_WAIT);
 
@@ -753,11 +765,9 @@ static void bgp_graceful_restart_timer_expire(struct event *thread)
 	afi_t afi;
 	safi_t safi;
 
-	if (bgp_debug_neighbor_events(peer)) {
-		zlog_debug("%pBP graceful restart timer expired", peer);
-		zlog_debug("%pBP graceful restart stalepath timer stopped",
-			   peer);
-	}
+	if (bgp_debug_neighbor_events(peer))
+		zlog_debug("%pBP graceful restart timer expired and graceful restart stalepath timer stopped for %s",
+			   peer, bgp_peer_get_connection_direction(connection));
 
 	FOREACH_AFI_SAFI (afi, safi) {
 		if (!peer->nsf[afi][safi])
@@ -781,11 +791,10 @@ static void bgp_graceful_restart_timer_expire(struct event *thread)
 				continue;
 
 			if (bgp_debug_neighbor_events(peer))
-				zlog_debug(
-					"%pBP Long-lived stale timer (%s) started for %d sec",
-					peer,
-					get_afi_safi_str(afi, safi, false),
-					peer->llgr[afi][safi].stale_time);
+				zlog_debug("%pBP Long-lived stale timer (%s) started for %d sec for %s",
+					   peer, get_afi_safi_str(afi, safi, false),
+					   peer->llgr[afi][safi].stale_time,
+					   bgp_peer_get_connection_direction(connection));
 
 			SET_FLAG(peer->af_sflags[afi][safi],
 				 PEER_STATUS_LLGR_WAIT);
@@ -816,8 +825,8 @@ static void bgp_graceful_stale_timer_expire(struct event *thread)
 	safi_t safi;
 
 	if (bgp_debug_neighbor_events(peer))
-		zlog_debug("%pBP graceful restart stalepath timer expired",
-			   peer);
+		zlog_debug("%pBP graceful restart stalepath timer expired for %s", peer,
+			   bgp_peer_get_connection_direction(connection));
 
 	/* NSF delete stale route */
 	FOREACH_AFI_SAFI_NSF (afi, safi)
@@ -880,8 +889,8 @@ bool bgp_update_delay_configured(struct bgp *bgp)
    on ending the update delay. */
 void bgp_update_delay_end(struct bgp *bgp)
 {
-	EVENT_OFF(bgp->t_update_delay);
-	EVENT_OFF(bgp->t_establish_wait);
+	event_cancel(&bgp->t_update_delay);
+	event_cancel(&bgp->t_establish_wait);
 
 	/* Reset update-delay related state */
 	bgp->update_delay_over = 1;
@@ -947,7 +956,7 @@ void bgp_start_routeadv(struct bgp *bgp)
 		if (!peer_established(connection))
 			continue;
 
-		EVENT_OFF(connection->t_routeadv);
+		event_cancel(&connection->t_routeadv);
 		BGP_TIMER_ON(connection->t_routeadv, bgp_routeadv_timer, 0);
 	}
 }
@@ -968,7 +977,7 @@ void bgp_adjust_routeadv(struct peer *peer)
 		 * different
 		 * duration and schedule write thread immediately.
 		 */
-		EVENT_OFF(connection->t_routeadv);
+		event_cancel(&connection->t_routeadv);
 
 		peer->synctime = monotime(NULL);
 		/* If suppress fib pending is enabled, route is advertised to
@@ -1000,7 +1009,7 @@ void bgp_adjust_routeadv(struct peer *peer)
 	 */
 	diff = difftime(nowtime, peer->last_update);
 	if (diff > (double)peer->v_routeadv) {
-		EVENT_OFF(connection->t_routeadv);
+		event_cancel(&connection->t_routeadv);
 		BGP_TIMER_ON(connection->t_routeadv, bgp_routeadv_timer, 0);
 		return;
 	}
@@ -1027,7 +1036,7 @@ void bgp_adjust_routeadv(struct peer *peer)
 		remain = peer->v_routeadv;
 	diff = peer->v_routeadv - diff;
 	if (diff <= (double)remain) {
-		EVENT_OFF(connection->t_routeadv);
+		event_cancel(&connection->t_routeadv);
 		BGP_TIMER_ON(connection->t_routeadv, bgp_routeadv_timer, diff);
 	}
 }
@@ -1107,7 +1116,7 @@ static void bgp_maxmed_onstartup_timer(struct event *thread)
 	zlog_info("Max med on startup ended - timer expired.");
 
 	bgp = EVENT_ARG(thread);
-	EVENT_OFF(bgp->t_maxmed_onstartup);
+	event_cancel(&bgp->t_maxmed_onstartup);
 	bgp->maxmed_onstartup_over = 1;
 
 	bgp_maxmed_update(bgp);
@@ -1149,7 +1158,7 @@ static void bgp_update_delay_timer(struct event *thread)
 	zlog_info("Update delay ended - timer expired.");
 
 	bgp = EVENT_ARG(thread);
-	EVENT_OFF(bgp->t_update_delay);
+	event_cancel(&bgp->t_update_delay);
 	bgp_update_delay_end(bgp);
 }
 
@@ -1161,7 +1170,7 @@ static void bgp_establish_wait_timer(struct event *thread)
 	zlog_info("Establish wait - timer expired.");
 
 	bgp = EVENT_ARG(thread);
-	EVENT_OFF(bgp->t_establish_wait);
+	event_cancel(&bgp->t_establish_wait);
 	bgp_check_update_delay(bgp);
 }
 
@@ -1242,10 +1251,10 @@ void bgp_fsm_change_status(struct peer_connection *connection,
 	if (bgp_debug_neighbor_events(peer)) {
 		struct vrf *vrf = vrf_lookup_by_id(bgp->vrf_id);
 
-		zlog_debug("%s : vrf %s(%u), Status: %s established_peers %u", __func__,
+		zlog_debug("%s : vrf %s(%u), Status: %s established_peers %u for %s", __func__,
 			   vrf ? vrf->name : "Unknown", bgp->vrf_id,
-			   lookup_msg(bgp_status_msg, status, NULL),
-			   bgp->established_peers);
+			   lookup_msg(bgp_status_msg, status, NULL), bgp->established_peers,
+			   bgp_peer_get_connection_direction(connection));
 	}
 
 	/* Set to router ID to the value provided by RIB if there are no peers
@@ -1258,7 +1267,7 @@ void bgp_fsm_change_status(struct peer_connection *connection,
 	/* Transition into Clearing or Deleted must /always/ clear all routes..
 	 * (and must do so before actually changing into Deleted..
 	 */
-	if (status >= Clearing && (peer->established || peer == bgp->peer_self)) {
+	if (status >= Clearing && (peer->established || peer != bgp->peer_self)) {
 		bgp_clear_route_all(peer);
 
 		/* If no route was queued for the clear-node processing,
@@ -1281,7 +1290,8 @@ void bgp_fsm_change_status(struct peer_connection *connection,
 		 * Clearing
 		 * (or Deleted).
 		 */
-		if (!work_queue_is_scheduled(peer->clear_node_queue) &&
+		if (!CHECK_FLAG(peer->flags, PEER_FLAG_CLEARING_BATCH) &&
+		    !work_queue_is_scheduled(peer->clear_node_queue) &&
 		    status != Deleted)
 			BGP_EVENT_ADD(connection, Clearing_Completed);
 	}
@@ -1322,10 +1332,10 @@ void bgp_fsm_change_status(struct peer_connection *connection,
 		bgp_update_delay_process_status_change(peer);
 
 	if (bgp_debug_neighbor_events(peer))
-		zlog_debug("%s fd %d went from %s to %s", peer->host,
-			   connection->fd,
+		zlog_debug("%s fd %d went from %s to %s for %s", peer->host, connection->fd,
 			   lookup_msg(bgp_status_msg, connection->ostatus, NULL),
-			   lookup_msg(bgp_status_msg, connection->status, NULL));
+			   lookup_msg(bgp_status_msg, connection->status, NULL),
+			   bgp_peer_get_connection_direction(connection));
 }
 
 /* Flush the event queue and ensure the peer is shut down */
@@ -1357,8 +1367,8 @@ enum bgp_fsm_state_progress bgp_stop(struct peer_connection *connection)
 	if (peer_dynamic_neighbor_no_nsf(peer) &&
 	    !(CHECK_FLAG(peer->flags, PEER_FLAG_DELETE))) {
 		if (bgp_debug_neighbor_events(peer))
-			zlog_debug("%s (dynamic neighbor) deleted (%s)",
-				   peer->host, __func__);
+			zlog_debug("%s (dynamic neighbor) deleted (%s) for %s", __func__,
+				   peer->host, bgp_peer_get_connection_direction(connection));
 		peer_delete(peer);
 		return BGP_FSM_FAILURE_AND_DELETE;
 	}
@@ -1377,6 +1387,9 @@ enum bgp_fsm_state_progress bgp_stop(struct peer_connection *connection)
 					 peer->last_reset == PEER_DOWN_MULTIHOP_CHANGE))
 			bfd_sess_uninstall(peer->bfd_config->session);
 
+		if (peer->bfd_config)
+			event_cancel(&peer->bfd_config->t_hold_timer);
+
 		/* Notify BGP conditional advertisement process */
 		peer->advmap_table_change = true;
 
@@ -1397,20 +1410,19 @@ enum bgp_fsm_state_progress bgp_stop(struct peer_connection *connection)
 
 		/* graceful restart */
 		if (connection->t_gr_stale) {
-			EVENT_OFF(connection->t_gr_stale);
+			event_cancel(&connection->t_gr_stale);
 			if (bgp_debug_neighbor_events(peer))
-				zlog_debug(
-					"%pBP graceful restart stalepath timer stopped",
-					peer);
+				zlog_debug("%pBP graceful restart stalepath timer stopped for %s",
+					   peer, bgp_peer_get_connection_direction(connection));
 		}
 		if (CHECK_FLAG(peer->sflags, PEER_STATUS_NSF_WAIT)) {
 			if (bgp_debug_neighbor_events(peer)) {
-				zlog_debug(
-					"%pBP graceful restart timer started for %d sec",
-					peer, peer->v_gr_restart);
-				zlog_debug(
-					"%pBP graceful restart stalepath timer started for %d sec",
-					peer, peer->bgp->stalepath_time);
+				zlog_debug("%pBP graceful restart timer started for %d sec for %s",
+					   peer, peer->v_gr_restart,
+					   bgp_peer_get_connection_direction(connection));
+				zlog_debug("%pBP graceful restart stalepath timer started for %d sec for %s",
+					   peer, peer->bgp->stalepath_time,
+					   bgp_peer_get_connection_direction(connection));
 			}
 			BGP_TIMER_ON(connection->t_gr_restart,
 				     bgp_graceful_restart_timer_expire,
@@ -1427,12 +1439,11 @@ enum bgp_fsm_state_progress bgp_stop(struct peer_connection *connection)
 
 		/* Stop route-refresh stalepath timer */
 		if (peer->t_refresh_stalepath) {
-			EVENT_OFF(peer->t_refresh_stalepath);
+			event_cancel(&peer->t_refresh_stalepath);
 
 			if (bgp_debug_neighbor_events(peer))
-				zlog_debug(
-					"%pBP route-refresh restart stalepath timer stopped",
-					peer);
+				zlog_debug("%pBP route-refresh restart stalepath timer stopped for %s",
+					   peer, bgp_peer_get_connection_direction(connection));
 		}
 
 		/* If peer reset before receiving EOR, decrement EOR count and
@@ -1454,9 +1465,9 @@ enum bgp_fsm_state_progress bgp_stop(struct peer_connection *connection)
 					gr_info->eor_required--;
 
 				if (BGP_DEBUG(update, UPDATE_OUT))
-					zlog_debug("peer %s, EOR_required %d",
-						   peer->host,
-						   gr_info->eor_required);
+					zlog_debug("peer %s, EOR_required %d for %s", peer->host,
+						   gr_info->eor_required,
+						   bgp_peer_get_connection_direction(connection));
 
 				/* There is no pending EOR message */
 				if (gr_info->eor_required == 0) {
@@ -1465,7 +1476,7 @@ enum bgp_fsm_state_progress bgp_stop(struct peer_connection *connection)
 							gr_info->t_select_deferral);
 						XFREE(MTYPE_TMP, info);
 					}
-					EVENT_OFF(gr_info->t_select_deferral);
+					event_cancel(&gr_info->t_select_deferral);
 					gr_info->eor_received = 0;
 				}
 			}
@@ -1475,8 +1486,8 @@ enum bgp_fsm_state_progress bgp_stop(struct peer_connection *connection)
 		peer->resettime = peer->uptime = monotime(NULL);
 
 		if (BGP_DEBUG(update_groups, UPDATE_GROUPS))
-			zlog_debug("%s remove from all update group",
-				   peer->host);
+			zlog_debug("%s remove from all update group for %s", peer->host,
+				   bgp_peer_get_connection_direction(connection));
 		update_group_remove_peer_afs(peer);
 
 		/* Reset peer synctime */
@@ -1490,17 +1501,17 @@ enum bgp_fsm_state_progress bgp_stop(struct peer_connection *connection)
 	bgp_writes_off(connection);
 	bgp_reads_off(connection);
 
-	EVENT_OFF(connection->t_connect_check_r);
-	EVENT_OFF(connection->t_connect_check_w);
+	event_cancel(&connection->t_connect_check_r);
+	event_cancel(&connection->t_connect_check_w);
 
-	EVENT_OFF(connection->t_stop_with_notify);
+	event_cancel(&connection->t_stop_with_notify);
 
 	/* Stop all timers. */
-	EVENT_OFF(connection->t_start);
-	EVENT_OFF(connection->t_connect);
-	EVENT_OFF(connection->t_holdtime);
-	EVENT_OFF(connection->t_routeadv);
-	EVENT_OFF(connection->t_delayopen);
+	event_cancel(&connection->t_start);
+	event_cancel(&connection->t_connect);
+	event_cancel(&connection->t_holdtime);
+	event_cancel(&connection->t_routeadv);
+	event_cancel(&connection->t_delayopen);
 
 	/* Clear input and output buffer.  */
 	frr_with_mutex (&connection->io_mtx) {
@@ -1522,6 +1533,7 @@ enum bgp_fsm_state_progress bgp_stop(struct peer_connection *connection)
 	if (connection->fd >= 0) {
 		close(connection->fd);
 		connection->fd = -1;
+		connection->dir = UNKNOWN;
 	}
 
 	/* Reset capabilities. */
@@ -1596,8 +1608,8 @@ bgp_stop_with_error(struct peer_connection *connection)
 
 	if (peer_dynamic_neighbor_no_nsf(peer)) {
 		if (bgp_debug_neighbor_events(peer))
-			zlog_debug("%s (dynamic neighbor) deleted (%s)",
-				   peer->host, __func__);
+			zlog_debug("%s (dynamic neighbor) deleted (%s) for %s", __func__,
+				   peer->host, bgp_peer_get_connection_direction(connection));
 		peer_delete(peer);
 		return BGP_FSM_FAILURE;
 	}
@@ -1618,8 +1630,8 @@ bgp_stop_with_notify(struct peer_connection *connection, uint8_t code,
 
 	if (peer_dynamic_neighbor_no_nsf(peer)) {
 		if (bgp_debug_neighbor_events(peer))
-			zlog_debug("%s (dynamic neighbor) deleted (%s)",
-				   peer->host, __func__);
+			zlog_debug("%s (dynamic neighbor) deleted (%s) for %s", __func__,
+				   peer->host, bgp_peer_get_connection_direction(connection));
 		peer_delete(peer);
 		return BGP_FSM_FAILURE;
 	}
@@ -1658,8 +1670,8 @@ static void bgp_connect_check(struct event *thread)
 	assert(!connection->t_read);
 	assert(!connection->t_write);
 
-	EVENT_OFF(connection->t_connect_check_r);
-	EVENT_OFF(connection->t_connect_check_w);
+	event_cancel(&connection->t_connect_check_r);
+	event_cancel(&connection->t_connect_check_w);
 
 	/* Check file descriptor. */
 	slen = sizeof(status);
@@ -1684,8 +1696,9 @@ static void bgp_connect_check(struct event *thread)
 		return;
 	} else {
 		if (bgp_debug_neighbor_events(peer))
-			zlog_debug("%s [Event] Connect failed %d(%s)",
-				   peer->host, status, safe_strerror(status));
+			zlog_debug("%s [Event] Connect failed %d(%s) for connection %s", peer->host,
+				   status, safe_strerror(status),
+				   bgp_peer_get_connection_direction(connection));
 		BGP_EVENT_ADD(connection, TCP_connection_open_failed);
 		return;
 	}
@@ -1724,10 +1737,12 @@ bgp_connect_success(struct peer_connection *connection)
 
 	if (bgp_debug_neighbor_events(peer)) {
 		if (!CHECK_FLAG(peer->sflags, PEER_STATUS_ACCEPT_PEER))
-			zlog_debug("%s open active, local address %pSU", peer->host,
-				   connection->su_local);
+			zlog_debug("%s open active, local address %pSU for %s", peer->host,
+				   connection->su_local,
+				   bgp_peer_get_connection_direction(connection));
 		else
-			zlog_debug("%s passive open", peer->host);
+			zlog_debug("%s passive open for %s", peer->host,
+				   bgp_peer_get_connection_direction(connection));
 	}
 
 	/* Send an open message */
@@ -1770,10 +1785,12 @@ bgp_connect_success_w_delayopen(struct peer_connection *connection)
 
 	if (bgp_debug_neighbor_events(peer)) {
 		if (!CHECK_FLAG(peer->sflags, PEER_STATUS_ACCEPT_PEER))
-			zlog_debug("%s open active, local address %pSU", peer->host,
-				   connection->su_local);
+			zlog_debug("%s open active, local address %pSU for %s", peer->host,
+				   connection->su_local,
+				   bgp_peer_get_connection_direction(connection));
 		else
-			zlog_debug("%s passive open", peer->host);
+			zlog_debug("%s passive open for %s", peer->host,
+				   bgp_peer_get_connection_direction(connection));
 	}
 
 	/* set the DelayOpenTime to the inital value */
@@ -1785,8 +1802,9 @@ bgp_connect_success_w_delayopen(struct peer_connection *connection)
 			     peer->v_delayopen);
 
 	if (bgp_debug_neighbor_events(peer))
-		zlog_debug("%s [FSM] BGP OPEN message delayed for %d seconds",
-			   peer->host, peer->delayopen);
+		zlog_debug("%s [FSM] BGP OPEN message delayed for %d seconds for connection %s",
+			   peer->host, peer->delayopen,
+			   bgp_peer_get_connection_direction(connection));
 
 	return BGP_FSM_SUCCESS;
 }
@@ -1799,8 +1817,8 @@ bgp_connect_fail(struct peer_connection *connection)
 
 	if (peer_dynamic_neighbor_no_nsf(peer)) {
 		if (bgp_debug_neighbor_events(peer))
-			zlog_debug("%s (dynamic neighbor) deleted (%s)",
-				   peer->host, __func__);
+			zlog_debug("%s (dynamic neighbor) deleted (%s) for %s", __func__,
+				   peer->host, bgp_peer_get_connection_direction(connection));
 		peer_delete(peer);
 		return BGP_FSM_FAILURE_AND_DELETE;
 	}
@@ -1843,9 +1861,8 @@ static enum bgp_fsm_state_progress bgp_start(struct peer_connection *connection)
 
 	if (connection->su.sa.sa_family == AF_UNSPEC) {
 		if (bgp_debug_neighbor_events(peer))
-			zlog_debug(
-				"%s [FSM] Unable to get neighbor's IP address, waiting...",
-				peer->host);
+			zlog_debug("%s [FSM] Unable to get neighbor's IP address, waiting... for %s",
+				   peer->host, bgp_peer_get_connection_direction(connection));
 		peer->last_reset = PEER_DOWN_NBR_ADDR;
 		return BGP_FSM_FAILURE;
 	}
@@ -1888,9 +1905,9 @@ static enum bgp_fsm_state_progress bgp_start(struct peer_connection *connection)
 	if (!bgp_peer_reg_with_nht(peer)) {
 		if (bgp_zebra_num_connects()) {
 			if (bgp_debug_neighbor_events(peer))
-				zlog_debug(
-					"%s [FSM] Waiting for NHT, no path to neighbor present",
-					peer->host);
+				zlog_debug("%s [FSM] Waiting for NHT, no path to neighbor present for %s",
+					   peer->host,
+					   bgp_peer_get_connection_direction(connection));
 			peer->last_reset = PEER_DOWN_WAITING_NHT;
 			BGP_EVENT_ADD(connection, TCP_connection_open_failed);
 			return BGP_FSM_SUCCESS;
@@ -1906,13 +1923,14 @@ static enum bgp_fsm_state_progress bgp_start(struct peer_connection *connection)
 	switch (status) {
 	case connect_error:
 		if (bgp_debug_neighbor_events(peer))
-			zlog_debug("%s [FSM] Connect error", peer->host);
+			zlog_debug("%s [FSM] Connect error for %s", peer->host,
+				   bgp_peer_get_connection_direction(connection));
 		BGP_EVENT_ADD(connection, TCP_connection_open_failed);
 		break;
 	case connect_success:
 		if (bgp_debug_neighbor_events(peer))
-			zlog_debug("%s [FSM] Connect immediately success, fd %d",
-				   peer->host, connection->fd);
+			zlog_debug("%s [FSM] Connect immediately success, fd %d for %s", peer->host,
+				   connection->fd, bgp_peer_get_connection_direction(connection));
 
 		BGP_EVENT_ADD(connection, TCP_connection_open);
 		break;
@@ -1920,8 +1938,9 @@ static enum bgp_fsm_state_progress bgp_start(struct peer_connection *connection)
 		/* To check nonblocking connect, we wait until socket is
 		   readable or writable. */
 		if (bgp_debug_neighbor_events(peer))
-			zlog_debug("%s [FSM] Non blocking connect waiting result, fd %d",
-				   peer->host, connection->fd);
+			zlog_debug("%s [FSM] Non blocking connect waiting result, fd %d for %s",
+				   peer->host, connection->fd,
+				   bgp_peer_get_connection_direction(connection));
 		if (connection->fd < 0) {
 			flog_err(EC_BGP_FSM, "%s peer's fd is negative value %d",
 				 __func__, peer->connection->fd);
@@ -1968,14 +1987,12 @@ bgp_reconnect(struct peer_connection *connection)
 static enum bgp_fsm_state_progress
 bgp_fsm_open(struct peer_connection *connection)
 {
-	struct peer *peer = connection->peer;
-
 	/* If DelayOpen is active, we may still need to send an open message */
 	if ((connection->status == Connect) || (connection->status == Active))
 		bgp_open_send(connection);
 
 	/* Send keepalive and make keepalive timer */
-	bgp_keepalive_send(peer);
+	bgp_keepalive_send(connection);
 
 	return BGP_FSM_SUCCESS;
 }
@@ -2003,7 +2020,8 @@ bgp_fsm_holdtime_expire(struct peer_connection *connection)
 	struct peer *peer = connection->peer;
 
 	if (bgp_debug_neighbor_events(peer))
-		zlog_debug("%s [FSM] Hold timer expire", peer->host);
+		zlog_debug("%s [FSM] Hold timer expire for %s", peer->host,
+			   bgp_peer_get_connection_direction(connection));
 
 	/* RFC8538 updates RFC 4724 by defining an extension that permits
 	 * the Graceful Restart procedures to be performed when the BGP
@@ -2022,7 +2040,7 @@ static enum bgp_fsm_state_progress
 bgp_fsm_delayopen_timer_expire(struct peer_connection *connection)
 {
 	/* Stop the DelayOpenTimer */
-	EVENT_OFF(connection->t_delayopen);
+	event_cancel(&connection->t_delayopen);
 
 	/* Send open message to peer */
 	bgp_open_send(connection);
@@ -2180,13 +2198,19 @@ bgp_establish(struct peer_connection *connection)
 	/* assign update-group/subgroup */
 	update_group_adjust_peer_afs(peer);
 
+	/* Stop BFD Strict mode Hold-Timer */
+	if (peer->bfd_config)
+		event_cancel(&peer->bfd_config->t_hold_timer);
+
 	/* graceful restart */
 	UNSET_FLAG(peer->sflags, PEER_STATUS_NSF_WAIT);
 	if (bgp_debug_neighbor_events(peer)) {
 		if (BGP_PEER_RESTARTING_MODE(peer))
-			zlog_debug("%pBP BGP_RESTARTING_MODE", peer);
+			zlog_debug("%pBP BGP_RESTARTING_MODE %s", peer,
+				   bgp_peer_get_connection_direction(connection));
 		else if (BGP_PEER_HELPER_MODE(peer))
-			zlog_debug("%pBP BGP_HELPER_MODE", peer);
+			zlog_debug("%pBP BGP_HELPER_MODE %s", peer,
+				   bgp_peer_get_connection_direction(connection));
 	}
 
 	FOREACH_AFI_SAFI_NSF (afi, safi) {
@@ -2257,18 +2281,18 @@ bgp_establish(struct peer_connection *connection)
 	else {
 		UNSET_FLAG(peer->sflags, PEER_STATUS_NSF_MODE);
 		if (connection->t_gr_stale) {
-			EVENT_OFF(connection->t_gr_stale);
+			event_cancel(&connection->t_gr_stale);
 			if (bgp_debug_neighbor_events(peer))
-				zlog_debug(
-					"%pBP graceful restart stalepath timer stopped",
-					peer);
+				zlog_debug("%pBP graceful restart stalepath timer stopped for %s",
+					   peer, bgp_peer_get_connection_direction(connection));
 		}
 	}
 
 	if (connection->t_gr_restart) {
-		EVENT_OFF(connection->t_gr_restart);
+		event_cancel(&connection->t_gr_restart);
 		if (bgp_debug_neighbor_events(peer))
-			zlog_debug("%pBP graceful restart timer stopped", peer);
+			zlog_debug("%pBP graceful restart timer stopped for %s", peer,
+				   bgp_peer_get_connection_direction(connection));
 	}
 
 	/* Reset uptime, turn on keepalives, send current table. */
@@ -2282,11 +2306,11 @@ bgp_establish(struct peer_connection *connection)
 	 */
 	FOREACH_AFI_SAFI (afi, safi) {
 		if (peer->t_llgr_stale[afi][safi]) {
-			EVENT_OFF(peer->t_llgr_stale[afi][safi]);
+			event_cancel(&peer->t_llgr_stale[afi][safi]);
 			if (bgp_debug_neighbor_events(peer))
-				zlog_debug(
-					"%pBP Long-lived stale timer stopped for afi/safi: %d/%d",
-					peer, afi, safi);
+				zlog_debug("%pBP Long-lived stale timer stopped for afi/safi: %d/%d for %s",
+					   peer, afi, safi,
+					   bgp_peer_get_connection_direction(connection));
 		}
 
 		if (CHECK_FLAG(peer->af_cap[afi][safi],
@@ -2319,7 +2343,7 @@ bgp_establish(struct peer_connection *connection)
 	 * of read-only mode.
 	 */
 	if (!bgp_update_delay_active(peer->bgp)) {
-		EVENT_OFF(peer->connection->t_routeadv);
+		event_cancel(&peer->connection->t_routeadv);
 		BGP_TIMER_ON(peer->connection->t_routeadv, bgp_routeadv_timer,
 			     0);
 	}
@@ -2327,9 +2351,8 @@ bgp_establish(struct peer_connection *connection)
 	if (peer->doppelganger &&
 	    (peer->doppelganger->connection->status != Deleted)) {
 		if (bgp_debug_neighbor_events(peer))
-			zlog_debug(
-				"[Event] Deleting stub connection for peer %s",
-				peer->host);
+			zlog_debug("[Event] Deleting stub connection for peer %s for %s", peer->host,
+				   bgp_peer_get_connection_direction(peer->doppelganger->connection));
 
 		if (peer->doppelganger->connection->status > Active)
 			bgp_notify_send(peer->doppelganger->connection,
@@ -2358,7 +2381,7 @@ bgp_establish(struct peer_connection *connection)
 static enum bgp_fsm_state_progress
 bgp_fsm_keepalive(struct peer_connection *connection)
 {
-	EVENT_OFF(connection->t_holdtime);
+	event_cancel(&connection->t_holdtime);
 	return BGP_FSM_SUCCESS;
 }
 
@@ -2366,7 +2389,7 @@ bgp_fsm_keepalive(struct peer_connection *connection)
 static enum bgp_fsm_state_progress
 bgp_fsm_update(struct peer_connection *connection)
 {
-	EVENT_OFF(connection->t_holdtime);
+	event_cancel(&connection->t_holdtime);
 	return BGP_FSM_SUCCESS;
 }
 
@@ -2412,13 +2435,13 @@ void bgp_fsm_nht_update(struct peer_connection *connection, struct peer *peer,
 		break;
 	case Connect:
 		if (!has_valid_nexthops) {
-			EVENT_OFF(connection->t_connect);
+			event_cancel(&connection->t_connect);
 			BGP_EVENT_ADD(connection, TCP_fatal_error);
 		}
 		break;
 	case Active:
 		if (has_valid_nexthops) {
-			EVENT_OFF(connection->t_connect);
+			event_cancel(&connection->t_connect);
 			BGP_EVENT_ADD(connection, ConnectRetry_timer_expired);
 		}
 		break;
@@ -2636,11 +2659,10 @@ int bgp_event_update(struct peer_connection *connection,
 	next = FSM[connection->status - 1][event - 1].next_state;
 
 	if (bgp_debug_neighbor_events(peer) && connection->status != next)
-		zlog_debug("%s [FSM] %s (%s->%s), fd %d", peer->host,
-			   bgp_event_str[event],
+		zlog_debug("%s [FSM] %s (%s->%s), fd %d for %s", peer->host, bgp_event_str[event],
 			   lookup_msg(bgp_status_msg, connection->status, NULL),
-			   lookup_msg(bgp_status_msg, next, NULL),
-			   connection->fd);
+			   lookup_msg(bgp_status_msg, next, NULL), connection->fd,
+			   bgp_peer_get_connection_direction(connection));
 
 	peer->last_event = peer->cur_event;
 	peer->cur_event = event;
