@@ -529,7 +529,10 @@ static struct rtr_mgr_group *get_groups(struct list *cache_list)
 
 inline bool is_synchronized(struct rpki_vrf *rpki_vrf)
 {
-	return rpki_vrf->rtr_is_synced;
+	if (is_running(rpki_vrf))
+		return rpki_vrf->rtr_is_synced;
+	else
+		return false;
 }
 
 inline bool is_running(struct rpki_vrf *rpki_vrf)
@@ -921,7 +924,7 @@ static void stop(struct rpki_vrf *rpki_vrf)
 {
 	rpki_vrf->rtr_is_stopping = true;
 	if (is_running(rpki_vrf)) {
-		EVENT_OFF(rpki_vrf->t_rpki_sync);
+		event_cancel(&rpki_vrf->t_rpki_sync);
 		rtr_mgr_stop(rpki_vrf->rtr_config);
 		rtr_mgr_free(rpki_vrf->rtr_config);
 		rpki_vrf->rtr_is_running = false;
