@@ -1,23 +1,9 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * EIGRP daemon northbound implementation.
  *
  * Copyright (C) 2019 Network Device Education Foundation, Inc. ("NetDEF")
  *                    Rafael Zalamena
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA.
  */
 
 #include <zebra.h>
@@ -28,6 +14,7 @@
 #include "lib/table.h"
 #include "lib/vrf.h"
 #include "lib/zclient.h"
+#include "lib/distribute.h"
 
 #include "eigrp_structs.h"
 #include "eigrpd.h"
@@ -42,27 +29,25 @@ static void redistribute_get_metrics(const struct lyd_node *dnode,
 {
 	memset(em, 0, sizeof(*em));
 
-	if (yang_dnode_exists(dnode, "./bandwidth"))
-		em->bandwidth = yang_dnode_get_uint32(dnode, "./bandwidth");
-	if (yang_dnode_exists(dnode, "./delay"))
-		em->delay = yang_dnode_get_uint32(dnode, "./delay");
+	if (yang_dnode_exists(dnode, "bandwidth"))
+		em->bandwidth = yang_dnode_get_uint32(dnode, "bandwidth");
+	if (yang_dnode_exists(dnode, "delay"))
+		em->delay = yang_dnode_get_uint32(dnode, "delay");
 #if 0 /* TODO: How does MTU work? */
-	if (yang_dnode_exists(dnode, "./mtu"))
-		em->mtu[0] = yang_dnode_get_uint32(dnode, "./mtu");
+	if (yang_dnode_exists(dnode, "mtu"))
+		em->mtu[0] = yang_dnode_get_uint32(dnode, "mtu");
 #endif
-	if (yang_dnode_exists(dnode, "./load"))
-		em->load = yang_dnode_get_uint32(dnode, "./load");
-	if (yang_dnode_exists(dnode, "./reliability"))
-		em->reliability = yang_dnode_get_uint32(dnode, "./reliability");
+	if (yang_dnode_exists(dnode, "load"))
+		em->load = yang_dnode_get_uint32(dnode, "load");
+	if (yang_dnode_exists(dnode, "reliability"))
+		em->reliability = yang_dnode_get_uint32(dnode, "reliability");
 }
 
-static struct eigrp_interface *eigrp_interface_lookup(const struct eigrp *eigrp,
-						      const char *ifname)
+static struct eigrp_interface *eigrp_interface_lookup(struct eigrp *eigrp, const char *ifname)
 {
 	struct eigrp_interface *eif;
-	struct listnode *ln;
 
-	for (ALL_LIST_ELEMENTS_RO(eigrp->eiflist, ln, eif)) {
+	frr_each (eigrp_interface_hash, &eigrp->eifs, eif) {
 		if (strcmp(ifname, eif->ifp->name))
 			continue;
 
@@ -87,7 +72,7 @@ static int eigrpd_instance_create(struct nb_cb_create_args *args)
 		/* NOTHING */
 		break;
 	case NB_EV_PREPARE:
-		vrf = yang_dnode_get_string(args->dnode, "./vrf");
+		vrf = yang_dnode_get_string(args->dnode, "vrf");
 
 		pVrf = vrf_lookup_by_name(vrf);
 		if (pVrf)
@@ -95,7 +80,7 @@ static int eigrpd_instance_create(struct nb_cb_create_args *args)
 		else
 			vrfid = VRF_DEFAULT;
 
-		eigrp = eigrp_get(yang_dnode_get_uint16(args->dnode, "./asn"),
+		eigrp = eigrp_get(yang_dnode_get_uint16(args->dnode, "asn"),
 				  vrfid);
 		args->resource->ptr = eigrp;
 		break;
@@ -715,6 +700,22 @@ static int eigrpd_instance_neighbor_destroy(struct nb_cb_destroy_args *args)
 	return NB_OK;
 }
 
+/*
+ * XPath: /frr-eigrpd:eigrpd/instance/distribute-list
+ */
+static int eigrpd_instance_distribute_list_create(struct nb_cb_create_args *args)
+{
+	struct eigrp *eigrp;
+
+	if (args->event != NB_EV_APPLY)
+		return NB_OK;
+
+	eigrp = nb_running_get_entry(args->dnode, NULL, true);
+	group_distribute_list_create_helper(args, eigrp->distribute_ctx);
+
+	return NB_OK;
+}
+
 /*
  * XPath: /frr-eigrpd:eigrpd/instance/redistribute
  */
@@ -729,7 +730,7 @@ static int eigrpd_instance_redistribute_create(struct nb_cb_create_args *args)
 
 	switch (args->event) {
 	case NB_EV_VALIDATE:
-		proto = yang_dnode_get_enum(args->dnode, "./protocol");
+		proto = yang_dnode_get_enum(args->dnode, "protocol");
 		vrfname = yang_dnode_get_string(args->dnode, "../vrf");
 
 		pVrf = vrf_lookup_by_name(vrfname);
@@ -738,7 +739,7 @@ static int eigrpd_instance_redistribute_create(struct nb_cb_create_args *args)
 		else
 			vrfid = VRF_DEFAULT;
 
-		if (vrf_bitmap_check(zclient->redist[AFI_IP][proto], vrfid))
+		if (vrf_bitmap_check(&eigrp_zclient->redist[AFI_IP][proto], vrfid))
 			return NB_ERR_INCONSISTENCY;
 		break;
 	case NB_EV_PREPARE:
@@ -747,7 +748,7 @@ static int eigrpd_instance_redistribute_create(struct nb_cb_create_args *args)
 		break;
 	case NB_EV_APPLY:
 		eigrp = nb_running_get_entry(args->dnode, NULL, true);
-		proto = yang_dnode_get_enum(args->dnode, "./protocol");
+		proto = yang_dnode_get_enum(args->dnode, "protocol");
 		redistribute_get_metrics(args->dnode, &metrics);
 		eigrp_redistribute_set(eigrp, proto, metrics);
 		break;
@@ -769,7 +770,7 @@ static int eigrpd_instance_redistribute_destroy(struct nb_cb_destroy_args *args)
 		break;
 	case NB_EV_APPLY:
 		eigrp = nb_running_get_entry(args->dnode, NULL, true);
-		proto = yang_dnode_get_enum(args->dnode, "./protocol");
+		proto = yang_dnode_get_enum(args->dnode, "protocol");
 		eigrp_redistribute_unset(eigrp, proto);
 		break;
 	}
@@ -1134,7 +1135,7 @@ static int lib_interface_eigrp_instance_create(struct nb_cb_create_args *args)
 			break;
 		}
 
-		eigrp = eigrp_get(yang_dnode_get_uint16(args->dnode, "./asn"),
+		eigrp = eigrp_get(yang_dnode_get_uint16(args->dnode, "asn"),
 				  ifp->vrf->vrf_id);
 		eif = eigrp_interface_lookup(eigrp, ifp->name);
 		if (eif == NULL)
@@ -1146,7 +1147,7 @@ static int lib_interface_eigrp_instance_create(struct nb_cb_create_args *args)
 		break;
 	case NB_EV_APPLY:
 		ifp = nb_running_get_entry(args->dnode, NULL, true);
-		eigrp = eigrp_get(yang_dnode_get_uint16(args->dnode, "./asn"),
+		eigrp = eigrp_get(yang_dnode_get_uint16(args->dnode, "asn"),
 				  ifp->vrf->vrf_id);
 		eif = eigrp_interface_lookup(eigrp, ifp->name);
 		if (eif == NULL)
@@ -1416,6 +1417,45 @@ const struct frr_yang_module_info frr_eigrpd_info = {
 				.cli_show = eigrp_cli_show_neighbor,
 			}
 		},
+		{
+			.xpath = "/frr-eigrpd:eigrpd/instance/distribute-list",
+			.cbs = {
+				.create = eigrpd_instance_distribute_list_create,
+				.destroy = group_distribute_list_destroy,
+			}
+		},
+		{
+			.xpath = "/frr-eigrpd:eigrpd/instance/distribute-list/in/access-list",
+			.cbs = {
+				.modify = group_distribute_list_ipv4_modify,
+				.destroy = group_distribute_list_ipv4_destroy,
+				.cli_show = group_distribute_list_ipv4_cli_show,
+			}
+		},
+		{
+			.xpath = "/frr-eigrpd:eigrpd/instance/distribute-list/out/access-list",
+			.cbs = {
+				.modify = group_distribute_list_ipv4_modify,
+				.destroy = group_distribute_list_ipv4_destroy,
+				.cli_show = group_distribute_list_ipv4_cli_show,
+			}
+		},
+		{
+			.xpath = "/frr-eigrpd:eigrpd/instance/distribute-list/in/prefix-list",
+			.cbs = {
+				.modify = group_distribute_list_ipv4_modify,
+				.destroy = group_distribute_list_ipv4_destroy,
+				.cli_show = group_distribute_list_ipv4_cli_show,
+			}
+		},
+		{
+			.xpath = "/frr-eigrpd:eigrpd/instance/distribute-list/out/prefix-list",
+			.cbs = {
+				.modify = group_distribute_list_ipv4_modify,
+				.destroy = group_distribute_list_ipv4_destroy,
+				.cli_show = group_distribute_list_ipv4_cli_show,
+			}
+		},
 		{
 			.xpath = "/frr-eigrpd:eigrpd/instance/redistribute",
 			.cbs = {
