@@ -128,8 +128,7 @@ static void eigrp_update_receive_GR_ask(struct eigrp *eigrp,
 
 	/* iterate over all prefixes which weren't advertised by neighbor */
 	for (ALL_LIST_ELEMENTS_RO(nbr_prefixes, node1, prefix)) {
-		zlog_debug("GR receive: Neighbor not advertised %pFX",
-			   prefix->destination);
+		zlog_debug("GR receive: Neighbor not advertised %pFX", &prefix->destination);
 
 		fsm_msg.metrics = prefix->reported_metric;
 		/* set delay to MAX */
@@ -320,9 +319,7 @@ void eigrp_update_receive(struct eigrp *eigrp, struct ip *iph,
 				/*Here comes topology information save*/
 				pe = eigrp_prefix_descriptor_new();
 				pe->serno = eigrp->serno;
-				pe->destination =
-					(struct prefix *)prefix_ipv4_new();
-				prefix_copy(pe->destination, &dest_addr);
+				prefix_copy(&pe->destination, &dest_addr);
 				pe->af = AF_INET;
 				pe->state = EIGRP_FSM_STATE_PASSIVE;
 				pe->nt = EIGRP_TOPOLOGY_TYPE_REMOTE;
@@ -482,11 +479,10 @@ static void eigrp_update_place_on_nbr_queue(struct eigrp_neighbor *nbr,
 static void eigrp_update_send_to_all_nbrs(struct eigrp_interface *ei,
 					  struct eigrp_packet *ep)
 {
-	struct listnode *node, *nnode;
 	struct eigrp_neighbor *nbr;
 	bool packet_sent = false;
 
-	for (ALL_LIST_ELEMENTS(ei->nbrs, node, nnode, nbr)) {
+	frr_each (eigrp_nbr_hash, &ei->nbr_hash_head, nbr) {
 		struct eigrp_packet *ep_dup;
 
 		if (nbr->state != EIGRP_NEIGHBOR_UP)
@@ -567,7 +563,7 @@ void eigrp_update_send_EOT(struct eigrp_neighbor *nbr)
 				}
 			}
 			/* Get destination address from prefix */
-			dest_addr = pe->destination;
+			dest_addr = &pe->destination;
 
 			/* Check if any list fits */
 			if (eigrp_update_prefix_apply(
@@ -595,7 +591,7 @@ void eigrp_update_send(struct eigrp_interface *ei)
 	uint32_t seq_no = eigrp->sequence_number;
 	uint16_t eigrp_mtu = EIGRP_PACKET_MTU(ei->ifp->mtu);
 
-	if (ei->nbrs->count == 0)
+	if (eigrp_nbr_hash_count(&ei->nbr_hash_head) == 0)
 		return;
 
 	uint16_t length = EIGRP_HEADER_LEN;
@@ -651,7 +647,7 @@ void eigrp_update_send(struct eigrp_interface *ei)
 			has_tlv = 0;
 		}
 		/* Get destination address from prefix */
-		dest_addr = pe->destination;
+		dest_addr = &pe->destination;
 
 		if (eigrp_update_prefix_apply(eigrp, ei, EIGRP_FILTER_OUT,
 					      dest_addr)) {
@@ -694,10 +690,10 @@ void eigrp_update_send_all(struct eigrp *eigrp,
 			   struct eigrp_interface *exception)
 {
 	struct eigrp_interface *iface;
-	struct listnode *node, *node2, *nnode2;
+	struct listnode *node2, *nnode2;
 	struct eigrp_prefix_descriptor *pe;
 
-	for (ALL_LIST_ELEMENTS_RO(eigrp->eiflist, node, iface)) {
+	frr_each (eigrp_interface_hash, &eigrp->eifs, iface) {
 		if (iface != exception) {
 			eigrp_update_send(iface);
 		}
@@ -799,7 +795,7 @@ static void eigrp_update_send_GR_part(struct eigrp_neighbor *nbr)
 		/*
 		 * Filtering
 		 */
-		dest_addr = pe->destination;
+		dest_addr = &pe->destination;
 
 		if (eigrp_update_prefix_apply(eigrp, ei, EIGRP_FILTER_OUT,
 					      dest_addr)) {
@@ -1001,11 +997,10 @@ void eigrp_update_send_GR(struct eigrp_neighbor *nbr, enum GR_type gr_type,
 void eigrp_update_send_interface_GR(struct eigrp_interface *ei,
 				    enum GR_type gr_type, struct vty *vty)
 {
-	struct listnode *node;
 	struct eigrp_neighbor *nbr;
 
 	/* iterate over all neighbors on eigrp interface */
-	for (ALL_LIST_ELEMENTS_RO(ei->nbrs, node, nbr)) {
+	frr_each (eigrp_nbr_hash, &ei->nbr_hash_head, nbr) {
 		/* send GR to neighbor */
 		eigrp_update_send_GR(nbr, gr_type, vty);
 	}
@@ -1027,11 +1022,10 @@ void eigrp_update_send_interface_GR(struct eigrp_interface *ei,
 void eigrp_update_send_process_GR(struct eigrp *eigrp, enum GR_type gr_type,
 				  struct vty *vty)
 {
-	struct listnode *node;
 	struct eigrp_interface *ei;
 
 	/* iterate over all eigrp interfaces */
-	for (ALL_LIST_ELEMENTS_RO(eigrp->eiflist, node, ei)) {
+	frr_each (eigrp_interface_hash, &eigrp->eifs, ei) {
 		/* send GR to all neighbors on interface */
 		eigrp_update_send_interface_GR(ei, gr_type, vty);
 	}
