@@ -185,9 +185,6 @@ enum {
 	RTM_GETNEXTHOPBUCKET,
 #define RTM_GETNEXTHOPBUCKET	RTM_GETNEXTHOPBUCKET
 
-        RTM_SETHWFLAGS = 119,
-#define RTM_SETHWFLAGS RTM_SETHWFLAGS
-
 	RTM_NEWTUNNEL = 120,
 #define RTM_NEWTUNNEL	RTM_NEWTUNNEL
 	RTM_DELTUNNEL,
