@@ -1,23 +1,10 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * IS-IS Rout(e)ing protocol - OpenFabric extensions
  *
  * Copyright (C) 2018 Christian Franke
  *
  * This file is part of FRRouting (FRR)
- *
- * FRR is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the
- * Free Software Foundation; either version 2, or (at your option) any
- * later version.
- *
- * FRR is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #include <zebra.h>
 #include "isisd/fabricd.h"
@@ -53,7 +40,7 @@ struct fabricd {
 	enum fabricd_sync_state initial_sync_state;
 	time_t initial_sync_start;
 	struct isis_circuit *initial_sync_circuit;
-	struct thread *initial_sync_timeout;
+	struct event *initial_sync_timeout;
 
 	struct isis_spftree *spftree;
 	struct skiplist *neighbors;
@@ -62,8 +49,8 @@ struct fabricd {
 	uint8_t tier;
 	uint8_t tier_config;
 	uint8_t tier_pending;
-	struct thread *tier_calculation_timer;
-	struct thread *tier_set_timer;
+	struct event *tier_calculation_timer;
+	struct event *tier_set_timer;
 
 	int csnp_delay;
 	bool always_send_csnp;
@@ -220,10 +207,10 @@ struct fabricd *fabricd_new(struct isis_area *area)
 	rv->area = area;
 	rv->initial_sync_state = FABRICD_SYNC_PENDING;
 
-	rv->spftree =
-		isis_spftree_new(area, &area->lspdb[IS_LEVEL_2 - 1],
-				 area->isis->sysid, ISIS_LEVEL2, SPFTREE_IPV4,
-				 SPF_TYPE_FORWARD, F_SPFTREE_HOPCOUNT_METRIC);
+	rv->spftree = isis_spftree_new(
+		area, &area->lspdb[IS_LEVEL_2 - 1], area->isis->sysid,
+		ISIS_LEVEL2, SPFTREE_IPV4, SPF_TYPE_FORWARD,
+		F_SPFTREE_HOPCOUNT_METRIC, SR_ALGORITHM_SPF);
 	rv->neighbors = skiplist_new(0, neighbor_entry_list_cmp,
 				     neighbor_entry_del_void);
 	rv->neighbors_neighbors = hash_create(neighbor_entry_hash_key,
@@ -238,11 +225,11 @@ struct fabricd *fabricd_new(struct isis_area *area)
 
 void fabricd_finish(struct fabricd *f)
 {
-	THREAD_OFF(f->initial_sync_timeout);
+	event_cancel(&f->initial_sync_timeout);
 
-	THREAD_OFF(f->tier_calculation_timer);
+	event_cancel(&f->tier_calculation_timer);
 
-	THREAD_OFF(f->tier_set_timer);
+	event_cancel(&f->tier_set_timer);
 
 	isis_spftree_del(f->spftree);
 	neighbor_lists_clear(f);
@@ -250,9 +237,9 @@ void fabricd_finish(struct fabricd *f)
 	hash_free(f->neighbors_neighbors);
 }
 
-static void fabricd_initial_sync_timeout(struct thread *thread)
+static void fabricd_initial_sync_timeout(struct event *thread)
 {
-	struct fabricd *f = THREAD_ARG(thread);
+	struct fabricd *f = EVENT_ARG(thread);
 
 	if (IS_DEBUG_ADJ_PACKETS)
 		zlog_debug(
@@ -280,14 +267,14 @@ void fabricd_initial_sync_hello(struct isis_circuit *circuit)
 	if (f->initial_sync_timeout)
 		return;
 
-	thread_add_timer(master, fabricd_initial_sync_timeout, f,
-			 timeout, &f->initial_sync_timeout);
+	event_add_timer(master, fabricd_initial_sync_timeout, f, timeout,
+			&f->initial_sync_timeout);
 	f->initial_sync_start = monotime(NULL);
 
 	if (IS_DEBUG_ADJ_PACKETS)
 		zlog_debug(
-			"OpenFabric: Started initial synchronization with %s on %s",
-			sysid_print(circuit->u.p2p.neighbor->sysid),
+			"OpenFabric: Started initial synchronization with %pSY on %s",
+			circuit->u.p2p.neighbor->sysid,
 			circuit->interface->name);
 }
 
@@ -338,7 +325,7 @@ void fabricd_initial_sync_finish(struct isis_area *area)
 		  f->initial_sync_circuit->interface->name);
 	f->initial_sync_state = FABRICD_SYNC_COMPLETE;
 	f->initial_sync_circuit = NULL;
-	THREAD_OFF(f->initial_sync_timeout);
+	event_cancel(&f->initial_sync_timeout);
 }
 
 static void fabricd_bump_tier_calculation_timer(struct fabricd *f);
@@ -372,7 +359,9 @@ static uint8_t fabricd_calculate_fabric_tier(struct isis_area *area)
 		return ISIS_TIER_UNDEFINED;
 	}
 
-	zlog_info("OpenFabric: Found %s as furthest t0 from local system, dist == %u", rawlspid_print(furthest_t0->N.id), furthest_t0->d_N);
+	zlog_info(
+		"OpenFabric: Found %pLS as furthest t0 from local system, dist == %u",
+		furthest_t0->N.id, furthest_t0->d_N);
 
 	struct isis_spftree *remote_tree =
 		isis_run_hopcount_spf(area, furthest_t0->N.id, NULL);
@@ -385,8 +374,9 @@ static uint8_t fabricd_calculate_fabric_tier(struct isis_area *area)
 		isis_spftree_del(remote_tree);
 		return ISIS_TIER_UNDEFINED;
 	} else {
-		zlog_info("OpenFabric: Found %s as furthest from remote dist == %u", rawlspid_print(furthest_from_remote->N.id),
-			  furthest_from_remote->d_N);
+		zlog_info(
+			"OpenFabric: Found %pLS as furthest from remote dist == %u",
+			furthest_from_remote->N.id, furthest_from_remote->d_N);
 	}
 
 	int64_t tier = furthest_from_remote->d_N - furthest_t0->d_N;
@@ -402,16 +392,16 @@ static uint8_t fabricd_calculate_fabric_tier(struct isis_area *area)
 	return tier;
 }
 
-static void fabricd_tier_set_timer(struct thread *thread)
+static void fabricd_tier_set_timer(struct event *thread)
 {
-	struct fabricd *f = THREAD_ARG(thread);
+	struct fabricd *f = EVENT_ARG(thread);
 
 	fabricd_set_tier(f, f->tier_pending);
 }
 
-static void fabricd_tier_calculation_cb(struct thread *thread)
+static void fabricd_tier_calculation_cb(struct event *thread)
 {
-	struct fabricd *f = THREAD_ARG(thread);
+	struct fabricd *f = EVENT_ARG(thread);
 	uint8_t tier = ISIS_TIER_UNDEFINED;
 
 	tier = fabricd_calculate_fabric_tier(f->area);
@@ -421,28 +411,27 @@ static void fabricd_tier_calculation_cb(struct thread *thread)
 	zlog_info("OpenFabric: Got tier %hhu from algorithm. Arming timer.",
 		  tier);
 	f->tier_pending = tier;
-	thread_add_timer(master, fabricd_tier_set_timer, f,
-			 f->area->lsp_gen_interval[ISIS_LEVEL2 - 1],
-			 &f->tier_set_timer);
-
+	event_add_timer(master, fabricd_tier_set_timer, f,
+			f->area->lsp_gen_interval[ISIS_LEVEL2 - 1],
+			&f->tier_set_timer);
 }
 
 static void fabricd_bump_tier_calculation_timer(struct fabricd *f)
 {
 	/* Cancel timer if we already know our tier */
 	if (f->tier != ISIS_TIER_UNDEFINED || f->tier_set_timer) {
-		THREAD_OFF(f->tier_calculation_timer);
+		event_cancel(&f->tier_calculation_timer);
 		return;
 	}
 
 	/* If we need to calculate the tier, wait some
 	 * time for the topology to settle before running
 	 * the calculation */
-	THREAD_OFF(f->tier_calculation_timer);
+	event_cancel(&f->tier_calculation_timer);
 
-	thread_add_timer(master, fabricd_tier_calculation_cb, f,
-			 2 * f->area->lsp_gen_interval[ISIS_LEVEL2 - 1],
-			 &f->tier_calculation_timer);
+	event_add_timer(master, fabricd_tier_calculation_cb, f,
+			2 * f->area->lsp_gen_interval[ISIS_LEVEL2 - 1],
+			&f->tier_calculation_timer);
 }
 
 static void fabricd_set_tier(struct fabricd *f, uint8_t tier)
@@ -723,10 +712,10 @@ void fabricd_trigger_csnp(struct isis_area *area, bool circuit_scoped)
 		if (!circuit->t_send_csnp[1])
 			continue;
 
-		THREAD_OFF(circuit->t_send_csnp[ISIS_LEVEL2 - 1]);
-		thread_add_timer_msec(master, send_l2_csnp, circuit,
-				      isis_jitter(f->csnp_delay, CSNP_JITTER),
-				      &circuit->t_send_csnp[ISIS_LEVEL2 - 1]);
+		event_cancel(&circuit->t_send_csnp[ISIS_LEVEL2 - 1]);
+		event_add_timer_msec(master, send_l2_csnp, circuit,
+				     isis_jitter(f->csnp_delay, CSNP_JITTER),
+				     &circuit->t_send_csnp[ISIS_LEVEL2 - 1]);
 	}
 }
 
