@@ -560,12 +560,6 @@ const struct frr_yang_module_info frr_isisd_info = {
 				.destroy = isis_instance_fast_reroute_level_1_lfa_tiebreaker_destroy,
 			}
 		},
-		{
-			.xpath = "/frr-isisd:isis/instance/fast-reroute/level-1/lfa/tiebreaker/type",
-			.cbs = {
-				.modify = isis_instance_fast_reroute_level_1_lfa_tiebreaker_type_modify,
-			}
-		},
 		{
 			.xpath = "/frr-isisd:isis/instance/fast-reroute/level-1/remote-lfa/prefix-list",
 			.cbs = {
@@ -597,12 +591,6 @@ const struct frr_yang_module_info frr_isisd_info = {
 				.destroy = isis_instance_fast_reroute_level_2_lfa_tiebreaker_destroy,
 			}
 		},
-		{
-			.xpath = "/frr-isisd:isis/instance/fast-reroute/level-2/lfa/tiebreaker/type",
-			.cbs = {
-				.modify = isis_instance_fast_reroute_level_2_lfa_tiebreaker_type_modify,
-			}
-		},
 		{
 			.xpath = "/frr-isisd:isis/instance/fast-reroute/level-2/remote-lfa/prefix-list",
 			.cbs = {
