@@ -1,21 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * IS-IS Rout(e)ing protocol - isis_routemap.c
  *
  * Copyright (C) 2013-2015 Christian Franke <chris@opensourcerouting.org>
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -31,7 +18,7 @@
 #include "plist.h"
 #include "routemap.h"
 #include "table.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "vty.h"
 
 #include "isis_constants.h"
@@ -247,10 +234,51 @@ static const struct route_map_rule_cmd route_set_metric_cmd = {
 	route_set_metric_free
 };
 
+static void isis_route_map_update(const char *name)
+{
+	struct isis *isis;
+	struct listnode *node, *lnode;
+	struct isis_area *area;
+	int type;
+	int level;
+	int protocol;
+	struct isis_redist *redist;
+
+	for (ALL_LIST_ELEMENTS_RO(im->isis, node, isis))
+		for (ALL_LIST_ELEMENTS_RO(isis->area_list, node, area))
+			for (protocol = 0; protocol < REDIST_PROTOCOL_COUNT; protocol++)
+				for (type = 0; type < ZEBRA_ROUTE_MAX + 1; type++)
+					for (level = 0; level < ISIS_LEVELS; level++) {
+						if (area->redist_settings[protocol][type][level] ==
+						    NULL)
+							continue;
+						for (ALL_LIST_ELEMENTS_RO(area->redist_settings
+										  [protocol][type]
+										  [level],
+									  lnode, redist)) {
+							if (redist->redist == 0)
+								continue;
+
+							if (redist->map_name &&
+							    strcmp(redist->map_name, name) == 0) {
+								isis_redist_update(area, level + 1,
+										   afi_skt_for_redist_protocol(
+											   protocol),
+										   type,
+										   redist->table);
+							}
+						}
+					}
+}
+
 void isis_route_map_init(void)
 {
 	route_map_init();
 
+	route_map_add_hook(isis_route_map_update);
+	route_map_delete_hook(isis_route_map_update);
+	route_map_event_hook(isis_route_map_update);
+
 	route_map_match_ip_address_hook(generic_match_add);
 	route_map_no_match_ip_address_hook(generic_match_delete);
 
