@@ -571,6 +571,11 @@ struct event_loop *event_master_create(const char *name)
 	rv->spin = true;
 	rv->handle_signals = true;
 
+	/* tardy event warnings */
+	monotime(&rv->last_tardy_warning);
+	rv->last_tardy_warning.tv_sec -= (TARDY_WARNING_INTERVAL + TIMER_SECOND_MICRO - 1) /
+					 TIMER_SECOND_MICRO;
+
 	/* Set pthread owner, should be updated by actual owner */
 	rv->owner = pthread_self();
 	rv->cancel_req = list_new();
@@ -770,13 +775,13 @@ static struct event *thread_get(struct event_loop *m, uint8_t type,
 	thread->master = m;
 	thread->arg = arg;
 	thread->yield = EVENT_YIELD_TIME_SLOT; /* default */
+	thread->tardy_threshold = 0;
 	/* thread->ref is zeroed either by XCALLOC above or by memset before
 	 * being put on the "unuse" list by thread_add_unuse().
 	 * Setting it here again makes coverity complain about a missing
 	 * lock :(
 	 */
 	/* thread->ref = NULL; */
-	thread->ignore_timer_late = false;
 
 	/*
 	 * So if the passed in funcname is not what we have
@@ -1023,6 +1028,8 @@ static void _event_add_timer_timeval(const struct xref_eventsched *xref,
 			return;
 
 		thread = thread_get(m, EVENT_TIMER, func, arg, xref);
+		/* default lateness warning: 4s */
+		thread->tardy_threshold = TARDY_DEFAULT_THRESHOLD;
 
 		frr_with_mutex (&thread->mtx) {
 			thread->u.sands = t;
@@ -1498,13 +1505,7 @@ void event_cancel_async(struct event_loop *master, struct event **thread,
 {
 	assert(!(thread && eventobj) && (thread || eventobj));
 
-	if (thread && *thread)
-		frrtrace(9, frr_libfrr, event_cancel_async, master,
-			 (*thread)->xref->funcname, (*thread)->xref->xref.file,
-			 (*thread)->xref->xref.line, NULL, (*thread)->u.fd,
-			 (*thread)->u.val, (*thread)->arg,
-			 (*thread)->u.sands.tv_sec);
-	else
+	if (!thread)
 		frrtrace(9, frr_libfrr, event_cancel_async, master, NULL, NULL,
 			 0, NULL, 0, 0, eventobj, 0);
 
@@ -1514,6 +1515,14 @@ void event_cancel_async(struct event_loop *master, struct event **thread,
 		master->canceled = false;
 
 		if (thread) {
+			if (*thread)
+				frrtrace(9, frr_libfrr, event_cancel_async,
+					 master, (*thread)->xref->funcname,
+					 (*thread)->xref->xref.file,
+					 (*thread)->xref->xref.line, NULL,
+					 (*thread)->u.fd, (*thread)->u.val,
+					 (*thread)->arg,
+					 (*thread)->u.sands.tv_sec);
 			struct cancel_req *cr =
 				XCALLOC(MTYPE_TMP, sizeof(struct cancel_req));
 			cr->threadref = thread;
@@ -1685,34 +1694,12 @@ static void thread_process_io(struct event_loop *m, unsigned int num)
 static unsigned int thread_process_timers(struct event_loop *m,
 					  struct timeval *timenow)
 {
-	struct timeval prev = *timenow;
-	bool displayed = false;
 	struct event *thread;
 	unsigned int ready = 0;
 
 	while ((thread = event_timer_list_first(&m->timer))) {
 		if (timercmp(timenow, &thread->u.sands, <))
 			break;
-		prev = thread->u.sands;
-		prev.tv_sec += 4;
-		/*
-		 * If the timer would have popped 4 seconds in the
-		 * past then we are in a situation where we are
-		 * really getting behind on handling of events.
-		 * Let's log it and do the right thing with it.
-		 */
-		if (timercmp(timenow, &prev, >)) {
-			atomic_fetch_add_explicit(
-				&thread->hist->total_starv_warn, 1,
-				memory_order_seq_cst);
-			if (!displayed && !thread->ignore_timer_late) {
-				flog_warn(
-					EC_LIB_STARVE_THREAD,
-					"Thread Starvation: %pTHD was scheduled to pop greater than 4s ago",
-					thread);
-				displayed = true;
-			}
-		}
 
 		event_timer_list_pop(&m->timer);
 		thread->type = EVENT_READY;
@@ -1861,8 +1848,6 @@ struct event *event_fetch(struct event_loop *m, struct event *fetch)
 unsigned long event_consumed_time(RUSAGE_T *now, RUSAGE_T *start,
 				  unsigned long *cputime)
 {
-#ifdef HAVE_CLOCK_THREAD_CPUTIME_ID
-
 #ifdef __FreeBSD__
 	/*
 	 * FreeBSD appears to have an issue when calling clock_gettime
@@ -1883,13 +1868,8 @@ unsigned long event_consumed_time(RUSAGE_T *now, RUSAGE_T *start,
 		now->cpu.tv_nsec = start->cpu.tv_nsec + 1;
 	}
 #endif
-	*cputime = (now->cpu.tv_sec - start->cpu.tv_sec) * TIMER_SECOND_MICRO
-		   + (now->cpu.tv_nsec - start->cpu.tv_nsec) / 1000;
-#else
-	/* This is 'user + sys' time.  */
-	*cputime = timeval_elapsed(now->cpu.ru_utime, start->cpu.ru_utime)
-		   + timeval_elapsed(now->cpu.ru_stime, start->cpu.ru_stime);
-#endif
+	*cputime = (now->cpu.tv_sec - start->cpu.tv_sec) * TIMER_SECOND_MICRO +
+		   (now->cpu.tv_nsec - start->cpu.tv_nsec) / 1000;
 	return timeval_elapsed(now->real, start->real);
 }
 
@@ -1931,19 +1911,33 @@ void event_getrusage(RUSAGE_T *r)
 		return;
 	}
 
-#ifdef HAVE_CLOCK_THREAD_CPUTIME_ID
 	/* not currently implemented in Linux's vDSO, but maybe at some point
 	 * in the future?
 	 */
 	clock_gettime(CLOCK_THREAD_CPUTIME_ID, &r->cpu);
-#else /* !HAVE_CLOCK_THREAD_CPUTIME_ID */
-#if defined RUSAGE_THREAD
-#define FRR_RUSAGE RUSAGE_THREAD
-#else
-#define FRR_RUSAGE RUSAGE_SELF
-#endif
-	getrusage(FRR_RUSAGE, &(r->cpu));
-#endif
+}
+
+static void event_tardy_warn(struct event *thread, unsigned long since_us)
+{
+	char buf[64];
+	struct fbuf fb = { .buf = buf, .pos = buf, .len = sizeof(buf) };
+	double loadavg[3];
+	int rv;
+
+	rv = getloadavg(loadavg, array_size(loadavg));
+	if (rv < 0)
+		bprintfrr(&fb, "not available");
+	else {
+		for (int i = 0; i < rv; i++) {
+			bprintfrr(&fb, "%.2f", loadavg[i]);
+			if (i < rv - 1)
+				bputs(&fb, ", ");
+		}
+	}
+
+	flog_warn(EC_LIB_STARVE_THREAD,
+		  "CPU starvation: %pTHD getting executed %lums late, warning threshold %lums. System load: %pFB",
+		  thread, (since_us + 999) / 1000, (thread->tardy_threshold + 999) / 1000, &fb);
 }
 
 /*
@@ -1962,6 +1956,33 @@ void event_call(struct event *thread)
 	RUSAGE_T before, after;
 	bool suppress_warnings = EVENT_ARG(thread);
 
+	if (thread->tardy_threshold) {
+		int64_t timer_late_us = monotime_since(&thread->u.sands, NULL);
+
+		/* Timers have a tardiness warning defaulting to 4s.
+		 * It can be customized with event_set_tardy_threshold()
+		 * (bfdd does that since the protocol has really short timers)
+		 *
+		 * If we are more than that threshold late, print a warning
+		 * since we're running behind in calling timers (probably due
+		 * to high system load.)
+		 */
+		if (timer_late_us > (int64_t)thread->tardy_threshold) {
+			int64_t since_last_warning;
+			struct timeval *tw;
+
+			atomic_fetch_add_explicit(&thread->hist->total_starv_warn, 1,
+						  memory_order_seq_cst);
+
+			tw = &thread->master->last_tardy_warning;
+			since_last_warning = monotime_since(tw, NULL);
+			if (since_last_warning > TARDY_WARNING_INTERVAL) {
+				event_tardy_warn(thread, timer_late_us);
+				monotime(tw);
+			}
+		}
+	}
+
 	/* if the thread being called is the CLI, it may change cputime_enabled
 	 * ("service cputime-stats" command), which can result in nonsensical
 	 * and very confusing warnings
@@ -2145,9 +2166,9 @@ static ssize_t printfrr_thread_dbg(struct fbuf *buf, struct printfrr_eargs *ea,
 	char info[16] = "";
 
 	if (!thread)
-		return bputs(buf, "{(thread *)NULL}");
+		return bputs(buf, "{(event *)NULL}");
 
-	rv += bprintfrr(buf, "{(thread *)%p arg=%p", thread, thread->arg);
+	rv += bprintfrr(buf, "{(event *)%p arg=%p", thread, thread->arg);
 
 	if (thread->type < array_size(types) && types[thread->type])
 		rv += bprintfrr(buf, " %-6s", types[thread->type]);
