@@ -18,13 +18,6 @@
 
 #define MGMTD_MAX_XPATH_LEN XPATH_MAXLEN
 
-#define MGMTD_MAX_YANG_VALUE_LEN YANG_VALUE_MAXLEN
-
-#define MGMTD_MAX_NUM_XPATH_REG 128
-
-#define MGMTD_MAX_NUM_DATA_REQ_IN_BATCH 32
-#define MGMTD_MAX_NUM_DATA_REPLY_IN_BATCH 8
-
 enum mgmt_result {
 	MGMTD_SUCCESS = 0,
 	MGMTD_INVALID_PARAM,
@@ -35,4 +28,12 @@ enum mgmt_result {
 	MGMTD_UNKNOWN_FAILURE
 };
 
+enum mgmt_ds_id {
+	MGMTD_DS_NONE,
+	MGMTD_DS_RUNNING,
+	MGMTD_DS_CANDIDATE,
+	MGMTD_DS_OPERATIONAL,
+};
+#define MGMTD_DS_MAX_ID (MGMTD_DS_OPERATIONAL + 1)
+
 #endif /* _FRR_MGMTD_DEFINES_H */
