@@ -16,31 +16,31 @@ DEFINE_MTYPE_STATIC(LIB, SRV6_LOCATOR_CHUNK, "SRV6 locator chunk");
 DEFINE_MTYPE_STATIC(LIB, SRV6_SID_FORMAT, "SRv6 SID format");
 DEFINE_MTYPE_STATIC(LIB, SRV6_SID_CTX, "SRv6 SID context");
 
-const char *seg6local_action2str(uint32_t action)
+const char *seg6local_action2str_with_next_csid(uint32_t action, bool has_next_csid)
 {
 	switch (action) {
 	case ZEBRA_SEG6_LOCAL_ACTION_END:
-		return "End";
+		return has_next_csid ? "uN" : "End";
 	case ZEBRA_SEG6_LOCAL_ACTION_END_X:
-		return "End.X";
+		return has_next_csid ? "uA" : "End.X";
 	case ZEBRA_SEG6_LOCAL_ACTION_END_T:
-		return "End.T";
+		return has_next_csid ? "uDT" : "End.T";
 	case ZEBRA_SEG6_LOCAL_ACTION_END_DX2:
-		return "End.DX2";
+		return has_next_csid ? "uDX2" : "End.DX2";
 	case ZEBRA_SEG6_LOCAL_ACTION_END_DX6:
-		return "End.DX6";
+		return has_next_csid ? "uDX6" : "End.DX6";
 	case ZEBRA_SEG6_LOCAL_ACTION_END_DX4:
-		return "End.DX4";
+		return has_next_csid ? "uDX4" : "End.DX4";
 	case ZEBRA_SEG6_LOCAL_ACTION_END_DT6:
-		return "End.DT6";
+		return has_next_csid ? "uDT6" : "End.DT6";
 	case ZEBRA_SEG6_LOCAL_ACTION_END_DT4:
-		return "End.DT4";
+		return has_next_csid ? "uDT4" : "End.DT4";
 	case ZEBRA_SEG6_LOCAL_ACTION_END_B6:
-		return "End.B6";
+		return has_next_csid ? "uB6" : "End.B6";
 	case ZEBRA_SEG6_LOCAL_ACTION_END_B6_ENCAP:
-		return "End.B6.Encap";
+		return has_next_csid ? "uB6.Encap" : "End.B6.Encap";
 	case ZEBRA_SEG6_LOCAL_ACTION_END_BM:
-		return "End.BM";
+		return has_next_csid ? "uBM" : "End.BM";
 	case ZEBRA_SEG6_LOCAL_ACTION_END_S:
 		return "End.S";
 	case ZEBRA_SEG6_LOCAL_ACTION_END_AS:
@@ -48,7 +48,7 @@ const char *seg6local_action2str(uint32_t action)
 	case ZEBRA_SEG6_LOCAL_ACTION_END_AM:
 		return "End.AM";
 	case ZEBRA_SEG6_LOCAL_ACTION_END_DT46:
-		return "End.DT46";
+		return has_next_csid ? "uDT46" : "End.DT46";
 	case ZEBRA_SEG6_LOCAL_ACTION_UNSPEC:
 		return "unspec";
 	default:
@@ -56,6 +56,11 @@ const char *seg6local_action2str(uint32_t action)
 	}
 }
 
+const char *seg6local_action2str(uint32_t action)
+{
+	return seg6local_action2str_with_next_csid(action, false);
+}
+
 int snprintf_seg6_segs(char *str,
 		size_t size, const struct seg6_segs *segs)
 {
@@ -71,6 +76,21 @@ int snprintf_seg6_segs(char *str,
 	return strlen(str);
 }
 
+static void seg6local_flavors2json(json_object *json, const struct seg6local_flavors_info *flv_info)
+{
+	json_object *json_flavors;
+
+	json_flavors = json_object_new_array();
+	json_object_object_add(json, "flavors", json_flavors);
+
+	if (CHECK_SRV6_FLV_OP(flv_info->flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_PSP))
+		json_array_string_add(json_flavors, "psp");
+	if (CHECK_SRV6_FLV_OP(flv_info->flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_USP))
+		json_array_string_add(json_flavors, "usp");
+	if (CHECK_SRV6_FLV_OP(flv_info->flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_USD))
+		json_array_string_add(json_flavors, "usd");
+}
+
 void srv6_sid_structure2json(const struct seg6local_context *ctx, json_object *json)
 {
 	json_object_int_add(json, "blockLen", ctx->block_len);
@@ -82,6 +102,7 @@ void srv6_sid_structure2json(const struct seg6local_context *ctx, json_object *j
 void seg6local_context2json(const struct seg6local_context *ctx,
 			    uint32_t action, json_object *json)
 {
+	seg6local_flavors2json(json, &ctx->flv);
 	switch (action) {
 	case ZEBRA_SEG6_LOCAL_ACTION_END:
 		return;
@@ -116,35 +137,65 @@ void seg6local_context2json(const struct seg6local_context *ctx,
 	}
 }
 
+static char *seg6local_flavors2str(char *str, size_t size,
+				   const struct seg6local_flavors_info *flv_info)
+{
+	size_t len = 0;
+	bool first = true;
+
+	if (!CHECK_SRV6_FLV_OP(flv_info->flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_PSP |
+							  ZEBRA_SEG6_LOCAL_FLV_OP_USP |
+							  ZEBRA_SEG6_LOCAL_FLV_OP_USD))
+		return str;
+
+	len += snprintf(str + len, size - len, "(");
+	if (CHECK_SRV6_FLV_OP(flv_info->flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_PSP)) {
+		len += snprintf(str + len, size - len, "%sPSP", first ? "" : "/");
+		first = false;
+	}
+	if (CHECK_SRV6_FLV_OP(flv_info->flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_USP)) {
+		len += snprintf(str + len, size - len, "%sUSP", first ? "" : "/");
+		first = false;
+	}
+	if (CHECK_SRV6_FLV_OP(flv_info->flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_USD))
+		len += snprintf(str + len, size - len, "%sUSD", first ? "" : "/");
+
+	snprintf(str + len, size - len, ")");
+
+	return str;
+}
 const char *seg6local_context2str(char *str, size_t size,
 				  const struct seg6local_context *ctx,
 				  uint32_t action)
 {
-	switch (action) {
+	char flavor[SRV6_FLAVORS_STRLEN], *p_flavor;
 
+	flavor[0] = '\0';
+	p_flavor = seg6local_flavors2str(flavor, sizeof(flavor), &ctx->flv);
+	switch (action) {
 	case ZEBRA_SEG6_LOCAL_ACTION_END:
-		snprintf(str, size, "-");
+		snprintf(str, size, "%s", p_flavor);
 		return str;
 
 	case ZEBRA_SEG6_LOCAL_ACTION_END_X:
 	case ZEBRA_SEG6_LOCAL_ACTION_END_DX6:
-		snprintfrr(str, size, "nh6 %pI6", &ctx->nh6);
+		snprintfrr(str, size, "nh6 %pI6%s", &ctx->nh6, p_flavor);
 		return str;
 
 	case ZEBRA_SEG6_LOCAL_ACTION_END_DX4:
-		snprintfrr(str, size, "nh4 %pI4", &ctx->nh4);
+		snprintfrr(str, size, "nh4 %pI4%s", &ctx->nh4, p_flavor);
 		return str;
 
 	case ZEBRA_SEG6_LOCAL_ACTION_END_T:
 	case ZEBRA_SEG6_LOCAL_ACTION_END_DT6:
 	case ZEBRA_SEG6_LOCAL_ACTION_END_DT4:
 	case ZEBRA_SEG6_LOCAL_ACTION_END_DT46:
-		snprintf(str, size, "table %u", ctx->table);
+		snprintf(str, size, "table %u%s", ctx->table, p_flavor);
 		return str;
 
 	case ZEBRA_SEG6_LOCAL_ACTION_END_B6:
 	case ZEBRA_SEG6_LOCAL_ACTION_END_B6_ENCAP:
-		snprintfrr(str, size, "nh6 %pI6", &ctx->nh6);
+		snprintfrr(str, size, "nh6 %pI6%s", &ctx->nh6, p_flavor);
 		return str;
 	case ZEBRA_SEG6_LOCAL_ACTION_END_DX2:
 	case ZEBRA_SEG6_LOCAL_ACTION_END_BM:
@@ -470,3 +521,11 @@ json_object *srv6_locator_detailed_json(const struct srv6_locator *loc)
 
 	return jo_root;
 }
+
+/* clang-format off */
+const struct frr_yang_module_info ietf_srv6_types_info = {
+	.name = "ietf-srv6-types",
+	.ignore_cfg_cbs = true,
+	.nodes = { { .xpath = NULL } },
+};
+/* clang-format on */
