@@ -41,6 +41,15 @@ extern "C" {
 #define ZEBRA_DEFAULT_SEG6_LOCAL_FLV_LCBLOCK_LEN 32
 #define ZEBRA_DEFAULT_SEG6_LOCAL_FLV_LCNODE_FN_LEN 16
 
+/* SR Policy Headend Behaviors as per RFC 8986 section #5 */
+enum srv6_headend_behavior {
+	SRV6_HEADEND_BEHAVIOR_H_INSERT,
+	SRV6_HEADEND_BEHAVIOR_H_ENCAPS,
+	SRV6_HEADEND_BEHAVIOR_H_ENCAPS_RED,
+	SRV6_HEADEND_BEHAVIOR_H_ENCAPS_L2,
+	SRV6_HEADEND_BEHAVIOR_H_ENCAPS_L2_RED,
+};
+
 enum seg6_mode_t {
 	INLINE,
 	ENCAP,
@@ -88,6 +97,9 @@ struct seg6_segs {
 	struct in6_addr segs[256];
 };
 
+/* flavors psp, usd, usp, next-csid */
+#define SRV6_FLAVORS_STRLEN 50
+
 struct seg6local_flavors_info {
 	/* Flavor operations */
 	uint32_t flv_ops;
@@ -99,6 +111,7 @@ struct seg6local_flavors_info {
 };
 
 struct seg6_seg_stack {
+	enum srv6_headend_behavior encap_behavior;
 	uint8_t num_segs;
 	struct in6_addr seg[0]; /* 1 or more segs */
 };
@@ -134,6 +147,7 @@ struct srv6_locator {
 
 	uint8_t flags;
 #define SRV6_LOCATOR_USID (1 << 0) /* The SRv6 Locator is a uSID Locator */
+#define SRV6_LOCATOR_PSP  (1 << 1) /* The SRv6 Locator has the PSP flavor */
 
 	/* Pointer to the SID format. */
 	struct srv6_sid_format *sid_format;
@@ -176,18 +190,60 @@ struct srv6_locator_chunk {
 enum srv6_endpoint_behavior_codepoint {
 	SRV6_ENDPOINT_BEHAVIOR_RESERVED         = 0x0000,
 	SRV6_ENDPOINT_BEHAVIOR_END              = 0x0001,
+	SRV6_ENDPOINT_BEHAVIOR_END_PSP          = 0x0002,
 	SRV6_ENDPOINT_BEHAVIOR_END_X            = 0x0005,
+	SRV6_ENDPOINT_BEHAVIOR_END_X_PSP        = 0x0006,
+	SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS    = 0x000E,
 	SRV6_ENDPOINT_BEHAVIOR_END_DT6          = 0x0012,
 	SRV6_ENDPOINT_BEHAVIOR_END_DT4          = 0x0013,
 	SRV6_ENDPOINT_BEHAVIOR_END_DT46         = 0x0014,
+	SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS_RED = 0x001B,
+	SRV6_ENDPOINT_BEHAVIOR_END_PSP_USD      = 0x001D,
+	SRV6_ENDPOINT_BEHAVIOR_END_X_PSP_USD    = 0x0021,
 	SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID    = 0x002B,
-	SRV6_ENDPOINT_BEHAVIOR_END_X_NEXT_CSID  = 0x002C,
+	SRV6_ENDPOINT_BEHAVIOR_END_X_NEXT_CSID  = 0x0034,
+	SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID_PSP   = 0x002C,
+	SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID_PSP_USD   = 0x0030,
+	SRV6_ENDPOINT_BEHAVIOR_END_X_NEXT_CSID_PSP = 0x0035,
+	SRV6_ENDPOINT_BEHAVIOR_END_X_NEXT_CSID_PSP_USD = 0x0039,
 	SRV6_ENDPOINT_BEHAVIOR_END_DT6_USID     = 0x003E,
 	SRV6_ENDPOINT_BEHAVIOR_END_DT4_USID     = 0x003F,
 	SRV6_ENDPOINT_BEHAVIOR_END_DT46_USID    = 0x0040,
+	SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS_NEXT_CSID = 0x005D,
+	SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS_RED_NEXT_CSID = 0x005E,
 	SRV6_ENDPOINT_BEHAVIOR_OPAQUE           = 0xFFFF,
 };
 
+macro_inline uint16_t srv6_behavior_codepoint_get(enum srv6_endpoint_behavior_codepoint _codepoint,
+						  uint8_t _flags)
+{
+	if (_codepoint == SRV6_ENDPOINT_BEHAVIOR_END) {
+		if (CHECK_FLAG(_flags, SRV6_LOCATOR_USID)) {
+			if (CHECK_FLAG(_flags, SRV6_LOCATOR_PSP))
+				return SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID_PSP;
+			else
+				return SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID;
+		}
+		if (CHECK_FLAG(_flags, SRV6_LOCATOR_PSP))
+			return SRV6_ENDPOINT_BEHAVIOR_END_PSP;
+		return SRV6_ENDPOINT_BEHAVIOR_END;
+	}
+	return _codepoint;
+}
+
+#define SRV6_BEHAVIOR_CODEPOINT(_codepoint, _flags)                                                \
+	srv6_behavior_codepoint_get((_codepoint), (_flags))
+
+/*
+ * Return true if next-csid behavior is used, false otherwise
+ */
+static inline bool seg6local_has_next_csid(const struct seg6local_context *ctx)
+{
+	const struct seg6local_flavors_info *flv_info = &ctx->flv;
+
+	return CHECK_SRV6_FLV_OP(flv_info->flv_ops, ZEBRA_SEG6_LOCAL_FLV_OP_NEXT_CSID);
+}
+
 /*
  * Convert SRv6 endpoint behavior codepoints to human-friendly string.
  */
@@ -199,24 +255,48 @@ srv6_endpoint_behavior_codepoint2str(enum srv6_endpoint_behavior_codepoint behav
 		return "Reserved";
 	case SRV6_ENDPOINT_BEHAVIOR_END:
 		return "End";
+	case SRV6_ENDPOINT_BEHAVIOR_END_PSP:
+		return "End PSP";
+	case SRV6_ENDPOINT_BEHAVIOR_END_PSP_USD:
+		return "End PSP/USD";
 	case SRV6_ENDPOINT_BEHAVIOR_END_X:
 		return "End.X";
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_PSP:
+		return "End.X PSP";
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS:
+		return "End.B6.Encaps";
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_PSP_USD:
+		return "End.X PSP/USD";
 	case SRV6_ENDPOINT_BEHAVIOR_END_DT6:
 		return "End.DT6";
 	case SRV6_ENDPOINT_BEHAVIOR_END_DT4:
 		return "End.DT4";
 	case SRV6_ENDPOINT_BEHAVIOR_END_DT46:
 		return "End.DT46";
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS_RED:
+		return "End.B6.Encaps.Red";
 	case SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID:
 		return "uN";
+	case SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID_PSP:
+		return "uN PSP";
+	case SRV6_ENDPOINT_BEHAVIOR_END_NEXT_CSID_PSP_USD:
+		return "uN PSP/USD";
 	case SRV6_ENDPOINT_BEHAVIOR_END_X_NEXT_CSID:
 		return "uA";
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_NEXT_CSID_PSP:
+		return "uA PSP";
+	case SRV6_ENDPOINT_BEHAVIOR_END_X_NEXT_CSID_PSP_USD:
+		return "uA PSP/USD";
 	case SRV6_ENDPOINT_BEHAVIOR_END_DT6_USID:
 		return "uDT6";
 	case SRV6_ENDPOINT_BEHAVIOR_END_DT4_USID:
 		return "uDT4";
 	case SRV6_ENDPOINT_BEHAVIOR_END_DT46_USID:
 		return "uDT46";
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS_NEXT_CSID:
+		return "uB6.Encaps";
+	case SRV6_ENDPOINT_BEHAVIOR_END_B6_ENCAPS_RED_NEXT_CSID:
+		return "uB6.Encaps.Red";
 	case SRV6_ENDPOINT_BEHAVIOR_OPAQUE:
 		return "Opaque";
 	}
@@ -297,8 +377,28 @@ struct srv6_sid_ctx {
 	struct in_addr nh4;
 	struct in6_addr nh6;
 	vrf_id_t vrf_id;
+	ifindex_t ifindex;
 };
 
+static inline const char *srv6_headend_behavior2str(enum srv6_headend_behavior behavior,
+						    bool running_conf)
+{
+	switch (behavior) {
+	case SRV6_HEADEND_BEHAVIOR_H_INSERT:
+		return running_conf ? "H_Insert" : "H.Insert";
+	case SRV6_HEADEND_BEHAVIOR_H_ENCAPS:
+		return running_conf ? "H_Encaps" : "H.Encaps";
+	case SRV6_HEADEND_BEHAVIOR_H_ENCAPS_RED:
+		return running_conf ? "H_Encaps_Red" : "H.Encaps.Red";
+	case SRV6_HEADEND_BEHAVIOR_H_ENCAPS_L2:
+		return running_conf ? "H_Encaps_L2" : "H.Encaps.L2";
+	case SRV6_HEADEND_BEHAVIOR_H_ENCAPS_L2_RED:
+		return running_conf ? "H_Encaps_L2_Red" : "H.Encaps.L2.Red";
+	}
+
+	return "unknown";
+}
+
 static inline const char *seg6_mode2str(enum seg6_mode_t mode)
 {
 	switch (mode) {
@@ -355,6 +455,8 @@ static inline void *sid_copy(struct in6_addr *dst,
 	return memcpy(dst, src, sizeof(struct in6_addr));
 }
 
+const char *seg6local_action2str_with_next_csid(uint32_t action, bool has_next_csid);
+
 const char *
 seg6local_action2str(uint32_t action);
 
@@ -412,6 +514,8 @@ static inline const char *srv6_sid_ctx2str(char *str, size_t size,
 	return str;
 }
 
+extern const struct frr_yang_module_info ietf_srv6_types_info;
+
 int snprintf_seg6_segs(char *str,
 		size_t size, const struct seg6_segs *segs);
 
