@@ -10,7 +10,8 @@
 #include <stddef.h>
 #include <stdint.h>
 #include <stdbool.h>
-#include "compiler.h"
+
+#include "lib/compiler.h"
 #endif /* _TYPESAFE_EXPAND_MACROS */
 
 #ifdef __cplusplus
@@ -1170,9 +1171,9 @@ extern struct sskip_item *typesafe_skiplist_pop(struct sskip_head *head);
 }
 #endif
 
-/* this needs to stay at the end because both files include each other.
- * the resolved order is typesafe.h before typerb.h
+/* the separation between these two files is really only for documentation
+ * purposes, the are interdependent and are always included as a bundle
  */
-#include "typerb.h"
+#include "lib/typerb.h" // IWYU pragma: export
 
 #endif /* _FRR_TYPESAFE_H */
