@@ -1,21 +1,8 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Timer Wheel
  * Copyright (C) 2016 Cumulus Networks, Inc.
  * Donald Sharp
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #ifndef __WHEEL_H__
 #define __WHEEL_H__
@@ -25,8 +12,7 @@ extern "C" {
 #endif
 
 struct timer_wheel {
-	char *name;
-	struct thread_master *master;
+	struct event_loop *master;
 	int slots;
 	long long curr_slot;
 	unsigned int period;
@@ -34,7 +20,7 @@ struct timer_wheel {
 	unsigned int slots_to_skip;
 
 	struct list **wheel_slot_lists;
-	struct thread *timer;
+	struct event *timer;
 	/*
 	 * Key to determine what slot the item belongs in
 	 */
@@ -79,7 +65,7 @@ struct timer_wheel {
  * and cause significant amount of time handling thread events instead
  * of running your code.
  */
-struct timer_wheel *wheel_init(struct thread_master *master, int period,
+struct timer_wheel *wheel_init(struct event_loop *master, int period,
 			       size_t slots,
 			       unsigned int (*slot_key)(const void *),
 			       void (*slot_run)(void *), const char *run_name);
@@ -89,16 +75,6 @@ struct timer_wheel *wheel_init(struct thread_master *master, int period,
  */
 void wheel_delete(struct timer_wheel *);
 
-/*
- * Pause the Wheel from running
- */
-int wheel_stop(struct timer_wheel *wheel);
-
-/*
- * Start the wheel running again
- */
-int wheel_start(struct timer_wheel *wheel);
-
 /*
  * wheel - The Timer wheel being modified
  * item - The generic data structure that will be handed
