@@ -1,21 +1,11 @@
+// SPDX-License-Identifier: ISC
 /*
  * Copyright (c) 2015-19  David Lamparter, for NetDEF, Inc.
- *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
- *
- * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
- * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
- * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
- * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
- * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
- * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
- * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
 #include "zebra.h"
 
+#include <fcntl.h>
 #include <sys/un.h>
 #include <syslog.h>
 
@@ -218,7 +208,8 @@ static bool zlog_file_cycle(struct zlog_cfg_file *zcf)
 			break;
 		}
 
-		zt = zlog_target_clone(MTYPE_LOG_FD, &zcf->active->zt,
+		zt = zlog_target_clone(MTYPE_LOG_FD,
+				       zcf->active ? &zcf->active->zt : NULL,
 				       sizeof(*zlt));
 		zlt = container_of(zt, struct zlt_fd, zt);
 
@@ -257,6 +248,7 @@ bool zlog_file_set_filename(struct zlog_cfg_file *zcf, const char *filename)
 		return zlog_file_cycle(zcf);
 	}
 	assert(0);
+	return false;
 }
 
 bool zlog_file_set_fd(struct zlog_cfg_file *zcf, int fd)
@@ -271,6 +263,7 @@ bool zlog_file_set_fd(struct zlog_cfg_file *zcf, int fd)
 		return zlog_file_cycle(zcf);
 	}
 	assert(0);
+	return false;
 }
 
 struct rcu_close_rotate {
@@ -544,6 +537,7 @@ int zlog_syslog_get_facility(void)
 		return syslog_facility;
 	}
 	assert(0);
+	return 0;
 }
 
 void zlog_syslog_set_prio_min(int prio_min)
@@ -558,7 +552,7 @@ void zlog_syslog_set_prio_min(int prio_min)
 
 		if (syslog_prio_min != ZLOG_DISABLED) {
 			newztc = zlog_target_clone(MTYPE_LOG_SYSL,
-						   &zlt_syslog->zt,
+						   zlt_syslog ? &zlt_syslog->zt : NULL,
 						   sizeof(*newzt));
 			newzt = container_of(newztc, struct zlt_syslog, zt);
 			newzt->zt.prio_min = prio_min;
@@ -568,7 +562,7 @@ void zlog_syslog_set_prio_min(int prio_min)
 		}
 
 		zlog_target_free(MTYPE_LOG_SYSL,
-				 zlog_target_replace(&zlt_syslog->zt,
+				 zlog_target_replace(zlt_syslog ? &zlt_syslog->zt : NULL,
 						     &newzt->zt));
 
 		zlt_syslog = newzt;
@@ -581,4 +575,5 @@ int zlog_syslog_get_prio_min(void)
 		return syslog_prio_min;
 	}
 	assert(0);
+	return 0;
 }
