@@ -39,16 +39,6 @@ struct mgmt_commit_stats {
 	unsigned long commit_cnt;
 };
 
-struct mgmt_setcfg_stats {
-	struct timeval last_start;
-	struct timeval last_end;
-	unsigned long last_exec_tm;
-	unsigned long max_tm;
-	unsigned long min_tm;
-	unsigned long avg_tm;
-	unsigned long set_cfg_count;
-};
-
 PREDECL_LIST(mgmt_fe_sessions);
 
 PREDECL_LIST(mgmt_fe_adapters);
@@ -62,7 +52,6 @@ struct mgmt_fe_client_adapter {
 
 	int refcount;
 	struct mgmt_commit_stats cmt_stats;
-	struct mgmt_setcfg_stats setcfg_stats;
 
 	struct mgmt_fe_adapters_item list_linkage;
 };
@@ -86,56 +75,13 @@ mgmt_fe_adapter_unlock(struct mgmt_fe_client_adapter **adapter);
 extern struct msg_conn *mgmt_fe_create_adapter(int conn_fd,
 					       union sockunion *su);
 
-/*
- * Send set-config reply to the frontend client.
- *
- * session
- *    Unique session identifier.
- *
- * txn_id
- *    Unique transaction identifier.
- *
- * ds_id
- *    Datastore ID.
- *
- * req_id
- *    Config request ID.
- *
- * result
- *    Config request result (MGMT_*).
- *
- * error_if_any
- *    Buffer to store human-readable error message in case of error.
- *
- * implicit_commit
- *    TRUE if the commit is implicit, FALSE otherwise.
- *
- * Returns:
- *    0 on success, -1 on failures.
- */
-extern int mgmt_fe_send_set_cfg_reply(uint64_t session_id, uint64_t txn_id,
-					  Mgmtd__DatastoreId ds_id,
-					  uint64_t req_id,
-					  enum mgmt_result result,
-					  const char *error_if_any,
-					  bool implcit_commit);
-
 /*
  * Send commit-config reply to the frontend client.
  */
-extern int mgmt_fe_send_commit_cfg_reply(
-	uint64_t session_id, uint64_t txn_id, Mgmtd__DatastoreId src_ds_id,
-	Mgmtd__DatastoreId dst_ds_id, uint64_t req_id, bool validate_only,
-	enum mgmt_result result, const char *error_if_any);
-
-/*
- * Send get-config/get-data reply to the frontend client.
- */
-extern int mgmt_fe_send_get_reply(uint64_t session_id, uint64_t txn_id,
-				  Mgmtd__DatastoreId ds_id, uint64_t req_id,
-				  enum mgmt_result result,
-				  Mgmtd__YangDataReply *data_resp,
-				  const char *error_if_any);
+extern int mgmt_fe_send_commit_cfg_reply(uint64_t session_id, uint64_t txn_id,
+					 enum mgmt_ds_id src_ds_id, enum mgmt_ds_id dst_ds_id,
+					 uint64_t req_id, bool validate_only, bool unlock,
+					 enum mgmt_result result, const char *error_if_any);
 
 /**
  * Send get-tree data reply back to client.
@@ -232,10 +178,6 @@ extern int mgmt_fe_adapter_txn_error(uint64_t txn_id, uint64_t req_id,
  */
 extern char **mgmt_fe_get_all_selectors(void);
 
-/* Fetch frontend client session set-config stats */
-extern struct mgmt_setcfg_stats *
-mgmt_fe_get_session_setcfg_stats(uint64_t session_id);
-
 /* Fetch frontend client session commit stats */
 extern struct mgmt_commit_stats *
 mgmt_fe_get_session_commit_stats(uint64_t session_id);
