@@ -104,7 +104,7 @@ void ospf6_lsa_originate(struct ospf6 *ospf6, struct ospf6_lsa *lsa)
 	lsdb_self = ospf6_get_scoped_lsdb_self(lsa);
 	ospf6_lsdb_add(ospf6_lsa_copy(lsa), lsdb_self);
 
-	EVENT_OFF(lsa->refresh);
+	event_cancel(&lsa->refresh);
 	event_add_timer(master, ospf6_lsa_refresh, lsa, OSPF_LS_REFRESH_TIME,
 			&lsa->refresh);
 
@@ -169,8 +169,8 @@ void ospf6_lsa_purge(struct ospf6_lsa *lsa)
 	self = ospf6_lsdb_lookup(lsa->header->type, lsa->header->id,
 				 lsa->header->adv_router, lsdb_self);
 	if (self) {
-		EVENT_OFF(self->expire);
-		EVENT_OFF(self->refresh);
+		event_cancel(&self->expire);
+		event_cancel(&self->refresh);
 		ospf6_lsdb_remove(self, lsdb_self);
 	}
 
@@ -251,8 +251,8 @@ void ospf6_install_lsa(struct ospf6_lsa *lsa)
 					   lsa->name);
 			lsa->external_lsa_id = old->external_lsa_id;
 		}
-		EVENT_OFF(old->expire);
-		EVENT_OFF(old->refresh);
+		event_cancel(&old->expire);
+		event_cancel(&old->refresh);
 		ospf6_flood_clear(old);
 	}
 
@@ -524,7 +524,7 @@ void ospf6_flood_interface(struct ospf6_neighbor *from, struct ospf6_lsa *lsa,
 	} else {
 		/* reschedule retransmissions to all neighbors */
 		for (ALL_LIST_ELEMENTS(oi->neighbor_list, node, nnode, on)) {
-			EVENT_OFF(on->thread_send_lsupdate);
+			event_cancel(&on->thread_send_lsupdate);
 			event_add_event(master, ospf6_lsupdate_send_neighbor,
 					on, 0, &on->thread_send_lsupdate);
 		}
@@ -1061,6 +1061,7 @@ void ospf6_receive_lsa(struct ospf6_neighbor *from,
 						zlog_debug(
 							"%s, Not moving to HELPER role, So dicarding GraceLSA",
 							__func__);
+					ospf6_lsa_delete(new);
 					return;
 				}
 			}
