@@ -142,20 +142,20 @@ static void ospf6_set_redist_vrf_bitmaps(struct ospf6 *ospf6, bool set)
 				"%s: setting redist vrf %d bitmap for type %d",
 				__func__, ospf6->vrf_id, type);
 		if (set)
-			vrf_bitmap_set(&zclient->redist[AFI_IP6][type],
+			vrf_bitmap_set(&ospf6_zclient->redist[AFI_IP6][type],
 				       ospf6->vrf_id);
 		else
-			vrf_bitmap_unset(&zclient->redist[AFI_IP6][type],
+			vrf_bitmap_unset(&ospf6_zclient->redist[AFI_IP6][type],
 					 ospf6->vrf_id);
 	}
 
 	red_list = ospf6->redist[DEFAULT_ROUTE];
 	if (red_list) {
 		if (set)
-			vrf_bitmap_set(&zclient->default_information[AFI_IP6],
+			vrf_bitmap_set(&ospf6_zclient->default_information[AFI_IP6],
 				       ospf6->vrf_id);
 		else
-			vrf_bitmap_unset(&zclient->default_information[AFI_IP6],
+			vrf_bitmap_unset(&ospf6_zclient->default_information[AFI_IP6],
 					 ospf6->vrf_id);
 	}
 }
@@ -554,25 +554,25 @@ static void ospf6_disable(struct ospf6 *o)
 		ospf6_route_remove_all(o->route_table);
 		ospf6_route_remove_all(o->brouter_table);
 
-		EVENT_OFF(o->maxage_remover);
-		EVENT_OFF(o->t_spf_calc);
-		EVENT_OFF(o->t_ase_calc);
-		EVENT_OFF(o->t_distribute_update);
-		EVENT_OFF(o->t_ospf6_receive);
-		EVENT_OFF(o->t_external_aggr);
-		EVENT_OFF(o->gr_info.t_grace_period);
-		EVENT_OFF(o->t_write);
-		EVENT_OFF(o->t_abr_task);
+		event_cancel(&o->maxage_remover);
+		event_cancel(&o->t_spf_calc);
+		event_cancel(&o->t_ase_calc);
+		event_cancel(&o->t_distribute_update);
+		event_cancel(&o->t_ospf6_receive);
+		event_cancel(&o->t_external_aggr);
+		event_cancel(&o->gr_info.t_grace_period);
+		event_cancel(&o->t_write);
+		event_cancel(&o->t_abr_task);
 	}
 }
 
-void ospf6_master_init(struct event_loop *master)
+void ospf6_master_init(struct event_loop *mst)
 {
 	memset(&ospf6_master, 0, sizeof(ospf6_master));
 
 	om6 = &ospf6_master;
 	om6->ospf6 = list_new();
-	om6->master = master;
+	om6->master = mst;
 }
 
 void ospf6_master_delete(void)
