@@ -1,3 +1,4 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * This is an implementation of RFC7684 OSPFv2 Prefix/Link Attribute
  * Advertisement
@@ -8,20 +9,6 @@
  * Author: Anselme Sawadogo <anselmesawadogo@gmail.com>
  *
  * Copyright (C) 2016 - 2018 Orange Labs http://www.orange.com
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -38,12 +25,13 @@
 #include "vty.h"
 #include "stream.h"
 #include "log.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "hash.h"
 #include "sockunion.h" /* for inet_aton() */
 #include "network.h"
 #include "if.h"
 #include "libospf.h" /* for ospf interface types */
+#include <lib/json.h>
 
 #include "ospfd/ospfd.h"
 #include "ospfd/ospf_interface.h"
@@ -1728,85 +1716,138 @@ static void ospf_ext_lsa_schedule(struct ext_itf *exti, enum lsa_opcode op)
 
 /* Cisco experimental SubTLV */
 static uint16_t show_vty_ext_link_rmt_itf_addr(struct vty *vty,
-					       struct tlv_header *tlvh)
+					       struct tlv_header *tlvh,
+					       json_object *json)
 {
 	struct ext_subtlv_rmt_itf_addr *top =
 		(struct ext_subtlv_rmt_itf_addr *)tlvh;
 
 	check_tlv_size(EXT_SUBTLV_RMT_ITF_ADDR_SIZE, "Remote Itf. Address");
 
-	vty_out(vty,
-		"  Remote Interface Address Sub-TLV: Length %u\n	Address: %pI4\n",
-		ntohs(top->header.length), &top->value);
+	if (!json)
+		vty_out(vty,
+			"  Remote Interface Address Sub-TLV: Length %u\n	Address: %pI4\n",
+			ntohs(top->header.length), &top->value);
+	else
+		json_object_string_addf(json, "remoteInterfaceAddress", "%pI4",
+					&top->value);
 
 	return TLV_SIZE(tlvh);
 }
 
 /* Adjacency SID SubTLV */
 static uint16_t show_vty_ext_link_adj_sid(struct vty *vty,
-					  struct tlv_header *tlvh)
+					  struct tlv_header *tlvh,
+					  json_object *json)
 {
 	struct ext_subtlv_adj_sid *top = (struct ext_subtlv_adj_sid *)tlvh;
+	uint8_t tlv_size;
 
-	check_tlv_size(EXT_SUBTLV_ADJ_SID_SIZE, "Adjacency SID");
+	tlv_size = CHECK_FLAG(top->flags, EXT_SUBTLV_LINK_ADJ_SID_VFLG)
+			      ? SID_LABEL_SIZE(EXT_SUBTLV_ADJ_SID_SIZE)
+			      : SID_INDEX_SIZE(EXT_SUBTLV_ADJ_SID_SIZE);
+	check_tlv_size(tlv_size, "Adjacency SID");
 
-	vty_out(vty,
-		"  Adj-SID Sub-TLV: Length %u\n\tFlags: 0x%x\n\tMT-ID:0x%x\n\tWeight: 0x%x\n\t%s: %u\n",
-		ntohs(top->header.length), top->flags, top->mtid, top->weight,
-		CHECK_FLAG(top->flags, EXT_SUBTLV_LINK_ADJ_SID_VFLG) ? "Label"
-								     : "Index",
-		CHECK_FLAG(top->flags, EXT_SUBTLV_LINK_ADJ_SID_VFLG)
-			? GET_LABEL(ntohl(top->value))
-			: ntohl(top->value));
+	if (!json)
+		vty_out(vty,
+			"  Adj-SID Sub-TLV: Length %u\n\tFlags: 0x%x\n\tMT-ID:0x%x\n\tWeight: 0x%x\n\t%s: %u\n",
+			ntohs(top->header.length), top->flags, top->mtid,
+			top->weight,
+			CHECK_FLAG(top->flags, EXT_SUBTLV_LINK_ADJ_SID_VFLG)
+				? "Label"
+				: "Index",
+			CHECK_FLAG(top->flags, EXT_SUBTLV_LINK_ADJ_SID_VFLG)
+				? GET_LABEL(ntohl(top->value))
+				: ntohl(top->value));
+	else {
+		json_object_string_addf(json, "flags", "0x%x", top->flags);
+		json_object_string_addf(json, "mtID", "0x%x", top->mtid);
+		json_object_string_addf(json, "weight", "0x%x", top->weight);
+		if (CHECK_FLAG(top->flags, EXT_SUBTLV_LINK_ADJ_SID_VFLG))
+			json_object_int_add(json, "label",
+					    GET_LABEL(ntohl(top->value)));
+		else
+			json_object_int_add(json, "index", ntohl(top->value));
+	}
 
 	return TLV_SIZE(tlvh);
 }
 
 /* LAN Adjacency SubTLV */
 static uint16_t show_vty_ext_link_lan_adj_sid(struct vty *vty,
-					      struct tlv_header *tlvh)
+					      struct tlv_header *tlvh,
+					      json_object *json)
 {
 	struct ext_subtlv_lan_adj_sid *top =
 		(struct ext_subtlv_lan_adj_sid *)tlvh;
+	uint8_t tlv_size;
 
-	check_tlv_size(EXT_SUBTLV_LAN_ADJ_SID_SIZE, "Lan-Adjacency SID");
+	tlv_size = CHECK_FLAG(top->flags, EXT_SUBTLV_LINK_ADJ_SID_VFLG)
+			      ? SID_LABEL_SIZE(EXT_SUBTLV_LAN_ADJ_SID_SIZE)
+			      : SID_INDEX_SIZE(EXT_SUBTLV_LAN_ADJ_SID_SIZE);
+	check_tlv_size(tlv_size, "LAN-Adjacency SID");
 
-	vty_out(vty,
-		"  LAN-Adj-SID Sub-TLV: Length %u\n\tFlags: 0x%x\n\tMT-ID:0x%x\n\tWeight: 0x%x\n\tNeighbor ID: %pI4\n\t%s: %u\n",
-		ntohs(top->header.length), top->flags, top->mtid, top->weight,
-		&top->neighbor_id,
-		CHECK_FLAG(top->flags, EXT_SUBTLV_LINK_ADJ_SID_VFLG) ? "Label"
-								     : "Index",
-		CHECK_FLAG(top->flags, EXT_SUBTLV_LINK_ADJ_SID_VFLG)
-			? GET_LABEL(ntohl(top->value))
-			: ntohl(top->value));
+	if (!json)
+		vty_out(vty,
+			"  LAN-Adj-SID Sub-TLV: Length %u\n\tFlags: 0x%x\n\tMT-ID:0x%x\n\tWeight: 0x%x\n\tNeighbor ID: %pI4\n\t%s: %u\n",
+			ntohs(top->header.length), top->flags, top->mtid,
+			top->weight, &top->neighbor_id,
+			CHECK_FLAG(top->flags, EXT_SUBTLV_LINK_ADJ_SID_VFLG)
+				? "Label"
+				: "Index",
+			CHECK_FLAG(top->flags, EXT_SUBTLV_LINK_ADJ_SID_VFLG)
+				? GET_LABEL(ntohl(top->value))
+				: ntohl(top->value));
+	else {
+		json_object_string_addf(json, "flags", "0x%x", top->flags);
+		json_object_string_addf(json, "mtID", "0x%x", top->mtid);
+		json_object_string_addf(json, "weight", "0x%x", top->weight);
+		json_object_string_addf(json, "neighborID", "%pI4",
+					&top->neighbor_id);
+		if (CHECK_FLAG(top->flags, EXT_SUBTLV_LINK_ADJ_SID_VFLG))
+			json_object_int_add(json, "label",
+					    GET_LABEL(ntohl(top->value)));
+		else
+			json_object_int_add(json, "index", ntohl(top->value));
+	}
 
 	return TLV_SIZE(tlvh);
 }
 
 static uint16_t show_vty_unknown_tlv(struct vty *vty, struct tlv_header *tlvh,
-				     size_t buf_size)
+				     size_t buf_size, json_object *json)
 {
+	json_object *obj;
+
 	if (TLV_SIZE(tlvh) > buf_size) {
 		vty_out(vty, "    TLV size %d exceeds buffer size. Abort!",
 			TLV_SIZE(tlvh));
 		return buf_size;
 	}
-
-	vty_out(vty, "    Unknown TLV: [type(0x%x), length(0x%x)]\n",
-		ntohs(tlvh->type), ntohs(tlvh->length));
+	if (!json)
+		vty_out(vty, "    Unknown TLV: [type(0x%x), length(0x%x)]\n",
+			ntohs(tlvh->type), ntohs(tlvh->length));
+	else {
+		obj = json_object_new_object();
+		json_object_string_addf(obj, "type", "0x%x",
+					ntohs(tlvh->type));
+		json_object_string_addf(obj, "length", "0x%x",
+					ntohs(tlvh->length));
+		json_object_object_add(json, "unknownTLV", obj);
+	}
 
 	return TLV_SIZE(tlvh);
 }
 
 /* Extended Link Sub TLVs */
 static uint16_t show_vty_link_info(struct vty *vty, struct tlv_header *ext,
-				   size_t buf_size)
+				   size_t buf_size, json_object *json)
 {
 	struct ext_tlv_link *top = (struct ext_tlv_link *)ext;
 	struct tlv_header *tlvh;
 	uint16_t length = ntohs(top->header.length);
 	uint16_t sum = 0;
+	json_object *jadj = NULL, *obj = NULL;
 
 	/* Verify that TLV length is valid against remaining buffer size */
 	if (length > buf_size) {
@@ -1816,12 +1857,22 @@ static uint16_t show_vty_link_info(struct vty *vty, struct tlv_header *ext,
 		return buf_size;
 	}
 
-	vty_out(vty,
-		"  Extended Link TLV: Length %u\n	Link Type: 0x%x\n"
-		"	Link ID: %pI4\n",
-		ntohs(top->header.length), top->link_type,
-		&top->link_id);
-	vty_out(vty, "	Link data: %pI4\n", &top->link_data);
+	if (!json) {
+		vty_out(vty,
+			"  Extended Link TLV: Length %u\n	Link Type: 0x%x\n"
+			"	Link ID: %pI4\n",
+			ntohs(top->header.length), top->link_type,
+			&top->link_id);
+		vty_out(vty, "	Link data: %pI4\n", &top->link_data);
+	} else {
+		json_object_string_addf(json, "linkType", "0x%x",
+					top->link_type);
+		json_object_string_addf(json, "linkID", "%pI4", &top->link_id);
+		json_object_string_addf(json, "linkData", "%pI4",
+					&top->link_data);
+		jadj = json_object_new_array();
+		json_object_object_add(json, "adjacencySID", jadj);
+	}
 
 	/* Skip Extended TLV and parse sub-TLVs */
 	length -= EXT_TLV_LINK_SIZE;
@@ -1830,16 +1881,27 @@ static uint16_t show_vty_link_info(struct vty *vty, struct tlv_header *ext,
 	for (; sum < length && tlvh; tlvh = TLV_HDR_NEXT(tlvh)) {
 		switch (ntohs(tlvh->type)) {
 		case EXT_SUBTLV_ADJ_SID:
-			sum += show_vty_ext_link_adj_sid(vty, tlvh);
+			if (json) {
+				obj = json_object_new_object();
+				json_object_array_add(jadj, obj);
+			} else
+				obj = NULL;
+			sum += show_vty_ext_link_adj_sid(vty, tlvh, obj);
 			break;
 		case EXT_SUBTLV_LAN_ADJ_SID:
-			sum += show_vty_ext_link_lan_adj_sid(vty, tlvh);
+			if (json) {
+				obj = json_object_new_object();
+				json_object_array_add(jadj, obj);
+			} else
+				obj = NULL;
+			sum += show_vty_ext_link_lan_adj_sid(vty, tlvh, obj);
 			break;
 		case EXT_SUBTLV_RMT_ITF_ADDR:
-			sum += show_vty_ext_link_rmt_itf_addr(vty, tlvh);
+			sum += show_vty_ext_link_rmt_itf_addr(vty, tlvh, json);
 			break;
 		default:
-			sum += show_vty_unknown_tlv(vty, tlvh, length - sum);
+			sum += show_vty_unknown_tlv(vty, tlvh, length - sum,
+						    json);
 			break;
 		}
 	}
@@ -1854,9 +1916,12 @@ static void ospf_ext_link_show_info(struct vty *vty, struct json_object *json,
 	struct lsa_header *lsah = lsa->data;
 	struct tlv_header *tlvh;
 	uint16_t length = 0, sum = 0;
+	json_object *jlink = NULL;
 
-	if (json)
-		return;
+	if (json) {
+		jlink = json_object_new_object();
+		json_object_object_add(json, "extendedLink", jlink);
+	}
 
 	/* Initialize TLV browsing */
 	length = lsa->size - OSPF_LSA_HEADER_SIZE;
@@ -1865,10 +1930,12 @@ static void ospf_ext_link_show_info(struct vty *vty, struct json_object *json,
 	     tlvh = TLV_HDR_NEXT(tlvh)) {
 		switch (ntohs(tlvh->type)) {
 		case EXT_TLV_LINK:
-			sum += show_vty_link_info(vty, tlvh, length - sum);
+			sum += show_vty_link_info(vty, tlvh, length - sum,
+						  jlink);
 			break;
 		default:
-			sum += show_vty_unknown_tlv(vty, tlvh, length - sum);
+			sum += show_vty_unknown_tlv(vty, tlvh, length - sum,
+						    jlink);
 			break;
 		}
 	}
@@ -1876,34 +1943,51 @@ static void ospf_ext_link_show_info(struct vty *vty, struct json_object *json,
 
 /* Prefix SID SubTLV */
 static uint16_t show_vty_ext_pref_pref_sid(struct vty *vty,
-					   struct tlv_header *tlvh)
+					   struct tlv_header *tlvh,
+					   json_object *json)
 {
 	struct ext_subtlv_prefix_sid *top =
 		(struct ext_subtlv_prefix_sid *)tlvh;
+	uint8_t tlv_size;
 
-	check_tlv_size(EXT_SUBTLV_PREFIX_SID_SIZE, "Prefix SID");
-
-	vty_out(vty,
-		"  Prefix SID Sub-TLV: Length %u\n\tAlgorithm: %u\n\tFlags: 0x%x\n\tMT-ID:0x%x\n\t%s: %u\n",
-		ntohs(top->header.length), top->algorithm, top->flags,
-		top->mtid,
-		CHECK_FLAG(top->flags, EXT_SUBTLV_PREFIX_SID_VFLG) ? "Label"
-								   : "Index",
-		CHECK_FLAG(top->flags, EXT_SUBTLV_PREFIX_SID_VFLG)
-			? GET_LABEL(ntohl(top->value))
-			: ntohl(top->value));
+	tlv_size = CHECK_FLAG(top->flags, EXT_SUBTLV_PREFIX_SID_VFLG)
+			      ? SID_LABEL_SIZE(EXT_SUBTLV_PREFIX_SID_SIZE)
+			      : SID_INDEX_SIZE(EXT_SUBTLV_PREFIX_SID_SIZE);
+	check_tlv_size(tlv_size, "Prefix SID");
 
+	if (!json)
+		vty_out(vty,
+			"  Prefix SID Sub-TLV: Length %u\n\tAlgorithm: %u\n\tFlags: 0x%x\n\tMT-ID:0x%x\n\t%s: %u\n",
+			ntohs(top->header.length), top->algorithm, top->flags,
+			top->mtid,
+			CHECK_FLAG(top->flags, EXT_SUBTLV_PREFIX_SID_VFLG)
+				? "Label"
+				: "Index",
+			CHECK_FLAG(top->flags, EXT_SUBTLV_PREFIX_SID_VFLG)
+				? GET_LABEL(ntohl(top->value))
+				: ntohl(top->value));
+	else {
+		json_object_int_add(json, "algorithm", top->algorithm);
+		json_object_string_addf(json, "flags", "0x%x", top->flags);
+		json_object_string_addf(json, "mtID", "0x%x", top->mtid);
+		if (CHECK_FLAG(top->flags, EXT_SUBTLV_PREFIX_SID_VFLG))
+			json_object_int_add(json, "label",
+					    GET_LABEL(ntohl(top->value)));
+		else
+			json_object_int_add(json, "index", ntohl(top->value));
+	}
 	return TLV_SIZE(tlvh);
 }
 
 /* Extended Prefix SubTLVs */
 static uint16_t show_vty_pref_info(struct vty *vty, struct tlv_header *ext,
-				   size_t buf_size)
+				   size_t buf_size, json_object *json)
 {
 	struct ext_tlv_prefix *top = (struct ext_tlv_prefix *)ext;
 	struct tlv_header *tlvh;
 	uint16_t length = ntohs(top->header.length);
 	uint16_t sum = 0;
+	json_object *jsid = NULL;
 
 	/* Verify that TLV length is valid against remaining buffer size */
 	if (length > buf_size) {
@@ -1913,11 +1997,21 @@ static uint16_t show_vty_pref_info(struct vty *vty, struct tlv_header *ext,
 		return buf_size;
 	}
 
-	vty_out(vty,
-		"  Extended Prefix TLV: Length %u\n\tRoute Type: %u\n"
-		"\tAddress Family: 0x%x\n\tFlags: 0x%x\n\tAddress: %pI4/%u\n",
-		ntohs(top->header.length), top->route_type, top->af, top->flags,
-		&top->address, top->pref_length);
+	if (!json)
+		vty_out(vty,
+			"  Extended Prefix TLV: Length %u\n\tRoute Type: %u\n"
+			"\tAddress Family: 0x%x\n\tFlags: 0x%x\n\tAddress: %pI4/%u\n",
+			ntohs(top->header.length), top->route_type, top->af,
+			top->flags, &top->address, top->pref_length);
+	else {
+		json_object_int_add(json, "routeType", top->route_type);
+		json_object_string_addf(json, "addressFamily", "0x%x", top->af);
+		json_object_string_addf(json, "flags", "0x%x", top->flags);
+		json_object_string_addf(json, "address", "%pI4", &top->address);
+		json_object_int_add(json, "prefixLength", top->pref_length);
+		jsid = json_object_new_object();
+		json_object_object_add(json, "prefixSID", jsid);
+	}
 
 	/* Skip Extended Prefix TLV and parse sub-TLVs */
 	length -= EXT_TLV_PREFIX_SIZE;
@@ -1926,10 +2020,11 @@ static uint16_t show_vty_pref_info(struct vty *vty, struct tlv_header *ext,
 	for (; sum < length && tlvh; tlvh = TLV_HDR_NEXT(tlvh)) {
 		switch (ntohs(tlvh->type)) {
 		case EXT_SUBTLV_PREFIX_SID:
-			sum += show_vty_ext_pref_pref_sid(vty, tlvh);
+			sum += show_vty_ext_pref_pref_sid(vty, tlvh, jsid);
 			break;
 		default:
-			sum += show_vty_unknown_tlv(vty, tlvh, length - sum);
+			sum += show_vty_unknown_tlv(vty, tlvh, length - sum,
+						    json);
 			break;
 		}
 	}
@@ -1944,9 +2039,12 @@ static void ospf_ext_pref_show_info(struct vty *vty, struct json_object *json,
 	struct lsa_header *lsah = lsa->data;
 	struct tlv_header *tlvh;
 	uint16_t length = 0, sum = 0;
+	json_object *jpref = NULL;
 
-	if (json)
-		return;
+	if (json) {
+		jpref = json_object_new_object();
+		json_object_object_add(json, "extendedPrefix", jpref);
+	}
 
 	/* Initialize TLV browsing */
 	length = lsa->size - OSPF_LSA_HEADER_SIZE;
@@ -1955,10 +2053,12 @@ static void ospf_ext_pref_show_info(struct vty *vty, struct json_object *json,
 	     tlvh = TLV_HDR_NEXT(tlvh)) {
 		switch (ntohs(tlvh->type)) {
 		case EXT_TLV_PREFIX:
-			sum += show_vty_pref_info(vty, tlvh, length - sum);
+			sum += show_vty_pref_info(vty, tlvh, length - sum,
+						  jpref);
 			break;
 		default:
-			sum += show_vty_unknown_tlv(vty, tlvh, length - sum);
+			sum += show_vty_unknown_tlv(vty, tlvh, length - sum,
+						    jpref);
 			break;
 		}
 	}
