@@ -225,7 +225,7 @@ static void ospf_gr_restart_exit(struct ospf *ospf, const char *reason)
 		zlog_debug("GR: exiting graceful restart: %s", reason);
 
 	ospf->gr_info.restart_in_progress = false;
-	EVENT_OFF(ospf->gr_info.t_grace_period);
+	event_cancel(&ospf->gr_info.t_grace_period);
 
 	for (ALL_LIST_ELEMENTS_RO(ospf->areas, onode, area)) {
 		struct ospf_interface *oi;
@@ -241,7 +241,7 @@ static void ospf_gr_restart_exit(struct ospf *ospf, const char *reason)
 			/* Disable hello delay. */
 			if (oi->gr.hello_delay.t_grace_send) {
 				oi->gr.hello_delay.elapsed_seconds = 0;
-				EVENT_OFF(oi->gr.hello_delay.t_grace_send);
+				event_cancel(&oi->gr.hello_delay.t_grace_send);
 				OSPF_ISM_TIMER_MSEC_ON(oi->t_hello,
 						       ospf_hello_timer, 1);
 			}
