@@ -127,17 +127,17 @@ void ospf_nbr_free(struct ospf_neighbor *nbr)
 	}
 
 	/* Cancel all timers. */
-	EVENT_OFF(nbr->t_inactivity);
-	EVENT_OFF(nbr->t_db_desc);
-	EVENT_OFF(nbr->t_ls_req);
-	EVENT_OFF(nbr->t_ls_rxmt);
+	event_cancel(&nbr->t_inactivity);
+	event_cancel(&nbr->t_db_desc);
+	event_cancel(&nbr->t_ls_req);
+	event_cancel(&nbr->t_ls_rxmt);
 
 	/* Cancel all events. */ /* Thread lookup cost would be negligible. */
 	event_cancel_event(master, nbr);
 
 	bfd_sess_free(&nbr->bfd_session);
 
-	EVENT_OFF(nbr->gr_helper_info.t_grace_timer);
+	event_cancel(&nbr->gr_helper_info.t_grace_timer);
 
 	nbr->oi = NULL;
 	XFREE(MTYPE_OSPF_NEIGHBOR, nbr);
@@ -443,7 +443,7 @@ static struct ospf_neighbor *ospf_nbr_add(struct ospf_interface *oi,
 				nbr->nbr_nbma = nbr_nbma;
 
 				if (nbr_nbma->t_poll)
-					EVENT_OFF(nbr_nbma->t_poll);
+					event_cancel(&nbr_nbma->t_poll);
 
 				nbr->state_change = nbr_nbma->state_change + 1;
 			}
