@@ -1,20 +1,7 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * Multicast Traceroute for FRRouting
  * Copyright (C) 2018  Mladen Sablic
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
@@ -26,7 +13,6 @@
 #include "checksum.h"
 #include "prefix.h"
 #include "mtracebis_routeget.h"
-
 #include <sys/select.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -64,13 +50,14 @@ static void version(void)
 static void print_host(struct in_addr addr)
 {
 	struct hostent *h;
+	char buf[PREFIX_STRLEN];
 
 	h = gethostbyaddr(&addr, sizeof(addr), AF_INET);
 	if (h == NULL)
 		printf("?");
 	else
 		printf("%s", h->h_name);
-	printf(" (%s) ", inet_ntoa(addr));
+	printf(" (%s) ", inet_ntop(AF_INET, &addr, buf, sizeof(buf)));
 }
 
 static void print_line_no(int i)
