@@ -193,9 +193,9 @@ void pim_ifchannel_delete(struct pim_ifchannel *ch)
 
 	ch->upstream = NULL;
 
-	EVENT_OFF(ch->t_ifjoin_expiry_timer);
-	EVENT_OFF(ch->t_ifjoin_prune_pending_timer);
-	EVENT_OFF(ch->t_ifassert_timer);
+	event_cancel(&ch->t_ifjoin_expiry_timer);
+	event_cancel(&ch->t_ifjoin_prune_pending_timer);
+	event_cancel(&ch->t_ifassert_timer);
 
 	if (ch->parent) {
 		listnode_delete(ch->parent->sources, ch);
@@ -430,7 +430,7 @@ const char *pim_ifchannel_ifassert_name(enum pim_ifassert_state ifassert_state)
 */
 void reset_ifassert_state(struct pim_ifchannel *ch)
 {
-	EVENT_OFF(ch->t_ifassert_timer);
+	event_cancel(&ch->t_ifassert_timer);
 
 	pim_ifassert_winner_set(ch, PIM_IFASSERT_NOINFO, PIMADDR_ANY,
 				router->infinite_assert_metric);
@@ -578,6 +578,8 @@ struct pim_ifchannel *pim_ifchannel_add(struct interface *ifp, pim_sgaddr *sg,
 	ch->t_ifjoin_prune_pending_timer = NULL;
 	ch->ifjoin_creation = 0;
 
+	ch->prune_holdtime = 0;
+
 	RB_INSERT(pim_ifchannel_rb, &pim_ifp->ifchannel_rb, ch);
 
 	up = pim_upstream_add(pim_ifp->pim, sg, NULL, up_flags, __func__, ch);
@@ -938,13 +940,13 @@ void pim_ifchannel_join_add(struct interface *ifp, pim_addr neigh_addr,
 				return;
 			}
 		}
-		EVENT_OFF(ch->t_ifjoin_expiry_timer);
+		event_cancel(&ch->t_ifjoin_expiry_timer);
 		break;
 	case PIM_IFJOIN_PRUNE:
 		if (source_flags & PIM_ENCODE_RPT_BIT) {
 			pim_ifchannel_ifjoin_switch(__func__, ch,
 						    PIM_IFJOIN_NOINFO);
-			EVENT_OFF(ch->t_ifjoin_expiry_timer);
+			event_cancel(&ch->t_ifjoin_expiry_timer);
 			delete_on_noinfo(ch);
 			return;
 		} else
@@ -961,7 +963,7 @@ void pim_ifchannel_join_add(struct interface *ifp, pim_addr neigh_addr,
 		 * maximum of its current value and the HoldTime from the
 		 * triggering Join/Prune message.
 		 */
-		EVENT_OFF(ch->t_ifjoin_prune_pending_timer);
+		event_cancel(&ch->t_ifjoin_prune_pending_timer);
 
 		/* Check if SGRpt join Received */
 		if ((source_flags & PIM_ENCODE_RPT_BIT) &&
@@ -973,7 +975,7 @@ void pim_ifchannel_join_add(struct interface *ifp, pim_addr neigh_addr,
 			 * I transitions to the NoInfo state.The ET and PPT are
 			 * cancelled.
 			 */
-			EVENT_OFF(ch->t_ifjoin_expiry_timer);
+			event_cancel(&ch->t_ifjoin_expiry_timer);
 			pim_ifchannel_ifjoin_switch(__func__, ch,
 						    PIM_IFJOIN_NOINFO);
 			return;
@@ -988,7 +990,7 @@ void pim_ifchannel_join_add(struct interface *ifp, pim_addr neigh_addr,
 			if (remain > holdtime)
 				return;
 		}
-		EVENT_OFF(ch->t_ifjoin_expiry_timer);
+		event_cancel(&ch->t_ifjoin_expiry_timer);
 
 		break;
 	case PIM_IFJOIN_PRUNE_TMP:
@@ -1048,8 +1050,8 @@ void pim_ifchannel_prune(struct interface *ifp, pim_addr upstream,
 			   be taken not to use "ch" afterwards since it would be
 			   deleted. */
 
-			EVENT_OFF(ch->t_ifjoin_prune_pending_timer);
-			EVENT_OFF(ch->t_ifjoin_expiry_timer);
+			event_cancel(&ch->t_ifjoin_prune_pending_timer);
+			event_cancel(&ch->t_ifjoin_expiry_timer);
 			event_add_timer_msec(router->master,
 					     on_ifjoin_prune_pending_timer, ch,
 					     jp_override_interval_msec,
@@ -1086,7 +1088,7 @@ void pim_ifchannel_prune(struct interface *ifp, pim_addr upstream,
 		/* If we called ifjoin_prune() directly instead, care should
 		   be taken not to use "ch" afterwards since it would be
 		   deleted. */
-		EVENT_OFF(ch->t_ifjoin_prune_pending_timer);
+		event_cancel(&ch->t_ifjoin_prune_pending_timer);
 		event_add_timer_msec(router->master,
 				     on_ifjoin_prune_pending_timer, ch,
 				     jp_override_interval_msec,
@@ -1094,7 +1096,7 @@ void pim_ifchannel_prune(struct interface *ifp, pim_addr upstream,
 		break;
 	case PIM_IFJOIN_PRUNE:
 		if (source_flags & PIM_ENCODE_RPT_BIT) {
-			EVENT_OFF(ch->t_ifjoin_prune_pending_timer);
+			event_cancel(&ch->t_ifjoin_prune_pending_timer);
 			/*
 			 * While in Prune State, Receive SGRpt Prune.
 			 * RFC 7761 Sec 4.5.3:
@@ -1110,7 +1112,7 @@ void pim_ifchannel_prune(struct interface *ifp, pim_addr upstream,
 
 				if (rem > holdtime)
 					return;
-				EVENT_OFF(ch->t_ifjoin_expiry_timer);
+				event_cancel(&ch->t_ifjoin_expiry_timer);
 			}
 
 			event_add_timer(router->master, on_ifjoin_expiry_timer,
@@ -1121,7 +1123,7 @@ void pim_ifchannel_prune(struct interface *ifp, pim_addr upstream,
 	case PIM_IFJOIN_PRUNE_TMP:
 		if (source_flags & PIM_ENCODE_RPT_BIT) {
 			ch->ifjoin_state = PIM_IFJOIN_PRUNE;
-			EVENT_OFF(ch->t_ifjoin_expiry_timer);
+			event_cancel(&ch->t_ifjoin_expiry_timer);
 			event_add_timer(router->master, on_ifjoin_expiry_timer,
 					ch, holdtime,
 					&ch->t_ifjoin_expiry_timer);
@@ -1130,7 +1132,7 @@ void pim_ifchannel_prune(struct interface *ifp, pim_addr upstream,
 	case PIM_IFJOIN_PRUNE_PENDING_TMP:
 		if (source_flags & PIM_ENCODE_RPT_BIT) {
 			ch->ifjoin_state = PIM_IFJOIN_PRUNE_PENDING;
-			EVENT_OFF(ch->t_ifjoin_expiry_timer);
+			event_cancel(&ch->t_ifjoin_expiry_timer);
 			event_add_timer(router->master, on_ifjoin_expiry_timer,
 					ch, holdtime,
 					&ch->t_ifjoin_expiry_timer);
@@ -1484,8 +1486,8 @@ void pim_ifchannel_set_star_g_join_state(struct pim_ifchannel *ch, int eom,
 				break;
 
 			if (child->ifjoin_state == PIM_IFJOIN_PRUNE_PENDING_TMP)
-				EVENT_OFF(child->t_ifjoin_prune_pending_timer);
-			EVENT_OFF(child->t_ifjoin_expiry_timer);
+				event_cancel(&child->t_ifjoin_prune_pending_timer);
+			event_cancel(&child->t_ifjoin_expiry_timer);
 
 			PIM_IF_FLAG_UNSET_S_G_RPT(child->flags);
 			child->ifjoin_state = PIM_IFJOIN_NOINFO;
