@@ -10,6 +10,7 @@
 #include <zebra.h>
 
 #include "if.h"
+#include "pim_instance.h"
 
 #define PIM_PIM_BUFSIZE_READ  (20000)
 #define PIM_PIM_BUFSIZE_WRITE (20000)
@@ -21,6 +22,8 @@
 #define PIM_DEFAULT_OVERRIDE_INTERVAL_MSEC       (2500) /* RFC 4601: 4.11.  Timer Values */
 #define PIM_DEFAULT_CAN_DISABLE_JOIN_SUPPRESSION (0)    /* boolean */
 #define PIM_DEFAULT_T_PERIODIC                   (60)   /* RFC 4601: 4.11.  Timer Values */
+#define PIM_DEFAULT_T_PRUNE_LIMIT		 (210)	/* RFC 3973 4.8 PIM-DM Timers*/
+
 
 enum pim_msg_type {
 	PIM_MSG_TYPE_HELLO = 0,
@@ -31,7 +34,8 @@ enum pim_msg_type {
 	PIM_MSG_TYPE_ASSERT,
 	PIM_MSG_TYPE_GRAFT,
 	PIM_MSG_TYPE_GRAFT_ACK,
-	PIM_MSG_TYPE_CANDIDATE
+	PIM_MSG_TYPE_CANDIDATE,
+	PIM_MSG_TYPE_STATE_REFRESH
 };
 
 void pim_ifstat_reset(struct interface *ifp);
@@ -42,10 +46,12 @@ void pim_hello_restart_now(struct interface *ifp);
 void pim_hello_restart_triggered(struct interface *ifp);
 
 int pim_pim_packet(struct interface *ifp, uint8_t *buf, size_t len,
-		   pim_sgaddr sg);
+		   pim_sgaddr sg, bool is_mcast);
 
 int pim_msg_send(int fd, pim_addr src, pim_addr dst, uint8_t *pim_msg,
 		 int pim_msg_size, struct interface *ifp);
 
 int pim_hello_send(struct interface *ifp, uint16_t holdtime);
+
+int pim_sock_read_helper(int fd, struct pim_instance *pim, bool is_mcast);
 #endif /* PIM_PIM_H */
