@@ -539,12 +539,12 @@ void cli_show_rip_route(struct vty *vty, const struct lyd_node *dnode,
  */
 DEFPY_YANG (rip_timers,
        rip_timers_cmd,
-       "timers basic (5-2147483647)$update (5-2147483647)$timeout (5-2147483647)$garbage",
+       "timers basic (1-32767)$update (1-32767)$timeout (1-32767)$garbage",
        "Adjust routing timers\n"
        "Basic routing protocol update timers\n"
        "Routing table update timer value in second. Default is 30.\n"
        "Routing information timeout timer. Default is 180.\n"
-       "Garbage collection timer. Default is 120.\n")
+       "Garbage collection timer. Default is 240.\n")
 {
 	nb_cli_enqueue_change(vty, "./update-interval", NB_OP_MODIFY,
 			      update_str);
@@ -558,13 +558,13 @@ DEFPY_YANG (rip_timers,
 
 DEFPY_YANG (no_rip_timers,
        no_rip_timers_cmd,
-       "no timers basic [(5-2147483647) (5-2147483647) (5-2147483647)]",
+       "no timers basic [(1-32767) (1-32767) (1-32767)]",
        NO_STR
        "Adjust routing timers\n"
        "Basic routing protocol update timers\n"
        "Routing table update timer value in second. Default is 30.\n"
        "Routing information timeout timer. Default is 180.\n"
-       "Garbage collection timer. Default is 120.\n")
+       "Garbage collection timer. Default is 240.\n")
 {
 	nb_cli_enqueue_change(vty, "./update-interval", NB_OP_MODIFY, NULL);
 	nb_cli_enqueue_change(vty, "./holddown-interval", NB_OP_MODIFY, NULL);
@@ -681,9 +681,9 @@ DEFPY_YANG (ip_rip_split_horizon,
 {
 	const char *value;
 
-	if (no)
+	if (no && poisoned_reverse == NULL)
 		value = "disabled";
-	else if (poisoned_reverse)
+	else if (poisoned_reverse && no == NULL)
 		value = "poison-reverse";
 	else
 		value = "simple";
@@ -1233,6 +1233,23 @@ DEFPY_YANG(no_rip_distribute_list_prefix,
 	return nb_cli_apply_changes(vty, NULL);
 }
 
+/*
+ * XPath: /frr-ripd:clear-rip-route
+ */
+DEFPY_YANG (clear_ip_rip,
+       clear_ip_rip_cmd,
+       "clear ip rip [vrf WORD]",
+       CLEAR_STR
+       IP_STR
+       "Clear IP RIP database\n"
+       VRF_CMD_HELP_STR)
+{
+	if (vrf)
+		nb_cli_rpc_enqueue(vty, "vrf", vrf);
+
+	return nb_cli_rpc(vty, "/frr-ripd:clear-rip-route", NULL);
+}
+
 /* RIP node structure. */
 static struct cmd_node rip_node = {
 	.name = "rip",
@@ -1295,6 +1312,8 @@ void rip_cli_init(void)
 	install_element(INTERFACE_NODE, &ip_rip_bfd_profile_cmd);
 	install_element(INTERFACE_NODE, &no_ip_rip_bfd_profile_cmd);
 
+	install_element(ENABLE_NODE, &clear_ip_rip_cmd);
+
 	if_rmap_init(RIP_NODE);
 }
 /* clang-format off */
@@ -1355,6 +1374,22 @@ const struct frr_yang_module_info frr_ripd_cli_info = {
 			.xpath = "/frr-ripd:ripd/instance/non-passive-interface",
 			.cbs.cli_show = cli_show_rip_non_passive_interface,
 		},
+		{
+			.xpath = "/frr-ripd:ripd/instance/distribute-list/in/access-list",
+			.cbs.cli_show = group_distribute_list_ipv4_cli_show,
+		},
+		{
+			.xpath = "/frr-ripd:ripd/instance/distribute-list/out/access-list",
+			.cbs.cli_show = group_distribute_list_ipv4_cli_show,
+		},
+		{
+			.xpath = "/frr-ripd:ripd/instance/distribute-list/in/prefix-list",
+			.cbs.cli_show = group_distribute_list_ipv4_cli_show,
+		},
+		{
+			.xpath = "/frr-ripd:ripd/instance/distribute-list/out/prefix-list",
+			.cbs.cli_show = group_distribute_list_ipv4_cli_show,
+		},
 		{
 			.xpath = "/frr-ripd:ripd/instance/redistribute",
 			.cbs.cli_show = cli_show_rip_redistribute,
