@@ -383,7 +383,7 @@ static void rip_interface_clean(struct rip_interface *ri)
 	ri->enable_interface = 0;
 	ri->running = 0;
 
-	EVENT_OFF(ri->t_wakeup);
+	event_cancel(&ri->t_wakeup);
 }
 
 void rip_interfaces_clean(struct rip *rip)
@@ -443,7 +443,7 @@ int rip_if_down(struct interface *ifp)
 
 	ri = ifp->info;
 
-	EVENT_OFF(ri->t_wakeup);
+	event_cancel(&ri->t_wakeup);
 
 	rip = ri->rip;
 	if (rip) {
