@@ -1,30 +1,18 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /*
  * testing log message generator
  * Copyright (C) 2019-2020  David Lamparter for NetDEF, Inc.
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <zebra.h>
+#include <sys/resource.h>
 
 #include "vty.h"
 #include "command.h"
 #include "prefix.h"
 #include "nexthop.h"
 #include "log.h"
-#include "thread.h"
+#include "frrevent.h"
 #include "vrf.h"
 #include "zclient.h"
 #include "frr_pthread.h"
@@ -41,9 +29,9 @@ static size_t lp_ctr, lp_expect;
 static struct rusage lp_rusage;
 static struct vty *lp_vty;
 
-extern struct thread_master *master;
+extern struct event_loop *master;
 
-static void logpump_done(struct thread *thread)
+static void logpump_done(struct event *thread)
 {
 	double x;
 
@@ -118,7 +106,7 @@ static void *logpump_run(void *arg)
 	getrusage(RUSAGE_SELF, &lp_rusage);
 #endif
 
-	thread_add_timer_msec(master, logpump_done, NULL, 0, NULL);
+	event_add_timer_msec(master, logpump_done, NULL, 0, NULL);
 	return NULL;
 }
 
