@@ -1,6 +1,9 @@
+# SPDX-License-Identifier: ISC
 import frrtest
 
+
 class TestPrintfrr(frrtest.TestMultiOut):
-    program = './test_printfrr'
+    program = "./test_printfrr"
+
 
 TestPrintfrr.exit_cleanly()
