@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+# SPDX-License-Identifier: ISC
 
 #
 # test_bgp_conditional_advertisement.py
@@ -7,20 +8,6 @@
 # Samsung R&D Institute India - Bangalore.
 # Madhurilatha Kuruganti
 #
-# Permission to use, copy, modify, and/or distribute this software
-# for any purpose with or without fee is hereby granted, provided
-# that the above copyright notice and this permission notice appear
-# in all copies.
-#
-# THE SOFTWARE IS PROVIDED "AS IS" AND NETDEF DISCLAIMS ALL WARRANTIES
-# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
-# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL NETDEF BE LIABLE FOR
-# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
-# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
-# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
-# OF THIS SOFTWARE.
-#
 
 """
 Test BGP conditional advertisement functionality.
@@ -172,7 +159,7 @@ def setup_module(mod):
 
     router_list = tgen.routers()
 
-    for i, (rname, router) in enumerate(router_list.items(), 1):
+    for _, (rname, router) in enumerate(router_list.items(), 1):
         router.load_config(
             TopoRouter.RD_ZEBRA, os.path.join(CWD, "{}/zebra.conf".format(rname))
         )
@@ -439,7 +426,7 @@ def test_bgp_conditional_advertisement_tc_2_1():
     )
 
     test_func = functools.partial(exist_map_routes_present, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = 'TC21: exist-map routes present in "router2" BGP table - '
     assert result is None, msg + failed
@@ -468,7 +455,7 @@ def test_bgp_conditional_advertisement_tc_2_2():
     )
 
     test_func = functools.partial(exist_map_routes_not_present, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = 'TC22: exist-map routes not present in "router2" BGP table - '
     assert result is None, msg + failed
@@ -497,7 +484,7 @@ def test_bgp_conditional_advertisement_tc_2_3():
     )
 
     test_func = functools.partial(default_route_withdrawn, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC23: advertise-map with exist-map configuration is removed from peer - "
     assert result is None, msg + failed
@@ -526,7 +513,7 @@ def test_bgp_conditional_advertisement_tc_3_1():
     )
 
     test_func = functools.partial(non_exist_map_routes_not_present, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = 'TC31: non-exist-map routes not present in "router2" BGP table - '
     assert result is None, msg + failed
@@ -555,7 +542,7 @@ def test_bgp_conditional_advertisement_tc_3_2():
     )
 
     test_func = functools.partial(non_exist_map_routes_present, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = 'TC32: non-exist-map routes present in "router2" BGP table - '
     assert result is None, msg + failed
@@ -584,7 +571,7 @@ def test_bgp_conditional_advertisement_tc_3_3():
     )
 
     test_func = functools.partial(all_routes_advertised, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = (
         "TC33: advertise-map with non-exist-map configuration is removed from a peer - "
@@ -616,7 +603,7 @@ def test_bgp_conditional_advertisement_tc_4_1():
     )
 
     test_func = functools.partial(non_exist_map_no_condition_route_map, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = 'TC41: non-exist-map route-map removed in "router2" - '
     assert result is None, msg + failed
@@ -645,7 +632,7 @@ def test_bgp_conditional_advertisement_tc_4_2():
     )
 
     test_func = functools.partial(exist_map_no_condition_route_map, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = 'TC42: exist-map route-map removed in "router2" - '
     assert result is None, msg + failed
@@ -678,7 +665,7 @@ def test_bgp_conditional_advertisement_tc_5_1():
     )
 
     test_func = functools.partial(exist_map_routes_present_rmap_filter, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC51: exist-map routes present with route-map filter - "
     assert result is None, msg + failed
@@ -707,7 +694,7 @@ def test_bgp_conditional_advertisement_tc_5_2():
     )
 
     test_func = functools.partial(exist_map_routes_present_no_rmap_filter, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC52: exist-map routes present, no route-map filter - "
     assert result is None, msg + failed
@@ -737,7 +724,7 @@ def test_bgp_conditional_advertisement_tc_5_3():
     )
 
     test_func = functools.partial(non_exist_map_routes_present_rmap_filter, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC53: non-exist-map routes present, with route-map filter - "
     assert result is None, msg + failed
@@ -766,7 +753,7 @@ def test_bgp_conditional_advertisement_tc_5_4():
     )
 
     test_func = functools.partial(non_exist_map_routes_present_no_rmap_filter, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC54: non-exist-map routes present, no route-map filter - "
     assert result is None, msg + failed
@@ -804,7 +791,7 @@ def test_bgp_conditional_advertisement_tc_6_1():
     )
 
     test_func = functools.partial(exist_map_routes_not_present_rmap_filter, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC61: exist-map routes not present, route-map filter - "
     assert result is None, msg + failed
@@ -833,7 +820,7 @@ def test_bgp_conditional_advertisement_tc_6_2():
     )
 
     test_func = functools.partial(exist_map_routes_not_present_no_rmap_filter, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC62: exist-map routes not present, no route-map filter - "
     assert result is None, msg + failed
@@ -863,7 +850,7 @@ def test_bgp_conditional_advertisement_tc_6_3():
     )
 
     test_func = functools.partial(non_exist_map_routes_not_present_rmap_filter, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC63: non-exist-map routes not present, route-map filter - "
     assert result is None, msg + failed
@@ -894,7 +881,7 @@ def test_bgp_conditional_advertisement_tc_6_4():
     test_func = functools.partial(
         non_exist_map_routes_not_present_no_rmap_filter, router3
     )
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC64: non-exist-map routes not present, no route-map filter - "
     assert result is None, msg + failed
@@ -932,7 +919,7 @@ def test_bgp_conditional_advertisement_tc_7_1():
     )
 
     test_func = functools.partial(exist_map_routes_present_rmap2_filter, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC71: exist-map routes present, route-map filter - "
     assert result is None, msg + failed
@@ -961,7 +948,7 @@ def test_bgp_conditional_advertisement_tc_7_2():
     )
 
     test_func = functools.partial(exist_map_routes_present_no_rmap2_filter, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC72: exist-map routes present, no route-map filter - "
     assert result is None, msg + failed
@@ -991,7 +978,7 @@ def test_bgp_conditional_advertisement_tc_7_3():
     )
 
     test_func = functools.partial(non_exist_map_routes_present_rmap2_filter, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC73: non-exist-map routes present, route-map filter - "
     assert result is None, msg + failed
@@ -1020,7 +1007,7 @@ def test_bgp_conditional_advertisement_tc_7_4():
     )
 
     test_func = functools.partial(non_exist_map_routes_present_no_rmap2_filter, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC74: non-exist-map routes present, no route-map filter - "
     assert result is None, msg + failed
@@ -1058,7 +1045,7 @@ def test_bgp_conditional_advertisement_tc_8_1():
     )
 
     test_func = functools.partial(exist_map_routes_not_present_rmap2_filter, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC81: exist-map routes not present, route-map filter - "
     assert result is None, msg + failed
@@ -1087,7 +1074,7 @@ def test_bgp_conditional_advertisement_tc_8_2():
     )
 
     test_func = functools.partial(exist_map_routes_not_present_no_rmap2_filter, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC82: exist-map routes not present, no route-map filter - "
     assert result is None, msg + failed
@@ -1119,7 +1106,7 @@ def test_bgp_conditional_advertisement_tc_8_3():
     test_func = functools.partial(
         non_exist_map_routes_not_present_rmap2_filter, router3
     )
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC83: non-exist-map routes not present, route-map filter - "
     assert result is None, msg + failed
@@ -1150,7 +1137,7 @@ def test_bgp_conditional_advertisement_tc_8_4():
     test_func = functools.partial(
         non_exist_map_routes_not_present_no_rmap2_filter, router3
     )
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC84: non-exist-map routes not present, no route-map filter - "
     assert result is None, msg + failed
@@ -1188,7 +1175,7 @@ def test_bgp_conditional_advertisement_tc_9_1():
     )
 
     test_func = functools.partial(exist_map_routes_present_rmap2_network, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC91: exist-map routes present, route-map filter and network - "
     assert result is None, msg + failed
@@ -1217,7 +1204,7 @@ def test_bgp_conditional_advertisement_tc_9_2():
     )
 
     test_func = functools.partial(exist_map_routes_present_rmap2_no_network, router3)
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC92: exist-map routes present, route-map filter and no network - "
     assert result is None, msg + failed
@@ -1257,7 +1244,7 @@ def test_bgp_conditional_advertisement_tc_9_3():
     test_func = functools.partial(
         non_exist_map_routes_not_present_rmap2_network, router3
     )
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC93: non-exist-map routes not present, route-map filter and network - "
     assert result is None, msg + failed
@@ -1288,7 +1275,7 @@ def test_bgp_conditional_advertisement_tc_9_4():
     test_func = functools.partial(
         non_exist_map_routes_not_present_rmap2_no_network, router3
     )
-    success, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
+    _, result = topotest.run_and_expect(test_func, None, count=90, wait=1)
 
     msg = "TC94: non-exist-map routes not present, route-map filter and no network - "
     assert result is None, msg + failed
