@@ -1,23 +1,11 @@
 #!/usr/bin/env python
+# SPDX-License-Identifier: ISC
 #
 # Copyright (c) 2022 by VMware, Inc. ("VMware")
+#                       Shreenidhi A R <rshreenidhi@vmware.com>
 # Used Copyright (c) 2018 by Network Device Education Foundation, Inc. ("NetDEF")
 # in this file.
 #
-# Permission to use, copy, modify, and/or distribute this software
-# for any purpose with or without fee is hereby granted, provided
-# that the above copyright notice and this permission notice appear
-# in all copies.
-# Shreenidhi A R <rshreenidhi@vmware.com>
-# THE SOFTWARE IS PROVIDED "AS IS" AND VMWARE DISCLAIMS ALL WARRANTIES
-# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
-# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VMWARE BE LIABLE FOR
-# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
-# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
-# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
-# OF THIS SOFTWARE.
-#
 """
 Following tests are covered.
 1. Verify default-originate route with default static and network command
@@ -29,14 +17,15 @@ import sys
 import time
 import pytest
 import datetime
-from copy import deepcopy
 from lib.topolog import logger
+from time import sleep
 
 # pylint: disable=C0413
 # Import topogen and topotest helpers
 from lib.topogen import Topogen, get_topogen
 from lib.topojson import build_config_from_json
 from lib.topolog import logger
+from lib import topotest
 
 from lib.bgp import (
     verify_bgp_convergence,
@@ -224,7 +213,6 @@ def get_rib_route_uptime(tgen, addr_type, dut, input_dict):
 
     logger.info("Entering lib API: get_rib_route_uptime()")
     route_time = []
-    out_route_dict = {}
     router_list = tgen.routers()
     for routerInput in input_dict.keys():
         for router, rnode in router_list.items():
@@ -244,7 +232,6 @@ def get_rib_route_uptime(tgen, addr_type, dut, input_dict):
 
                 for static_route in static_routes:
                     if "vrf" in static_route and static_route["vrf"] is not None:
-
                         logger.info(
                             "[DUT: {}]: Verifying routes for VRF:"
                             " {}".format(router, static_route["vrf"])
@@ -279,21 +266,21 @@ def verify_the_uptime(time_stamp_before, time_stamp_after, incremented=None):
     if incremented == True:
         if uptime_before < uptime_after:
             logger.info(
-                "  The Uptime [{}] is incremented than [{}].......PASSED ".format(
+                "  The Uptime before [{}] is less than [{}].......PASSED ".format(
                     time_stamp_before, time_stamp_after
                 )
             )
             return True
         else:
             logger.error(
-                "  The Uptime [{}] is expected to be incremented than [{}].......FAILED ".format(
+                "  The Uptime before [{}] is greater than the uptime after [{}].......FAILED ".format(
                     time_stamp_before, time_stamp_after
                 )
             )
             return False
     else:
         logger.info(
-            "  The Uptime [{}] is not incremented than [{}] ".format(
+            "  The Uptime before [{}] the same as after [{}] ".format(
                 time_stamp_before, time_stamp_after
             )
         )
@@ -317,7 +304,6 @@ def get_best_path_route_in_FIB(tgen, topo, dut, network):
     on failure : return error message with boolean False
     """
     is_ipv4_best_path_found = False
-    is_ipv6_best_path_found = False
     rnode = tgen.routers()[dut]
     ipv4_show_bgp_json = run_frr_cmd(rnode, "sh ip bgp  json ", isjson=True)
     ipv6_show_bgp_json = run_frr_cmd(
@@ -604,6 +590,7 @@ def test_verify_bgp_default_originate_with_default_static_route_p1(request):
     step("Taking uptime snapshot before configuring default - originate")
     uptime_before_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
     uptime_before_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)
+    sleep(1)
 
     step(
         "Configure default-originate on R1 link-1 again for IPv4 and IPv6 address family"
@@ -1040,9 +1027,10 @@ def test_verify_bgp_default_originate_with_default_static_route_p1(request):
     result = verify_the_uptime(uptime_before_ipv6, uptime_after_ipv6, incremented=False)
     assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
 
-    step("Taking uptime snapshot before  removing   redisctribute static ")
+    step("Taking uptime snapshot before removing redistribute static")
     uptime_before_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
     uptime_before_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)
+    sleep(1)
 
     step("Remove redistribute static from IPv4 and IPv6 address family ")
     input_dict_1 = {
@@ -1086,6 +1074,7 @@ def test_verify_bgp_default_originate_with_default_static_route_p1(request):
     )
     assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
 
+    step("Now look that the route is not pointed at link2")
     result = verify_rib_default_route(
         tgen,
         topo,
@@ -1105,7 +1094,7 @@ def test_verify_bgp_default_originate_with_default_static_route_p1(request):
     )
     assert result is not True, "Testcase {} : Failed Error: {}".format(tc_name, result)
 
-    step("Taking uptime snapshot before  removing   redisctribute static ")
+    step("Taking uptime snapshot after removing redistribute static")
     uptime_after_ipv4 = get_rib_route_uptime(tgen, "ipv4", "r2", ipv4_uptime_dict)
     uptime_after_ipv6 = get_rib_route_uptime(tgen, "ipv6", "r2", ipv6_uptime_dict)
 
@@ -1568,8 +1557,14 @@ def test_verify_default_originate_with_2way_ecmp_p2(request):
     step("Ping R1 configure IPv4 and IPv6 loopback address from R2")
     pingaddr = topo["routers"]["r1"]["links"]["lo"]["ipv4"].split("/")[0]
     router = tgen.gears["r2"]
-    output = router.run("ping -c 4 -w 4 {}".format(pingaddr))
-    assert " 0% packet loss" in output, "Ping R1->R2  FAILED"
+
+    def ping_router():
+        output = router.run("ping -c 4 -w 4 {}".format(pingaddr))
+        logger.info(output)
+        if " 0% packet loss" not in output:
+            return False
+
+    _, res = topotest.run_and_expect(ping_router, None, count=10, wait=1)
     logger.info("Ping from R1 to R2 ... success")
 
     step("Shuting up the active route")
@@ -1577,7 +1572,7 @@ def test_verify_default_originate_with_2way_ecmp_p2(request):
     ipv_dict = get_best_path_route_in_FIB(tgen, topo, dut="r2", network=network)
     dut_links = topo["routers"]["r1"]["links"]
     active_interface = None
-    for key, values in dut_links.items():
+    for key, _ in dut_links.items():
         ipv4_address = dut_links[key]["ipv4"].split("/")[0]
         ipv6_address = dut_links[key]["ipv6"].split("/")[0]
         if ipv_dict["ipv4"] == ipv4_address and ipv_dict["ipv6"] == ipv6_address:
