@@ -10,8 +10,6 @@ import os
 import sys
 import time
 import pytest
-from time import sleep
-from copy import deepcopy
 from lib.topolog import logger
 
 # pylint: disable=C0413
@@ -22,32 +20,17 @@ from lib.topolog import logger
 
 from lib.bgp import (
     verify_bgp_convergence,
-    verify_graceful_restart,
     create_router_bgp,
-    verify_router_id,
     modify_as_number,
-    verify_as_numbers,
-    clear_bgp_and_verify,
-    clear_bgp,
     verify_bgp_rib,
     get_prefix_count_route,
     get_dut_as_number,
-    verify_rib_default_route,
-    verify_fib_default_route,
-    verify_bgp_advertised_routes_from_neighbor,
-    verify_bgp_received_routes_from_neighbor,
 )
 from lib.common_config import (
-    interface_status,
     verify_prefix_lists,
     verify_rib,
-    kill_router_daemons,
-    start_router_daemons,
-    shutdown_bringup_interface,
     step,
     required_linux_kernel_version,
-    stop_router,
-    start_router,
     create_route_maps,
     create_prefix_lists,
     get_frr_ipv6_linklocal,
@@ -58,7 +41,6 @@ from lib.common_config import (
     reset_config_on_routers,
     create_static_routes,
     check_router_status,
-    delete_route_maps,
 )
 
 pytestmark = [pytest.mark.bgpd, pytest.mark.staticd]
@@ -564,13 +546,7 @@ def test_verify_default_originate_route_with_non_default_VRF_p1(request):
             tc_name, result
         )
 
-        result = verify_rib(
-            tgen,
-            addr_type,
-            "r2",
-            static_routes_input,
-            next_hop=DEFAULT_ROUTE_NXT_HOP_R1[addr_type],
-        )
+        result = verify_rib(tgen, addr_type, "r2", static_routes_input)
         assert result is True, "Testcase {} : Failed \n Error: {}".format(
             tc_name, result
         )
