@@ -19,7 +19,6 @@ import os
 import sys
 import time
 import pytest
-from copy import deepcopy
 from lib.topolog import logger
 
 # pylint: disable=C0413
@@ -75,6 +74,7 @@ NETWORK1_1 = {"ipv4": "198.51.1.1/32", "ipv6": "2001:DB8::1:1/128"}
 DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "0::0/0"}
 NEXT_HOP_IP = {"ipv4": "Null0", "ipv6": "Null0"}
 
+
 def setup_module(mod):
     """
     Sets up the pytest environment
@@ -818,11 +818,11 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
 
     step("Configure IPv4 and IPv6 , EBGP neighbor between R3 and R2")
     step("Configure IPv4 and IPv6 IBGP neighbor between R3 and R4")
-    r0_local_as = topo['routers']['r0']['bgp']['local_as']
-    r1_local_as = topo['routers']['r1']['bgp']['local_as']
-    r2_local_as = topo['routers']['r2']['bgp']['local_as']
-    r3_local_as = topo['routers']['r3']['bgp']['local_as']
-    r4_local_as = topo['routers']['r4']['bgp']['local_as']
+    r0_local_as = topo["routers"]["r0"]["bgp"]["local_as"]
+    r1_local_as = topo["routers"]["r1"]["bgp"]["local_as"]
+    r2_local_as = topo["routers"]["r2"]["bgp"]["local_as"]
+    r3_local_as = topo["routers"]["r3"]["bgp"]["local_as"]
+    r4_local_as = topo["routers"]["r4"]["bgp"]["local_as"]
     input_dict = {
         "r0": {
             "bgp": {
@@ -1026,22 +1026,21 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
                         "action": "permit",
                         "seq_id": "1",
                         "set": {
-                                "path": {
-                                    "as_num": "200",
-                                    "as_action": "prepend",
-                                }
+                            "path": {
+                                "as_num": "200",
+                                "as_action": "prepend",
                             }
-
+                        },
                     },
                     {
                         "action": "permit",
                         "seq_id": "2",
                         "set": {
-                                "path": {
-                                    "as_num": "300",
-                                    "as_action": "prepend",
-                                }
+                            "path": {
+                                "as_num": "300",
+                                "as_action": "prepend",
                             }
+                        },
                     },
                 ],
                 "RMv6": [
@@ -1049,21 +1048,21 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
                         "action": "permit",
                         "seq_id": "1",
                         "set": {
-                                "path": {
-                                    "as_num": "200",
-                                    "as_action": "prepend",
-                                }
+                            "path": {
+                                "as_num": "200",
+                                "as_action": "prepend",
                             }
+                        },
                     },
                     {
                         "action": "permit",
                         "seq_id": "2",
                         "set": {
-                                "path": {
-                                    "as_num": "300",
-                                    "as_action": "prepend",
-                                }
+                            "path": {
+                                "as_num": "300",
+                                "as_action": "prepend",
                             }
+                        },
                     },
                 ],
             }
@@ -1122,22 +1121,21 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
                         "action": "permit",
                         "seq_id": "1",
                         "set": {
-                                "path": {
-                                    "as_num": "500",
-                                    "as_action": "prepend",
-                                }
+                            "path": {
+                                "as_num": "500",
+                                "as_action": "prepend",
                             }
-
+                        },
                     },
                     {
                         "action": "permit",
                         "seq_id": "2",
                         "set": {
-                                "path": {
-                                    "as_num": "600",
-                                    "as_action": "prepend",
-                                }
+                            "path": {
+                                "as_num": "600",
+                                "as_action": "prepend",
                             }
+                        },
                     },
                 ],
                 "RMv6": [
@@ -1145,21 +1143,21 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
                         "action": "permit",
                         "seq_id": "1",
                         "set": {
-                                "path": {
-                                    "as_num": "500",
-                                    "as_action": "prepend",
-                                }
+                            "path": {
+                                "as_num": "500",
+                                "as_action": "prepend",
                             }
+                        },
                     },
                     {
                         "action": "permit",
                         "seq_id": "2",
                         "set": {
-                                "path": {
-                                    "as_num": "600",
-                                    "as_action": "prepend",
-                                }
+                            "path": {
+                                "as_num": "600",
+                                "as_action": "prepend",
                             }
+                        },
                     },
                 ],
             }
@@ -1170,7 +1168,6 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
     assert result is True, "Test case {} : Failed \n Error: {}".format(tc_name, result)
     assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
 
-
     step("As path 500 added to IPv4 and IPv6 default -originate route received on R2")
     result = verify_rib_default_route(
         tgen,
@@ -1232,7 +1229,6 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         "Verify Configured metric value received on R2 along with as-path for IPv4 and IPv6 default routes "
     )
 
-
     DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "::/0"}
     result = verify_rib_default_route(
         tgen,
@@ -1244,7 +1240,6 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         expected_aspath="4000 500",
     )
 
-
     step(
         "Modify route-map seq1 configure metric 50 and route-map seq2 configure metric 100 IPv4 and IPv6 route-map "
     )
@@ -1294,7 +1289,6 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         "Verify Configured metric value received on R2 along with as-path for IPv4 and IPv6 default routes "
     )
 
-
     result = verify_rib_default_route(
         tgen,
         topo,
@@ -1314,7 +1308,6 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
                     {
                         "action": "permit",
                         "seq_id": "1",
-
                         "set": {
                             "path": {
                                 "as_num": "500",
@@ -1374,9 +1367,6 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         "Verify AS-prepend is deleted from default originate route and metric value only present on R2 for IPv4 and IPv6 default routes "
     )
 
-
-
-
     result = verify_rib_default_route(
         tgen,
         topo,
@@ -1388,7 +1378,6 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
     )
     assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
 
-
     step("Delete metric value  from IP4 and IPv6 route-map configured on R3 ")
     route_map = {
         "r3": {
@@ -1428,8 +1417,6 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         "Verify Metric value deleted from IPv4 and IPv6 default route on R2 ,verify  default routes "
     )
 
-
-
     result = verify_rib_default_route(
         tgen,
         topo,
@@ -1443,11 +1430,11 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
     assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
     step("Change  IPv4 and IPv6 , EBGP to IBGP neighbor between R3 and R2")
     step("Change IPv4 and IPv6 IBGP to EBGP neighbor between R3 and R4")
-    r0_local_as = topo['routers']['r0']['bgp']['local_as']
-    r1_local_as = topo['routers']['r1']['bgp']['local_as']
-    r2_local_as = topo['routers']['r2']['bgp']['local_as']
-    r3_local_as = topo['routers']['r3']['bgp']['local_as']
-    r4_local_as = topo['routers']['r4']['bgp']['local_as']
+    r0_local_as = topo["routers"]["r0"]["bgp"]["local_as"]
+    r1_local_as = topo["routers"]["r1"]["bgp"]["local_as"]
+    r2_local_as = topo["routers"]["r2"]["bgp"]["local_as"]
+    r3_local_as = topo["routers"]["r3"]["bgp"]["local_as"]
+    r4_local_as = topo["routers"]["r4"]["bgp"]["local_as"]
     input_dict = {
         "r0": {
             "bgp": {
@@ -1459,7 +1446,6 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
                 "local_as": r1_local_as,
             }
         },
-
         "r2": {
             "bgp": {
                 "local_as": 1111,
@@ -1645,8 +1631,6 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         "Verify Configured metric value received on R2 along with as-path for IPv4 and IPv6 default routes "
     )
 
-
-
     result = verify_rib_default_route(
         tgen,
         topo,
@@ -1656,7 +1640,6 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         locPrf=50,
     )
 
-
     assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
 
     step(
@@ -1708,9 +1691,6 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         "Verify Modified local-preference  value received on R2  for IPv4 and IPv6 default routes "
     )
 
-
-
-
     DEFAULT_ROUTES = {"ipv4": "0.0.0.0/0", "ipv6": "::/0"}
     result = verify_rib_default_route(
         tgen,
@@ -1724,13 +1704,15 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
     assert result is True, "Testcase {} : Failed Error: {}".format(tc_name, result)
     # updating the topology with the updated AS-Number to avoid conflict in con configuring the AS
     updated_topo = topo
-    updated_topo['routers']['r0']['bgp']['local_as']=get_dut_as_number(tgen,"r0")
-    updated_topo['routers']['r1']['bgp']['local_as']=get_dut_as_number(tgen,"r1")
-    updated_topo['routers']['r2']['bgp']['local_as']=get_dut_as_number(tgen,"r2")
-    updated_topo['routers']['r3']['bgp']['local_as']=get_dut_as_number(tgen,"r3")
-    updated_topo['routers']['r4']['bgp']['local_as']=get_dut_as_number(tgen,"r4")
+    updated_topo["routers"]["r0"]["bgp"]["local_as"] = get_dut_as_number(tgen, "r0")
+    updated_topo["routers"]["r1"]["bgp"]["local_as"] = get_dut_as_number(tgen, "r1")
+    updated_topo["routers"]["r2"]["bgp"]["local_as"] = get_dut_as_number(tgen, "r2")
+    updated_topo["routers"]["r3"]["bgp"]["local_as"] = get_dut_as_number(tgen, "r3")
+    updated_topo["routers"]["r4"]["bgp"]["local_as"] = get_dut_as_number(tgen, "r4")
 
-    step("Shut IPv4/IPv6 BGP neighbor from R4 ( R4-R3) using 'neighbor x.x.x.x shut' command ")
+    step(
+        "Shut IPv4/IPv6 BGP neighbor from R4 ( R4-R3) using 'neighbor x.x.x.x shut' command "
+    )
     local_as = get_dut_as_number(tgen, dut="r4")
     shut_neighbor = {
         "r4": {
@@ -1740,46 +1722,41 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
                     "ipv4": {
                         "unicast": {
                             "neighbor": {
-                                "r3": {
-                                    "dest_link": {
-                                        "r4": {"shutdown":True}
-                                    }
-                                }
+                                "r3": {"dest_link": {"r4": {"shutdown": True}}}
                             }
                         }
                     },
                     "ipv6": {
                         "unicast": {
                             "neighbor": {
-                                "r3": {
-                                    "dest_link": {
-                                        "r4": {"shutdown":True}
-                                    }
-                                }
+                                "r3": {"dest_link": {"r4": {"shutdown": True}}}
                             }
                         }
-                    }
-                }
+                    },
+                },
             }
         }
     }
     result = create_router_bgp(tgen, updated_topo, shut_neighbor)
     assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)
 
-    interface = topo['routers']['r3']['links']['r4']['interface']
-    input_dict = {
-            "r1": {
-                "interface_list": [interface],
-                "status": "down"
-            }
-        }
+    interface = topo["routers"]["r3"]["links"]["r4"]["interface"]
+    input_dict = {"r1": {"interface_list": [interface], "status": "down"}}
 
     result = interface_status(tgen, topo, input_dict)
-    assert result is True, "Testcase {} : Shut down the interface failed ! \n Error: {}".format(tc_name, result)
+    assert (
+        result is True
+    ), "Testcase {} : Shut down the interface failed ! \n Error: {}".format(
+        tc_name, result
+    )
 
     step("After shutting the interface verify the BGP convergence")
-    result = verify_bgp_convergence(tgen,topo,expected=False)
-    assert result is not True, "Testcase {} : Failed \n After shutting Down BGP convergence should Fail and return False \n Error: {}".format(tc_name, result)
+    result = verify_bgp_convergence(tgen, topo, expected=False)
+    assert (
+        result is not True
+    ), "Testcase {} : Failed \n After shutting Down BGP convergence should Fail and return False \n Error: {}".format(
+        tc_name, result
+    )
 
     step("verify default route deleted from R2 ")
     result = verify_rib_default_route(
@@ -1788,8 +1765,13 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         dut="r2",
         routes=DEFAULT_ROUTES,
         expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
-        expected=False)
-    assert result is not  True, "Testcase {} : Failed \n Error: After Shut down interface the default route is NOT expected but found in RIB -> {}".format( tc_name, result)
+        expected=False,
+    )
+    assert (
+        result is not True
+    ), "Testcase {} : Failed \n Error: After Shut down interface the default route is NOT expected but found in RIB -> {}".format(
+        tc_name, result
+    )
 
     result = verify_fib_default_route(
         tgen,
@@ -1797,11 +1779,17 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         dut="r2",
         routes=DEFAULT_ROUTES,
         expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
-        expected=False)
-    assert result is not  True, "Testcase {} : Failed \n Error:  After Shut down interface the default route is NOT expected but found in FIB -> {}".format( tc_name, result)
+        expected=False,
+    )
+    assert (
+        result is not True
+    ), "Testcase {} : Failed \n Error:  After Shut down interface the default route is NOT expected but found in FIB -> {}".format(
+        tc_name, result
+    )
 
-
-    step("no Shut IPv4/IPv6 BGP neighbor from R4 ( R4-R3) using 'neighbor x.x.x.x shut' command ")
+    step(
+        "no Shut IPv4/IPv6 BGP neighbor from R4 ( R4-R3) using 'neighbor x.x.x.x shut' command "
+    )
     local_as = get_dut_as_number(tgen, dut="r4")
     shut_neighbor = {
         "r4": {
@@ -1811,46 +1799,39 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
                     "ipv4": {
                         "unicast": {
                             "neighbor": {
-                                "r3": {
-                                    "dest_link": {
-                                        "r4": {"shutdown":False}
-                                    }
-                                }
+                                "r3": {"dest_link": {"r4": {"shutdown": False}}}
                             }
                         }
                     },
                     "ipv6": {
                         "unicast": {
                             "neighbor": {
-                                "r3": {
-                                    "dest_link": {
-                                        "r4": {"shutdown":False}
-                                    }
-                                }
+                                "r3": {"dest_link": {"r4": {"shutdown": False}}}
                             }
                         }
-                    }
-                }
+                    },
+                },
             }
         }
     }
     result = create_router_bgp(tgen, updated_topo, shut_neighbor)
     assert result is True, "Testcase {} : Failed \n Error: {}".format(tc_name, result)
 
-    interface = topo['routers']['r3']['links']['r4']['interface']
-    input_dict = {
-            "r1": {
-                "interface_list": [interface],
-                "status": "up"
-            }
-        }
+    interface = topo["routers"]["r3"]["links"]["r4"]["interface"]
+    input_dict = {"r1": {"interface_list": [interface], "status": "up"}}
 
     result = interface_status(tgen, topo, input_dict)
-    assert result is True, "Testcase {} : Bring up interface failed ! \n Error: {}".format(tc_name, result)
+    assert (
+        result is True
+    ), "Testcase {} : Bring up interface failed ! \n Error: {}".format(tc_name, result)
 
     step("After no shutting the interface verify the BGP convergence")
-    result = verify_bgp_convergence(tgen,topo,expected=True)
-    assert result is True, "Testcase {} : Failed \n After shutting Down BGP convergence should Fail and return False \n Error: {}".format(tc_name, result)
+    result = verify_bgp_convergence(tgen, topo, expected=True)
+    assert (
+        result is True
+    ), "Testcase {} : Failed \n After shutting Down BGP convergence should Fail and return False \n Error: {}".format(
+        tc_name, result
+    )
 
     step("After no shut neighbor , verify default route relearn on R2")
     result = verify_rib_default_route(
@@ -1859,8 +1840,13 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         dut="r2",
         routes=DEFAULT_ROUTES,
         expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
-        expected=True)
-    assert result is   True, "Testcase {} : Failed \n Error: After no Shut down interface the default route is  expected but found in RIB -> {}".format( tc_name, result)
+        expected=True,
+    )
+    assert (
+        result is True
+    ), "Testcase {} : Failed \n Error: After no Shut down interface the default route is  expected but found in RIB -> {}".format(
+        tc_name, result
+    )
 
     result = verify_fib_default_route(
         tgen,
@@ -1868,10 +1854,13 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         dut="r2",
         routes=DEFAULT_ROUTES,
         expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
-        expected=True)
-    assert result is True, "Testcase {} : Failed \n Error:  After Shut down interface the default route is  expected but found in FIB -> {}".format( tc_name, result)
-
-
+        expected=True,
+    )
+    assert (
+        result is True
+    ), "Testcase {} : Failed \n Error:  After Shut down interface the default route is  expected but found in FIB -> {}".format(
+        tc_name, result
+    )
 
     step("Remove IPv4/IPv6 static route configure on R4")
     for addr_type in ADDR_TYPES:
@@ -1881,7 +1870,7 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
                     {
                         "network": [NETWORK1_1[addr_type]],
                         "next_hop": NEXT_HOP_IP[addr_type],
-                        "delete": True
+                        "delete": True,
                     }
                 ]
             }
@@ -1902,12 +1891,16 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
                 ]
             }
         }
-        result = verify_fib_routes(tgen, addr_type, "r4", static_routes_input, expected=False)
-        assert result is not  True, "Testcase {} : Failed \n Error: {}".format(
+        result = verify_fib_routes(
+            tgen, addr_type, "r4", static_routes_input, expected=False
+        )
+        assert result is not True, "Testcase {} : Failed \n Error: {}".format(
             tc_name, result
         )
-        result = verify_bgp_rib(tgen, addr_type, "r4", static_routes_input, expected=False)
-        assert result is not  True, "Testcase {} : Failed \n Error: {}".format(
+        result = verify_bgp_rib(
+            tgen, addr_type, "r4", static_routes_input, expected=False
+        )
+        assert result is not True, "Testcase {} : Failed \n Error: {}".format(
             tc_name, result
         )
 
@@ -1918,8 +1911,13 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         dut="r2",
         routes=DEFAULT_ROUTES,
         expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
-        expected= False)
-    assert result is not  True, "Testcase {} : Failed \n Error: After removing static  the default route is NOT expected but found in RIB -> {}".format( tc_name, result)
+        expected=False,
+    )
+    assert (
+        result is not True
+    ), "Testcase {} : Failed \n Error: After removing static  the default route is NOT expected but found in RIB -> {}".format(
+        tc_name, result
+    )
 
     result = verify_fib_default_route(
         tgen,
@@ -1927,9 +1925,13 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         dut="r2",
         routes=DEFAULT_ROUTES,
         expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
-        expected= False)
-    assert result is not True, "Testcase {} : Failed \n Error:  After removing static the default route is NOT expected but found in FIB -> {}".format( tc_name, result)
-
+        expected=False,
+    )
+    assert (
+        result is not True
+    ), "Testcase {} : Failed \n Error:  After removing static the default route is NOT expected but found in FIB -> {}".format(
+        tc_name, result
+    )
 
     step("Configuring the static route back in r4")
     for addr_type in ADDR_TYPES:
@@ -1959,12 +1961,16 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
                 ]
             }
         }
-        result = verify_fib_routes(tgen, addr_type, "r4", static_routes_input, expected=True)
-        assert result is   True, "Testcase {} : Failed \n Error: {}".format(
+        result = verify_fib_routes(
+            tgen, addr_type, "r4", static_routes_input, expected=True
+        )
+        assert result is True, "Testcase {} : Failed \n Error: {}".format(
             tc_name, result
         )
-        result = verify_bgp_rib(tgen, addr_type, "r4", static_routes_input, expected=True)
-        assert result is    True, "Testcase {} : Failed \n Error: {}".format(
+        result = verify_bgp_rib(
+            tgen, addr_type, "r4", static_routes_input, expected=True
+        )
+        assert result is True, "Testcase {} : Failed \n Error: {}".format(
             tc_name, result
         )
 
@@ -1975,8 +1981,13 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         dut="r2",
         routes=DEFAULT_ROUTES,
         expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
-        expected= True)
-    assert result is   True, "Testcase {} : Failed \n Error: After removing static  the default route is  expected but found in RIB -> {}".format( tc_name, result)
+        expected=True,
+    )
+    assert (
+        result is True
+    ), "Testcase {} : Failed \n Error: After removing static  the default route is  expected but found in RIB -> {}".format(
+        tc_name, result
+    )
 
     result = verify_fib_default_route(
         tgen,
@@ -1984,8 +1995,13 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         dut="r2",
         routes=DEFAULT_ROUTES,
         expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
-        expected= True)
-    assert result is  True, "Testcase {} : Failed \n Error:  After removing static  the default route is  expected but found in FIB -> {}".format( tc_name, result)
+        expected=True,
+    )
+    assert (
+        result is True
+    ), "Testcase {} : Failed \n Error:  After removing static  the default route is  expected but found in FIB -> {}".format(
+        tc_name, result
+    )
 
     step("Deactivate IPv4 and IPv6 neighbor configured from R4 ( R4-R3)")
 
@@ -1999,15 +2015,14 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
                                 "r3": {"dest_link": {"r4": {"deactivate": "ipv4"}}}
                             }
                         },
-
-                    },"ipv6": {
+                    },
+                    "ipv6": {
                         "unicast": {
                             "neighbor": {
                                 "r3": {"dest_link": {"r4": {"deactivate": "ipv6"}}}
                             }
                         },
-
-                    }
+                    },
                 }
             }
         }
@@ -2022,8 +2037,13 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         dut="r2",
         routes=DEFAULT_ROUTES,
         expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
-        expected= False)
-    assert result is not  True, "Testcase {} : Failed \n Error: After Deactivating the BGP neighbor the default route is NOT expected but found in RIB -> {}".format( tc_name, result)
+        expected=False,
+    )
+    assert (
+        result is not True
+    ), "Testcase {} : Failed \n Error: After Deactivating the BGP neighbor the default route is NOT expected but found in RIB -> {}".format(
+        tc_name, result
+    )
 
     result = verify_fib_default_route(
         tgen,
@@ -2031,8 +2051,13 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         dut="r2",
         routes=DEFAULT_ROUTES,
         expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
-        expected= False)
-    assert result is not  True, "Testcase {} : Failed \n Error: After Deactivating the BGP neighbor  the default route is NOT expected but found in FIB -> {}".format( tc_name, result)
+        expected=False,
+    )
+    assert (
+        result is not True
+    ), "Testcase {} : Failed \n Error: After Deactivating the BGP neighbor  the default route is NOT expected but found in FIB -> {}".format(
+        tc_name, result
+    )
 
     step("Activate IPv4 and IPv6 neighbor configured from R4 ( R4-R3)")
 
@@ -2046,15 +2071,14 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
                                 "r3": {"dest_link": {"r4": {"activate": "ipv4"}}}
                             }
                         },
-
-                    },"ipv6": {
+                    },
+                    "ipv6": {
                         "unicast": {
                             "neighbor": {
                                 "r3": {"dest_link": {"r4": {"activate": "ipv6"}}}
                             }
                         },
-
-                    }
+                    },
                 }
             }
         }
@@ -2064,7 +2088,7 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
 
     step("Verify bgp convergence.")
     bgp_convergence = verify_bgp_convergence(tgen, updated_topo)
-    assert bgp_convergence is  True, "Testcase {} : Failed \n Error: {}".format(
+    assert bgp_convergence is True, "Testcase {} : Failed \n Error: {}".format(
         tc_name, bgp_convergence
     )
     step("After Activating the BGP neighbor   , verify default route learned on R2")
@@ -2074,8 +2098,13 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         dut="r2",
         routes=DEFAULT_ROUTES,
         expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
-        expected= True)
-    assert result is   True, "Testcase {} : Failed \n Error: After Deactivating the BGP neighbor the default route is   expected but found in RIB -> {}".format( tc_name, result)
+        expected=True,
+    )
+    assert (
+        result is True
+    ), "Testcase {} : Failed \n Error: After Deactivating the BGP neighbor the default route is   expected but found in RIB -> {}".format(
+        tc_name, result
+    )
 
     result = verify_fib_default_route(
         tgen,
@@ -2083,10 +2112,16 @@ def test_verify_default_originate_after_BGP_attributes_p1(request):
         dut="r2",
         routes=DEFAULT_ROUTES,
         expected_nexthop=DEFAULT_ROUTE_NXT_HOP_R3,
-        expected= True)
-    assert result is   True, "Testcase {} : Failed \n Error:  After Deactivating the BGP neighbor the default route is   expected but found in FIB -> {}".format( tc_name, result)
+        expected=True,
+    )
+    assert (
+        result is True
+    ), "Testcase {} : Failed \n Error:  After Deactivating the BGP neighbor the default route is   expected but found in FIB -> {}".format(
+        tc_name, result
+    )
     write_test_footer(tc_name)
 
+
 if __name__ == "__main__":
     args = ["-s"] + sys.argv[1:]
     sys.exit(pytest.main(args))
