@@ -179,7 +179,7 @@ def setup_module(mod):
         pe.cmd_raises("sysctl -w net.ipv4.tcp_l3mdev_accept={}".format(l3mdev_accept))
 
     # For all registered routers, load the zebra configuration file
-    for (name, router) in tgen.routers().items():
+    for name, router in tgen.routers().items():
         router.load_config(
             TopoRouter.RD_ZEBRA, os.path.join(CWD, "{}/zebra.conf".format(name))
         )
@@ -230,18 +230,18 @@ def evpn_gateway_ip_show_op_check(trigger=" "):
         "zebra_vrf_ipv6": "show ipv6 route vrf vrf-blue json",
     }
 
-    for (name, pe) in tgen.gears.items():
+    for name, pe in tgen.gears.items():
         if name not in PES:
             continue
 
-        for (cmd_key, command) in show_commands.items():
+        for cmd_key, command in show_commands.items():
             expected_op_file = "{0}/{1}/{2}_{3}.json".format(
                 CWD, name, cmd_key, trigger
             )
             expected_op = json.loads(open(expected_op_file).read())
 
             test_func = partial(topotest.router_json_cmp, pe, command, expected_op)
-            ret, result = topotest.run_and_expect(test_func, None, count=30, wait=1)
+            _, result = topotest.run_and_expect(test_func, None, count=30, wait=1)
             assertmsg = '"{0}" JSON output mismatch for {1}'.format(name, command)
             if result is not None:
                 return result, assertmsg
