@@ -1,24 +1,11 @@
 #!/usr/bin/env python
+# SPDX-License-Identifier: ISC
 
 #
 # Copyright (c) 2020 by VMware, Inc. ("VMware")
 # Used Copyright (c) 2018 by Network Device Education Foundation,
 # Inc. ("NetDEF") in this file.
 #
-# Permission to use, copy, modify, and/or distribute this software
-# for any purpose with or without fee is hereby granted, provided
-# that the above copyright notice and this permission notice appear
-# in all copies.
-#
-# THE SOFTWARE IS PROVIDED "AS IS" AND VMWARE DISCLAIMS ALL WARRANTIES
-# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
-# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL VMWARE BE LIABLE FOR
-# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY
-# DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
-# WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
-# ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
-# OF THIS SOFTWARE.
-#
 
 """
 Following tests are covered to test BGP VRF Lite:
@@ -62,10 +49,8 @@ from lib.topolog import logger
 from lib.bgp import (
     verify_bgp_convergence,
     create_router_bgp,
-    verify_bgp_community,
     verify_bgp_rib,
-    clear_bgp,
-    verify_best_path_as_per_bgp_attribute
+    verify_best_path_as_per_bgp_attribute,
 )
 from lib.topojson import build_config_from_json
 
@@ -476,7 +461,6 @@ def test_bgp_best_path_with_dynamic_import_p0(request):
         check_router_status(tgen)
 
     for addr_type in ADDR_TYPES:
-
         step(
             "Redistribute configured static routes into BGP process" " on R1/R2 and R3"
         )
@@ -508,7 +492,6 @@ def test_bgp_best_path_with_dynamic_import_p0(request):
         )
 
     for addr_type in ADDR_TYPES:
-
         step("Import from default vrf into vrf ISR on R1 and R2 as below")
 
         input_dict_vrf = {}
@@ -566,7 +549,6 @@ def test_bgp_best_path_with_dynamic_import_p0(request):
     )
 
     for addr_type in ADDR_TYPES:
-
         step("Verify Pre-emption")
 
         input_routes_r3 = {
@@ -598,7 +580,6 @@ def test_bgp_best_path_with_dynamic_import_p0(request):
     shutdown_bringup_interface(tgen, "r4", intf_r4_r1, False)
 
     for addr_type in ADDR_TYPES:
-
         input_routes_r3 = {
             "r3": {"static_routes": [{"network": [NETWORK3_3[addr_type]]}]}
         }
@@ -629,7 +610,6 @@ def test_bgp_best_path_with_dynamic_import_p0(request):
     shutdown_bringup_interface(tgen, "r4", intf_r4_r1, True)
 
     for addr_type in ADDR_TYPES:
-
         input_routes_r3 = {
             "r3": {"static_routes": [{"network": [NETWORK3_3[addr_type]]}]}
         }
@@ -659,7 +639,6 @@ def test_bgp_best_path_with_dynamic_import_p0(request):
     step("Active-Standby scenario(as-path prepend and Local pref)")
 
     for addr_type in ADDR_TYPES:
-
         step("Create prefix-list")
 
         input_dict_pf = {
@@ -683,7 +662,6 @@ def test_bgp_best_path_with_dynamic_import_p0(request):
         )
 
     for addr_type in ADDR_TYPES:
-
         step("Create route-map to match prefix-list and set localpref 500")
 
         input_dict_rm = {
@@ -791,15 +769,10 @@ def test_bgp_best_path_with_dynamic_import_p0(request):
     attribute = "locPrf"
 
     for addr_type in ADDR_TYPES:
-
         step("Verify bestpath is installed as per highest localpref")
 
         input_routes_r3 = {
-            "r3": {
-                "static_routes": [
-                    {"network": [NETWORK3_3[addr_type], NETWORK3_4[addr_type]]}
-                ]
-            }
+            "r3": {"static_routes": [{"network": [NETWORK3_4[addr_type]]}]}
         }
 
         result = verify_best_path_as_per_bgp_attribute(
@@ -810,7 +783,6 @@ def test_bgp_best_path_with_dynamic_import_p0(request):
         )
 
     for addr_type in ADDR_TYPES:
-
         step("Create route-map to match prefix-list and set localpref 700")
 
         input_dict_rm = {
@@ -838,15 +810,10 @@ def test_bgp_best_path_with_dynamic_import_p0(request):
         )
 
     for addr_type in ADDR_TYPES:
-
         step("Verify bestpath is changed as per highest localpref")
 
         input_routes_r3 = {
-            "r3": {
-                "static_routes": [
-                    {"network": [NETWORK3_3[addr_type], NETWORK3_4[addr_type]]}
-                ]
-            }
+            "r3": {"static_routes": [{"network": [NETWORK3_4[addr_type]]}]}
         }
 
         result = verify_best_path_as_per_bgp_attribute(
@@ -857,7 +824,6 @@ def test_bgp_best_path_with_dynamic_import_p0(request):
         )
 
     for addr_type in ADDR_TYPES:
-
         step("Create route-map to match prefix-list and set as-path prepend")
 
         input_dict_rm = {
@@ -890,15 +856,10 @@ def test_bgp_best_path_with_dynamic_import_p0(request):
     attribute = "path"
 
     for addr_type in ADDR_TYPES:
-
         step("Verify bestpath is changed as per shortest as-path")
 
         input_routes_r3 = {
-            "r3": {
-                "static_routes": [
-                    {"network": [NETWORK3_3[addr_type], NETWORK3_4[addr_type]]}
-                ]
-            }
+            "r3": {"static_routes": [{"network": [NETWORK3_4[addr_type]]}]}
         }
 
         result = verify_best_path_as_per_bgp_attribute(
