@@ -58,9 +58,6 @@ if [ ! -e Makefile ]; then
 	fi
 
 	bash configure >&3 \
-		--enable-static-bin \
-		--enable-static \
-		--enable-shared \
 		--enable-dev-build \
 		--with-moduledir=/usr/lib/frr/modules \
 		--prefix=/usr \
@@ -69,6 +66,8 @@ if [ ! -e Makefile ]; then
 		--sbindir=/usr/lib/frr \
 		--enable-multipath=0 \
 		--enable-fpm \
+		--enable-grpc \
+		--enable-scripting \
 		--enable-sharpd \
 		$EXTRA_CONFIGURE \
 		--with-pkg-extra-version=-topotests \
