@@ -245,7 +245,7 @@ def check_ping6(name, dest_addr, expect_connected):
         if match not in output:
             return "ping fail"
 
-    match = "{} packet loss".format("0%" if expect_connected else "100%")
+    match = "{} packet loss".format(", 0%" if expect_connected else ", 100%")
     logger.info("[+] check {} {} {}".format(name, dest_addr, match))
     tgen = get_topogen()
     func = functools.partial(_check, name, dest_addr, match)
@@ -333,7 +333,7 @@ def test_ping_step1():
 
     # Setup encap route on rt1, decap route on rt2
     tgen.gears["rt1"].vtysh_cmd(
-        "sharp install seg6-routes fc00:0:9::1 nexthop-seg6 2001:db8:1::2 encap fc00:0:1:2:6:f00d:: 1"
+        "sharp install seg6-routes fc00:0:9::1 nexthop-seg6 2001:db8:1::2 encap fc00:0:2:6:f00d:: 1"
     )
     tgen.gears["rt6"].vtysh_cmd(
         "sharp install seg6local-routes fc00:0:f00d:: nexthop-seg6local eth-dst End_DT6 254 1"
@@ -443,7 +443,8 @@ def test_ping_step2():
     if tgen.routers_have_failure():
         pytest.skip(tgen.errors)
 
-    check_ping6("rt1", "fc00:0:9::1", False)
+    # ping should pass because route to fc00:0:2:6:f00d:: is still valid
+    check_ping6("rt1", "fc00:0:9::1", True)
 
 
 #
@@ -643,7 +644,8 @@ def test_ping_step4():
     if tgen.routers_have_failure():
         pytest.skip(tgen.errors)
 
-    check_ping6("rt1", "fc00:0:9::1", False)
+    # ping should pass because route to fc00:0:2:6:f00d:: is still valid
+    check_ping6("rt1", "fc00:0:9::1", True)
 
 
 #
@@ -838,7 +840,8 @@ def test_ping_step6():
     if tgen.routers_have_failure():
         pytest.skip(tgen.errors)
 
-    check_ping6("rt1", "fc00:0:9::1", False)
+    # ping should pass because route to fc00:0:2:6:f00d:: is still valid
+    check_ping6("rt1", "fc00:0:9::1", True)
 
 
 #
@@ -1033,7 +1036,8 @@ def test_ping_step8():
     if tgen.routers_have_failure():
         pytest.skip(tgen.errors)
 
-    check_ping6("rt1", "fc00:0:9::1", False)
+    # ping should pass because route to fc00:0:2:6:f00d:: is still valid
+    check_ping6("rt1", "fc00:0:9::1", True)
 
 
 #
