@@ -3,7 +3,7 @@
 
 # test_ospf6_ecmp_inter_area.py
 #
-# Copyright (c) 2021, 2022 by Martin Buck
+# Copyright (c) 2021, 2022, 2024 by Martin Buck
 # Copyright (c) 2016 by
 # Network Device Education Foundation, Inc. ("NetDEF")
 #
@@ -11,37 +11,45 @@
 r"""
 test_ospf6_ecmp_inter_area.py: Test OSPFv3 ECMP inter-area nexthop update
 
-Check proper removal of ECMP nexthops after a path used by one nexthop
-disappears. We remove a path by bringing down a link required by that
-path which is not adjacent to the router being checked. This is important
-because when bringing down adjacent links, the kernel might remove the
-nexthops itself without ospf6d having to do anything.
+Check proper addition and removal of ECMP nexthops in 2 cases: Parallel
+paths to one ABR and parallel ABRs. We test nexthop removal triggered by
+path removal by bringing down a link required by that path which is not
+adjacent to the router being checked. This is important because when
+bringing down adjacent links, the kernel might remove the nexthops itself
+without ospf6d having to do anything.
 
-Useful as a regression test for #9720.
+Useful as a regression test for #9720 and #15777.
 
 Topology:
-             .
-      Area 0 . Area 1
-             .
-   -- R2 --  . ---- R6
-  /        \ ./
-R1 -- R3 -- R5 ---- R7  Area 1
-  \        /  \\ ..............
-   -- R4 --    \--- R8  Area 0
-                \
-                 -- R9
+                  .
+           Area 0 . Area 1
+                  .
+    -- R2 ------ R5 -----
+   /              .\     \
+  /               . |     \
+R1 --- R3 ------ R6 ------ R7
+  \            / |. |
+   \          /  |. |
+    -- R4 ----   |. |
+                / ./
+              R8 --
+                  .
 
-We check routes on R1, primarily those towards R6/7/8/9. Those to R6/7 are
-inter-area routes with R5 being ABR, those to R8/9 are intra-area routes
-and are used for reference. R6/R8 announce external routes, R7/R9 announce
-internal routes.
+We check routes on R1, primarily those towards R7/8. Those to R7 are
+inter-area routes with R5/6 being ABRs, those to R8 are intra-area routes
+and are used for reference. R7/R8 announce one internal and one external
+route each.
 
 With all links up, we expect 3 ECMP paths and 3 nexthops on R1 towards each
-of R6/7/8/9. Then we bring down the R2-R5 link, causing only 2 remaining
-paths and 2 nexthops on R1. The test is successful if the number of nexthops
-for the routes on R1 is as expected.
+of R7/8. Then we bring down the R3-R6 link, causing only 2 remaining
+paths and 2 nexthops on R1. Then we bring down the R2-R5 link, causing only
+1 remaining path and 1 nexthop on R1.
+
+The test is successful if the number of nexthops and their interfaces for
+the routes on R1 is as expected.
 """
 
+import json
 import os
 import sys
 from functools import partial
@@ -55,7 +63,7 @@ sys.path.append(os.path.join(CWD, "../"))
 # Import topogen and topotest helpers
 from lib import topotest
 from lib.topogen import Topogen, TopoRouter, get_topogen
-from lib.topolog import logger
+from lib.common_config import write_test_header, write_test_footer, step
 
 # Required to instantiate the topology builder class.
 
@@ -65,20 +73,24 @@ pytestmark = [pytest.mark.ospf6d]
 def build_topo(tgen):
     "Build function"
 
-    # Create 9 routers
-    for routern in range(1, 10):
+    # Create 8 routers
+    for routern in range(1, 9):
         tgen.add_router("r{}".format(routern))
-
     tgen.gears["r1"].add_link(tgen.gears["r2"])
     tgen.gears["r1"].add_link(tgen.gears["r3"])
     tgen.gears["r1"].add_link(tgen.gears["r4"])
     tgen.gears["r2"].add_link(tgen.gears["r5"])
-    tgen.gears["r3"].add_link(tgen.gears["r5"])
-    tgen.gears["r4"].add_link(tgen.gears["r5"])
-    tgen.gears["r5"].add_link(tgen.gears["r6"])
+    tgen.gears["r3"].add_link(tgen.gears["r6"])
+    tgen.gears["r4"].add_link(tgen.gears["r6"])
     tgen.gears["r5"].add_link(tgen.gears["r7"])
     tgen.gears["r5"].add_link(tgen.gears["r8"])
-    tgen.gears["r5"].add_link(tgen.gears["r9"])
+    tgen.gears["r6"].add_link(tgen.gears["r7"])
+    tgen.gears["r6"].add_link(tgen.gears["r8"])
+    # Additional "loopback" interfaces. Not used for communication, just to
+    # hold an address we use to inject intra-/inter-area routes (the one on
+    # the real "lo" loopback is used for external routes).
+    tgen.gears["r7"].add_link(tgen.gears["r7"])
+    tgen.gears["r8"].add_link(tgen.gears["r8"])
 
 
 def setup_module(mod):
@@ -100,17 +112,42 @@ def setup_module(mod):
     tgen.start_router()
 
 
-def test_wait_protocol_convergence():
+def expect_routes_json(router, exp_routes_json_fname, stepmsg):
+    "Wait until OSPFv3 routes match JSON spec"
+    step(
+        "waiting for OSPFv3 router '{}' routes/nexthops to match {} ({})".format(
+            router, exp_routes_json_fname, stepmsg
+        )
+    )
+
+    json_file = "{}/{}/{}".format(CWD, router, exp_routes_json_fname)
+    expected = json.loads(open(json_file).read())
+    tgen = get_topogen()
+    test_func = partial(
+        topotest.router_json_cmp,
+        tgen.gears[router],
+        "show ipv6 route ospf6 json",
+        expected,
+    )
+    _, result = topotest.run_and_expect(test_func, None, count=60, wait=0.5)
+    assertmsg = '"{}" JSON output mismatches ({})'.format(router, stepmsg)
+    assert result is None, assertmsg
+
+
+def test_wait_protocol_convergence(request):
     "Wait for OSPFv3 to converge"
+    tc_name = request.node.name
+    write_test_header(tc_name)
+
     tgen = get_topogen()
     if tgen.routers_have_failure():
         pytest.skip(tgen.errors)
 
-    logger.info("waiting for protocols to converge")
+    step("waiting for protocols to converge")
 
     def expect_neighbor_full(router, neighbor):
         "Wait until OSPFv3 neighborship is full"
-        logger.info(
+        step(
             "waiting for OSPFv3 router '{}' neighborship with '{}'".format(
                 router, neighbor
             )
@@ -131,57 +168,45 @@ def test_wait_protocol_convergence():
     expect_neighbor_full("r2", "10.254.254.1")
     expect_neighbor_full("r2", "10.254.254.5")
     expect_neighbor_full("r3", "10.254.254.1")
-    expect_neighbor_full("r3", "10.254.254.5")
+    expect_neighbor_full("r3", "10.254.254.6")
     expect_neighbor_full("r4", "10.254.254.1")
-    expect_neighbor_full("r4", "10.254.254.5")
+    expect_neighbor_full("r4", "10.254.254.6")
     expect_neighbor_full("r5", "10.254.254.2")
-    expect_neighbor_full("r5", "10.254.254.3")
-    expect_neighbor_full("r5", "10.254.254.4")
-    expect_neighbor_full("r5", "10.254.254.6")
     expect_neighbor_full("r5", "10.254.254.7")
     expect_neighbor_full("r5", "10.254.254.8")
-    expect_neighbor_full("r5", "10.254.254.9")
-    expect_neighbor_full("r6", "10.254.254.5")
+    expect_neighbor_full("r6", "10.254.254.3")
+    expect_neighbor_full("r6", "10.254.254.7")
+    expect_neighbor_full("r6", "10.254.254.8")
     expect_neighbor_full("r7", "10.254.254.5")
+    expect_neighbor_full("r7", "10.254.254.6")
     expect_neighbor_full("r8", "10.254.254.5")
-    expect_neighbor_full("r9", "10.254.254.5")
+    expect_neighbor_full("r8", "10.254.254.6")
+
+    expect_routes_json("r1", "show_ipv6_routes_ospf6-1.json", "post-convergence")
+
+    write_test_footer(tc_name)
 
 
-def test_ecmp_inter_area():
+def test_ecmp_inter_area(request):
     "Test whether OSPFv3 ECMP nexthops are properly updated for inter-area routes after link down"
+    tc_name = request.node.name
+    write_test_header(tc_name)
+
     tgen = get_topogen()
     if tgen.routers_have_failure():
         pytest.skip(tgen.errors)
 
-    def num_nexthops(router):
-        routes = tgen.gears[router].vtysh_cmd("show ipv6 ospf6 route json", isjson=True)
-        route_prefixes_infos = sorted(routes.get("routes", {}).items())
-        return [len(ri.get("nextHops", [])) for rp, ri in route_prefixes_infos]
+    step("triggering R3-R6 link down")
+    tgen.gears["r3"].run("ip link set r3-eth1 down")
 
-    def expect_num_nexthops(router, expected_num_nexthops, count):
-        "Wait until number of nexthops for routes matches expectation"
-        logger.info(
-            "waiting for OSPFv3 router '{}' nexthops {}".format(
-                router, expected_num_nexthops
-            )
-        )
-        test_func = partial(num_nexthops, router)
-        _, result = topotest.run_and_expect(
-            test_func, expected_num_nexthops, count=count, wait=3
-        )
-        assert (
-            result == expected_num_nexthops
-        ), "'{}' wrong number of route nexthops".format(router)
+    expect_routes_json("r1", "show_ipv6_routes_ospf6-2.json", "post-R3-R6-link-down")
 
-    # Check nexthops pre link-down
-    expect_num_nexthops("r1", [1, 1, 1, 3, 3, 3, 3, 3, 3, 3], 4)
-
-    logger.info("triggering R2-R4 link down")
+    step("triggering R2-R5 link down")
     tgen.gears["r2"].run("ip link set r2-eth1 down")
 
-    # tgen.mininet_cli()
-    # Check nexthops post link-down
-    expect_num_nexthops("r1", [1, 1, 1, 2, 2, 2, 2, 2, 2, 2], 8)
+    expect_routes_json("r1", "show_ipv6_routes_ospf6-3.json", "post-R2-R5-link-down")
+
+    write_test_footer(tc_name)
 
 
 def teardown_module(_mod):
@@ -190,14 +215,19 @@ def teardown_module(_mod):
     tgen.stop_topology()
 
 
-def test_memory_leak():
+def test_memory_leak(request):
     "Run the memory leak test and report results."
+    tc_name = request.node.name
+    write_test_header(tc_name)
+
     tgen = get_topogen()
     if not tgen.is_memleak_enabled():
         pytest.skip("Memory leak test/report is disabled")
 
     tgen.report_memory_leaks()
 
+    write_test_footer(tc_name)
+
 
 if __name__ == "__main__":
     args = ["-s"] + sys.argv[1:]
