@@ -534,6 +534,12 @@ const char *rtm_rta2str(int type)
 		return "NH_ID";
 	case RTA_EXPIRES:
 		return "EXPIRES";
+	case RTA_VIA:
+		return "VIA";
+	case RTA_ENCAP_TYPE:
+		return "RTA_ENCAP_TYPE";
+	case RTA_ENCAP:
+		return "RTA_ENCAP";
 	default:
 		return "UNKNOWN";
 	}
@@ -1403,8 +1409,8 @@ next_rta:
 		}
 		break;
 	case NHA_ENCAP:
-		/* TODO: handle MPLS labels. */
-		zlog_debug("      unparsed MPLS labels");
+		/* TODO: handle MPLS labels or SRv6 SIDs. */
+		zlog_debug("      unparsed MPLS labels or SRv6 SIDs");
 		break;
 	case NHA_GROUPS:
 		/* TODO: handle this message. */
