@@ -1,19 +1,6 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
 /* zebra table Manager for routing table identifier management
  * Copyright (C) 2018 6WIND
- *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License as published by the Free
- * Software Foundation; either version 2 of the License, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along
- * with this program; see the file COPYING; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "zebra.h"
@@ -37,21 +24,6 @@
 #include "zebra/table_manager.h"
 #include "zebra/zebra_errors.h"
 
-/* routing table identifiers
- *
- */
-#if !defined(GNU_LINUX)
-/* BSD systems
- */
-#else
-/* Linux Systems
- */
-#define RT_TABLE_ID_LOCAL                  255
-#define RT_TABLE_ID_MAIN                   254
-#define RT_TABLE_ID_DEFAULT                253
-#define RT_TABLE_ID_COMPAT                 252
-#define RT_TABLE_ID_UNSPEC                 0
-#endif /* !def(GNU_LINUX) */
 #define RT_TABLE_ID_UNRESERVED_MIN         1
 #define RT_TABLE_ID_UNRESERVED_MAX         0xffffffff
 
@@ -76,8 +48,7 @@ void table_manager_enable(struct zebra_vrf *zvrf)
 	    && strcmp(zvrf_name(zvrf), VRF_DEFAULT_NAME)) {
 		struct zebra_vrf *def = zebra_vrf_lookup_by_id(VRF_DEFAULT);
 
-		if (def)
-			zvrf->tbl_mgr = def->tbl_mgr;
+		zvrf->tbl_mgr = def->tbl_mgr;
 		return;
 	}
 	zvrf->tbl_mgr = XCALLOC(MTYPE_TM_TABLE, sizeof(struct table_manager));
@@ -293,52 +264,11 @@ void table_manager_disable(struct zebra_vrf *zvrf)
 	zvrf->tbl_mgr = NULL;
 }
 
-int table_manager_range(struct vty *vty, bool add, struct zebra_vrf *zvrf,
-			const char *start_table_str, const char *end_table_str)
+void table_manager_range(bool add, struct zebra_vrf *zvrf, uint32_t start,
+			 uint32_t end)
 {
-	uint32_t start;
-	uint32_t end;
-
-	if (add) {
-		if (!start_table_str || !end_table_str) {
-			vty_out(vty, "%% Labels not specified\n");
-			return CMD_WARNING_CONFIG_FAILED;
-		}
-		start = atoi(start_table_str);
-		end = atoi(end_table_str);
-		if (end < start) {
-			vty_out(vty, "%% End table is less than Start table\n");
-			return CMD_WARNING_CONFIG_FAILED;
-		}
-
-#if !defined(GNU_LINUX)
-/* BSD systems
- */
-#else
-		/* Linux Systems
-		 */
-		if ((start >= RT_TABLE_ID_COMPAT && start <= RT_TABLE_ID_LOCAL)
-		    || (end >= RT_TABLE_ID_COMPAT
-			&& end <= RT_TABLE_ID_LOCAL)) {
-			vty_out(vty, "%% Values forbidden in range [%u;%u]\n",
-				RT_TABLE_ID_COMPAT, RT_TABLE_ID_LOCAL);
-			return CMD_WARNING_CONFIG_FAILED;
-		}
-		if (start < RT_TABLE_ID_COMPAT && end > RT_TABLE_ID_LOCAL) {
-			vty_out(vty,
-				"%% Range overlaps range [%u;%u] forbidden\n",
-				RT_TABLE_ID_COMPAT, RT_TABLE_ID_LOCAL);
-			return CMD_WARNING_CONFIG_FAILED;
-		}
-#endif
-		if (zvrf->tbl_mgr
-		    && ((zvrf->tbl_mgr->start && zvrf->tbl_mgr->start != start)
-			|| (zvrf->tbl_mgr->end && zvrf->tbl_mgr->end != end))) {
-			vty_out(vty,
-				"%% New range will be taken into account at restart\n");
-		}
+	if (add)
 		table_range_add(zvrf, start, end);
-	} else
+	else
 		table_range_add(zvrf, 0, 0);
-	return CMD_SUCCESS;
 }
