@@ -485,7 +485,7 @@ static inline void zfpm_write_on(void)
  */
 static inline void zfpm_read_off(void)
 {
-	EVENT_OFF(zfpm_g->t_read);
+	event_cancel(&zfpm_g->t_read);
 }
 
 /*
@@ -493,17 +493,17 @@ static inline void zfpm_read_off(void)
  */
 static inline void zfpm_write_off(void)
 {
-	EVENT_OFF(zfpm_g->t_write);
+	event_cancel(&zfpm_g->t_write);
 }
 
 static inline void zfpm_connect_off(void)
 {
-	EVENT_OFF(zfpm_g->t_connect);
+	event_cancel(&zfpm_g->t_connect);
 }
 
 static inline void zfpm_conn_down_off(void)
 {
-	EVENT_OFF(zfpm_g->t_conn_down);
+	event_cancel(&zfpm_g->t_conn_down);
 }
 
 /*
@@ -577,7 +577,7 @@ static void zfpm_connection_up(const char *detail)
 	/*
 	 * Start thread to push existing routes to the FPM.
 	 */
-	EVENT_OFF(zfpm_g->t_conn_up);
+	event_cancel(&zfpm_g->t_conn_up);
 
 	zfpm_rnodes_iter_init(&zfpm_g->t_conn_up_state.iter);
 	zfpm_g->fpm_mac_dump_done = false;
@@ -1703,7 +1703,7 @@ static void zfpm_stop_stats_timer(void)
 		return;
 
 	zfpm_debug("Stopping existing stats timer");
-	EVENT_OFF(zfpm_g->t_stats);
+	event_cancel(&zfpm_g->t_stats);
 }
 
 /*
