@@ -14,6 +14,7 @@ extern "C" {
 extern const struct frr_yang_module_info frr_zebra_info;
 
 /* prototypes */
+struct yang_data *zebra_max_multipath_get_elem(struct nb_cb_get_elem_args *args);
 int get_route_information_rpc(struct nb_cb_rpc_args *args);
 int get_v6_mroute_info_rpc(struct nb_cb_rpc_args *args);
 int get_vrf_info_rpc(struct nb_cb_rpc_args *args);
@@ -31,9 +32,12 @@ int get_debugs_rpc(struct nb_cb_rpc_args *args);
 int zebra_mcast_rpf_lookup_modify(struct nb_cb_modify_args *args);
 int zebra_ip_forwarding_modify(struct nb_cb_modify_args *args);
 int zebra_ip_forwarding_destroy(struct nb_cb_destroy_args *args);
+struct yang_data *zebra_ip_forwarding_get_elem(struct nb_cb_get_elem_args *args);
 int zebra_ipv6_forwarding_modify(struct nb_cb_modify_args *args);
 int zebra_ipv6_forwarding_destroy(struct nb_cb_destroy_args *args);
 int zebra_workqueue_hold_timer_modify(struct nb_cb_modify_args *args);
+struct yang_data *zebra_ipv6_forwarding_get_elem(struct nb_cb_get_elem_args *args);
+struct yang_data *zebra_state_mpls_forwarding_get_elem(struct nb_cb_get_elem_args *args);
 int zebra_zapi_packets_modify(struct nb_cb_modify_args *args);
 int zebra_import_kernel_table_table_id_modify(struct nb_cb_modify_args *args);
 int zebra_import_kernel_table_table_id_destroy(struct nb_cb_destroy_args *args);
@@ -196,7 +200,6 @@ int lib_interface_zebra_evpn_mh_df_preference_modify(
 	struct nb_cb_modify_args *args);
 int lib_interface_zebra_evpn_mh_bypass_modify(struct nb_cb_modify_args *args);
 int lib_interface_zebra_evpn_mh_uplink_modify(struct nb_cb_modify_args *args);
-#if defined(HAVE_RTADV)
 int lib_interface_zebra_ipv6_router_advertisements_send_advertisements_modify(
 	struct nb_cb_modify_args *args);
 int lib_interface_zebra_ipv6_router_advertisements_max_rtr_adv_interval_modify(
@@ -265,7 +268,14 @@ int lib_interface_zebra_ipv6_router_advertisements_dnssl_dnssl_domain_lifetime_m
 	struct nb_cb_modify_args *args);
 int lib_interface_zebra_ipv6_router_advertisements_dnssl_dnssl_domain_lifetime_destroy(
 	struct nb_cb_destroy_args *args);
-#endif /* defined(HAVE_RTADV) */
+int lib_interface_zebra_ipv6_router_advertisements_pref64_pref64_prefix_create(
+	struct nb_cb_create_args *args);
+int lib_interface_zebra_ipv6_router_advertisements_pref64_pref64_prefix_destroy(
+	struct nb_cb_destroy_args *args);
+int lib_interface_zebra_ipv6_router_advertisements_pref64_pref64_prefix_lifetime_modify(
+	struct nb_cb_modify_args *args);
+int lib_interface_zebra_ipv6_router_advertisements_pref64_pref64_prefix_lifetime_destroy(
+	struct nb_cb_destroy_args *args);
 #if HAVE_BFDD == 0
 int lib_interface_zebra_ptm_enable_modify(struct nb_cb_modify_args *args);
 #endif
@@ -449,6 +459,9 @@ lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_srv6_segs_stack_e
 struct yang_data *
 lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_srv6_segs_stack_entry_seg_get_elem(
 	struct nb_cb_get_elem_args *args);
+struct yang_data *
+lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_srv6_segs_stack_encap_behavior_get_elem(
+	struct nb_cb_get_elem_args *args);
 const void *
 lib_vrf_zebra_ribs_rib_route_route_entry_nexthop_group_nexthop_mpls_label_stack_entry_get_next(
 	struct nb_cb_get_next_args *args);
