@@ -197,9 +197,14 @@ static int show_nht_rm(struct vty *vty, int af_type, const char *vrf_all,
 		       const char *vrf_name, bool use_json)
 {
 	struct zebra_vrf *zvrf;
+	vrf_id_t vrf_id = VRF_DEFAULT;
 	json_object *json = NULL;
 	json_object *json_vrfs = NULL;
 
+	/* Check for single vrf name. Note that this macro returns on error. */
+	if (vrf_all == NULL && vrf_name != NULL)
+		VRF_GET_ID(vrf_id, vrf_name, false);
+
 	if (use_json) {
 		json = json_object_new_object();
 		json_vrfs = json_object_new_object();
@@ -236,10 +241,6 @@ static int show_nht_rm(struct vty *vty, int af_type, const char *vrf_all,
 	} else {
 		json_object *json_proto = NULL;
 		json_object *json_vrf = NULL;
-		vrf_id_t vrf_id = VRF_DEFAULT;
-
-		if (vrf_name)
-			VRF_GET_ID(vrf_id, vrf_name, false);
 
 		zvrf = zebra_vrf_lookup_by_id(vrf_id);
 		if (!zvrf) {
@@ -1183,7 +1184,7 @@ void zebra_route_map_set_delay_timer(uint32_t value)
 	if (!value && zebra_t_rmap_update) {
 		/* Event driven route map updates is being disabled */
 		/* But there's a pending timer. Fire it off now */
-		EVENT_OFF(zebra_t_rmap_update);
+		event_cancel(&zebra_t_rmap_update);
 		zebra_route_map_update_timer(NULL);
 	}
 }
@@ -1193,7 +1194,7 @@ void zebra_routemap_finish(void)
 	/* Set zebra_rmap_update_timer to 0 so that it wont schedule again */
 	zebra_rmap_update_timer = 0;
 	/* Thread off if any scheduled already */
-	EVENT_OFF(zebra_t_rmap_update);
+	event_cancel(&zebra_t_rmap_update);
 	route_map_finish();
 }
 
@@ -1295,7 +1296,7 @@ static void zebra_route_map_mark_update(const char *rmap_name)
 {
 	/* rmap_update_timer of 0 means don't do route updates */
 	if (zebra_rmap_update_timer)
-		EVENT_OFF(zebra_t_rmap_update);
+		event_cancel(&zebra_t_rmap_update);
 
 	event_add_timer(zrouter.master, zebra_route_map_update_timer, NULL,
 			zebra_rmap_update_timer, &zebra_t_rmap_update);
