@@ -180,7 +180,7 @@ void zebra_router_sweep_route(void)
 
 void zebra_router_sweep_nhgs(void)
 {
-	zebra_nhg_sweep_table(zrouter.nhgs_id);
+	zebra_nhg_sweep_table(zrouter.nhgs_id, false);
 }
 
 static void zebra_router_free_table(struct zebra_router_table *zrt)
@@ -237,7 +237,7 @@ void zebra_router_terminate(void)
 		zrouter.ra_wheel = NULL;
 	}
 
-	EVENT_OFF(zrouter.t_rib_sweep);
+	event_cancel(&zrouter.t_rib_sweep);
 
 	RB_FOREACH_SAFE (zrt, zebra_router_table_head, &zrouter.tables, tmp)
 		zebra_router_free_table(zrt);
@@ -291,6 +291,9 @@ void zebra_router_init(bool asic_offload, bool notify_on_ack,
 
 	zrouter.nhg_keep = ZEBRA_DEFAULT_NHG_KEEP_TIMER;
 
+	/* Initialize the red-black tree for router tables */
+	RB_INIT(zebra_router_table_head, &zrouter.tables);
+
 	/*Init V6 RA batching stuffs*/
 	zrouter.ra_wheel = wheel_init(zrouter.master, RTADV_TIMER_WHEEL_PERIOD_MS,
 				      RTADV_TIMER_WHEEL_SLOTS_NO, interface_hash_key, process_rtadv,
