@@ -16,6 +16,7 @@
 #include "vrf.h"
 #include "srv6.h"
 #include "lib/json.h"
+#include "termtable.h"
 
 #include "zebra/zserv.h"
 #include "zebra/zebra_router.h"
@@ -210,7 +211,7 @@ DEFUN (show_srv6_locator_detail,
 	}
 
 	for (ALL_LIST_ELEMENTS_RO(srv6->locators, node, locator)) {
-		struct listnode *node;
+		struct listnode *nnode;
 		struct srv6_locator_chunk *chunk;
 
 		if (strcmp(locator->name, locator_name) != 0)
@@ -245,10 +246,11 @@ DEFUN (show_srv6_locator_detail,
 			if (CHECK_FLAG(locator->flags, SRV6_LOCATOR_USID))
 				vty_out(vty, "Behavior: uSID\n");
 		}
+		if (CHECK_FLAG(locator->flags, SRV6_LOCATOR_PSP))
+			vty_out(vty, "Flavor: PSP\n");
 
 		vty_out(vty, "Chunks:\n");
-		for (ALL_LIST_ELEMENTS_RO((struct list *)locator->chunks, node,
-					  chunk)) {
+		for (ALL_LIST_ELEMENTS_RO((struct list *)locator->chunks, nnode, chunk)) {
 			prefix2str(&chunk->prefix, str, sizeof(str));
 			vty_out(vty, "- prefix: %s, owner: %s\n", str,
 				zebra_route_string(chunk->proto));
@@ -259,6 +261,405 @@ DEFUN (show_srv6_locator_detail,
 	return CMD_SUCCESS;
 }
 
+static const char *show_srv6_sid_seg6_action(enum seg6local_action_t behavior)
+{
+	switch (behavior) {
+	case ZEBRA_SEG6_LOCAL_ACTION_END:
+		return "uN";
+	case ZEBRA_SEG6_LOCAL_ACTION_END_X:
+		return "uA";
+	case ZEBRA_SEG6_LOCAL_ACTION_END_DX6:
+		return "uDX6";
+	case ZEBRA_SEG6_LOCAL_ACTION_END_DX4:
+		return "uDX4";
+	case ZEBRA_SEG6_LOCAL_ACTION_END_DT6:
+		return "uDT6";
+	case ZEBRA_SEG6_LOCAL_ACTION_END_DT4:
+		return "uDT4";
+	case ZEBRA_SEG6_LOCAL_ACTION_END_DT46:
+		return "uDT46";
+	case ZEBRA_SEG6_LOCAL_ACTION_UNSPEC:
+		return "unspec";
+	case ZEBRA_SEG6_LOCAL_ACTION_END_T:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_DX2:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_B6:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_B6_ENCAP:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_BM:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_S:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_AS:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_AM:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_BPF:
+		break;
+	}
+
+	return "unknown";
+}
+
+static const char *show_srv6_sid_seg6_context(char *str, size_t size, const struct srv6_sid_ctx *ctx,
+					      enum seg6local_action_t behavior)
+{
+	struct vrf *vrf;
+	struct interface *ifp;
+
+	switch (behavior) {
+	case ZEBRA_SEG6_LOCAL_ACTION_END:
+		break;
+	case ZEBRA_SEG6_LOCAL_ACTION_END_X:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_DX6:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_DX4:
+		RB_FOREACH (vrf, vrf_id_head, &vrfs_by_id) {
+			ifp = if_lookup_by_index(ctx->ifindex, vrf->vrf_id);
+			if (ifp)
+				snprintf(str, size, "Interface '%s'", ifp->name);
+		}
+		break;
+	case ZEBRA_SEG6_LOCAL_ACTION_END_T:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_DT6:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_DT4:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_DT46:
+		vrf = vrf_lookup_by_id(ctx->vrf_id);
+		snprintf(str, size, "VRF '%s'", vrf ? vrf->name : "<unknown>");
+		break;
+	case ZEBRA_SEG6_LOCAL_ACTION_END_DX2:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_B6:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_B6_ENCAP:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_BM:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_S:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_AS:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_AM:
+	case ZEBRA_SEG6_LOCAL_ACTION_END_BPF:
+	case ZEBRA_SEG6_LOCAL_ACTION_UNSPEC:
+		break;
+	}
+
+	return str;
+}
+
+static void do_show_srv6_sid_line(struct ttable *tt, struct zebra_srv6_sid *sid)
+{
+	struct listnode *node;
+	struct zserv *client;
+	char clients[256];
+	char ctx[256] = {};
+	char behavior[256] = {};
+	char alloc_mode_str[10] = {};
+	char locator_name[SRV6_LOCNAME_SIZE];
+	int ret;
+
+	/* Zclients */
+	if (listcount(sid->client_list)) {
+		bool first = true;
+		int i = 0;
+		for (ALL_LIST_ELEMENTS_RO(sid->client_list, node, client)) {
+			if (first) {
+				ret = snprintf(clients + i, sizeof(clients) - i, "%s(%d)",
+					       zebra_route_string(client->proto), client->instance);
+				first = false;
+			} else {
+				ret = snprintf(clients + i, sizeof(clients) - i, ", %s(%d)",
+					       zebra_route_string(client->proto), client->instance);
+			}
+
+			if (ret > 0)
+				i += ret;
+		}
+	}
+
+	/* Behavior */
+	if (sid->locator) {
+		if ((sid->locator->sid_format &&
+		     sid->locator->sid_format->type == SRV6_SID_FORMAT_TYPE_USID) ||
+		    (!sid->locator->sid_format &&
+		     CHECK_FLAG(sid->locator->flags, SRV6_LOCATOR_USID))) {
+			snprintf(behavior, sizeof(behavior), "%s",
+				 show_srv6_sid_seg6_action(sid->ctx->ctx.behavior));
+		} else {
+			snprintf(behavior, sizeof(behavior), "%s",
+				 seg6local_action2str(sid->ctx->ctx.behavior));
+		}
+	}
+
+	/* SID context */
+	show_srv6_sid_seg6_context(ctx, sizeof(ctx), &sid->ctx->ctx, sid->ctx->ctx.behavior);
+
+	if (strlen(ctx) == 0)
+		snprintf(ctx, sizeof(ctx), "-");
+
+	if (sid->locator)
+		snprintf(locator_name, sizeof(locator_name), "%s", sid->locator->name);
+	else
+		snprintf(locator_name, sizeof(locator_name), "-");
+
+	snprintf(alloc_mode_str, sizeof(alloc_mode_str), "%s",
+		 srv6_sid_alloc_mode2str(sid->alloc_mode));
+
+	ttable_add_row(tt, "%pI6|%s|%s|%s|%s|%s", &sid->value, behavior, ctx, clients, locator_name,
+		       alloc_mode_str);
+}
+
+static void do_show_srv6_sid_json(struct vty *vty, json_object **json, struct srv6_locator *locator,
+				  struct zebra_srv6_sid_ctx *sid_ctx)
+{
+	json_object *json_sid_ctx = NULL;
+	json_object *json_sid = NULL;
+	json_object *json_sid_clients = NULL;
+	json_object *json_sid_client = NULL;
+	struct vrf *vrf;
+	struct zebra_vrf *zvrf;
+	struct interface *ifp;
+	struct listnode *node;
+	struct zserv *client;
+	char buf[256];
+
+	if (!sid_ctx || !sid_ctx->sid)
+		return;
+
+	if (locator && sid_ctx->sid->locator != locator)
+		return;
+
+	json_sid = json_object_new_object();
+	json_sid_ctx = json_object_new_object();
+
+	json_object_string_addf(json_sid, "sid", "%pI6", &sid_ctx->sid->value);
+	if ((sid_ctx->sid->locator->sid_format &&
+	     sid_ctx->sid->locator->sid_format->type == SRV6_SID_FORMAT_TYPE_USID) ||
+	    (!sid_ctx->sid->locator->sid_format &&
+	     CHECK_FLAG(sid_ctx->sid->locator->flags, SRV6_LOCATOR_USID))) {
+		json_object_string_add(json_sid, "behavior",
+				       show_srv6_sid_seg6_action(sid_ctx->ctx.behavior));
+	} else {
+		json_object_string_add(json_sid, "behavior",
+				       seg6local_action2str(sid_ctx->ctx.behavior));
+	}
+
+	if (sid_ctx->ctx.vrf_id) {
+		json_object_int_add(json_sid_ctx, "vrfId", sid_ctx->ctx.vrf_id);
+
+		vrf = vrf_lookup_by_id(sid_ctx->ctx.vrf_id);
+		if (vrf)
+			json_object_string_add(json_sid_ctx, "vrfName", vrf->name);
+
+		zvrf = vrf_info_lookup(sid_ctx->ctx.vrf_id);
+		if (vrf)
+			json_object_int_add(json_sid_ctx, "table", zvrf->table_id);
+	}
+	if (sid_ctx->ctx.ifindex) {
+		json_object_int_add(json_sid_ctx, "interfaceIndex", sid_ctx->ctx.ifindex);
+		RB_FOREACH (vrf, vrf_id_head, &vrfs_by_id) {
+			ifp = if_lookup_by_index(sid_ctx->ctx.ifindex, vrf->vrf_id);
+			if (ifp)
+				json_object_string_add(json_sid_ctx, "interfaceName", ifp->name);
+		}
+	}
+	if (memcmp(&sid_ctx->ctx.nh6, &in6addr_any, sizeof(struct in6_addr)) != 0) {
+		json_object_string_addf(json_sid_ctx, "nexthopIpv6Address", "%pI6",
+					&sid_ctx->ctx.nh6);
+	}
+	json_object_object_add(json_sid, "context", json_sid_ctx);
+
+	json_object_string_add(json_sid, "locator", sid_ctx->sid->locator->name);
+	json_object_string_add(json_sid, "allocationMode",
+			       srv6_sid_alloc_mode2str(sid_ctx->sid->alloc_mode));
+
+	/* Zclients */
+	json_sid_clients = json_object_new_array();
+	if (listcount(sid_ctx->sid->client_list)) {
+		for (ALL_LIST_ELEMENTS_RO(sid_ctx->sid->client_list, node, client)) {
+			json_sid_client = json_object_new_object();
+			json_object_string_add(json_sid_client, "protocol",
+					       zebra_route_string(client->proto));
+			json_object_int_add(json_sid_client, "instance", client->instance);
+			json_object_array_add(json_sid_clients, json_sid_client);
+		}
+	}
+	json_object_object_add(json_sid, "clients", json_sid_clients);
+
+	json_object_object_add(*json, inet_ntop(AF_INET6, &sid_ctx->sid->value, buf, sizeof(buf)),
+			       json_sid);
+}
+
+static void do_show_srv6_sid_specific(struct vty *vty, json_object **json,
+				      struct srv6_locator *locator,
+				      struct zebra_srv6_sid_ctx *sid_ctx)
+{
+	struct ttable *tt;
+
+	if (json) {
+		do_show_srv6_sid_json(vty, json, locator, sid_ctx);
+	} else {
+		/* Prepare table. */
+		tt = ttable_new(&ttable_styles[TTSTYLE_BLANK]);
+
+		ttable_add_row(tt, "SID|Behavior|Context|Daemon/Instance|Locator|AllocationType");
+		tt->style.cell.rpad = 2;
+		tt->style.corner = ' ';
+		ttable_restyle(tt);
+		ttable_rowseps(tt, 0, BOTTOM, true, '-');
+
+		if (!sid_ctx || !sid_ctx->sid) {
+			ttable_del(tt);
+			return;
+		}
+
+		if (locator && sid_ctx->sid->locator != locator) {
+			ttable_del(tt);
+			return;
+		}
+
+		do_show_srv6_sid_line(tt, sid_ctx->sid);
+
+		ttable_colseps(tt, 1, RIGHT, true, ' ');
+		ttable_colseps(tt, 2, LEFT, true, ' ');
+		ttable_colseps(tt, 2, RIGHT, true, ' ');
+		ttable_colseps(tt, 3, LEFT, true, ' ');
+		ttable_colseps(tt, 3, RIGHT, true, ' ');
+		ttable_colseps(tt, 4, LEFT, true, ' ');
+		ttable_colseps(tt, 4, RIGHT, true, ' ');
+		ttable_colseps(tt, 5, LEFT, true, ' ');
+
+		/* Dump the generated table. */
+		if (tt->nrows > 1) {
+			char *table;
+
+			table = ttable_dump(tt, "\n");
+			vty_out(vty, "%s\n", table);
+			XFREE(MTYPE_TMP_TTABLE, table);
+		}
+		ttable_del(tt);
+	}
+}
+
+static void do_show_srv6_sid_all(struct vty *vty, json_object **json, struct srv6_locator *locator)
+{
+	struct zebra_srv6 *srv6 = zebra_srv6_get_default();
+	struct zebra_srv6_sid_ctx *ctx;
+	struct listnode *node;
+	struct ttable *tt;
+
+	if (json) {
+		for (ALL_LIST_ELEMENTS_RO(srv6->sids, node, ctx)) {
+			/* Skip contexts not associated with any SID */
+			if (!ctx->sid)
+				continue;
+
+			/* Skip SIDs from locators we are not interested in */
+			if (locator && ctx->sid->locator != locator)
+				continue;
+
+			do_show_srv6_sid_json(vty, json, locator, ctx);
+		}
+	} else {
+		/* Prepare table. */
+		tt = ttable_new(&ttable_styles[TTSTYLE_BLANK]);
+		ttable_add_row(tt, "SID|Behavior|Context|Daemon/Instance|Locator|AllocationType");
+		tt->style.cell.rpad = 2;
+		tt->style.corner = ' ';
+		ttable_restyle(tt);
+		ttable_rowseps(tt, 0, BOTTOM, true, '-');
+
+		for (ALL_LIST_ELEMENTS_RO(srv6->sids, node, ctx)) {
+			/* Skip contexts not associated with any SID */
+			if (!ctx->sid)
+				continue;
+
+			/* Skip SIDs from locators we are not interested in */
+			if (locator && ctx->sid->locator != locator)
+				continue;
+
+			do_show_srv6_sid_line(tt, ctx->sid);
+		}
+
+		ttable_colseps(tt, 1, RIGHT, true, ' ');
+		ttable_colseps(tt, 2, LEFT, true, ' ');
+		ttable_colseps(tt, 2, RIGHT, true, ' ');
+		ttable_colseps(tt, 3, LEFT, true, ' ');
+		ttable_colseps(tt, 3, RIGHT, true, ' ');
+		ttable_colseps(tt, 4, LEFT, true, ' ');
+		ttable_colseps(tt, 4, RIGHT, true, ' ');
+		ttable_colseps(tt, 5, LEFT, true, ' ');
+
+		/* Dump the generated table. */
+		if (tt->nrows > 1) {
+			char *table;
+
+			table = ttable_dump(tt, "\n");
+			vty_out(vty, "%s\n", table);
+			XFREE(MTYPE_TMP_TTABLE, table);
+		}
+		ttable_del(tt);
+	}
+}
+
+DEFPY (show_srv6_sid,
+       show_srv6_sid_cmd,
+       "show segment-routing srv6 [locator NAME$locator_name] sid [X:X::X:X$sid_value] [json]",
+       SHOW_STR
+       "Segment Routing\n"
+       "Segment Routing SRv6\n"
+       "Locator Information\n"
+       "Locator Name\n"
+       "SID\n"
+       "SID value\n"
+       JSON_STR)
+{
+	bool uj = use_json(argc, argv);
+	struct zebra_srv6 *srv6 = zebra_srv6_get_default();
+	struct srv6_locator *locator = NULL;
+	struct zebra_srv6_sid_ctx *sid_ctx = NULL, *c;
+	struct listnode *node;
+	json_object *json = NULL;
+
+	if (uj)
+		json = json_object_new_object();
+
+	if (locator_name) {
+		locator = zebra_srv6_locator_lookup(locator_name);
+		if (!locator) {
+			if (uj)
+				vty_json(vty, json); /* Return empty json */
+			else
+				vty_out(vty, "%% Can't find the SRv6 locator\n");
+			return CMD_WARNING;
+		}
+	}
+
+	if (!IPV6_ADDR_SAME(&sid_value, &in6addr_any)) {
+		for (ALL_LIST_ELEMENTS_RO(srv6->sids, node, c)) {
+			if (c->sid && IPV6_ADDR_SAME(&c->sid->value, &sid_value)) {
+				sid_ctx = c;
+				break;
+			}
+		}
+
+		if (!sid_ctx) {
+			if (uj)
+				vty_json(vty, json); /* Return empty json */
+			else
+				vty_out(vty, "%% Can't find the SRv6 SID\n");
+			return CMD_WARNING;
+		}
+	}
+
+	if (locator && sid_ctx)
+		if (!sid_ctx->sid || sid_ctx->sid->locator != locator) {
+			if (uj)
+				vty_json(vty, json); /* Return empty json */
+			else
+				vty_out(vty, "%% Can't find the SRv6 SID in the provided locator\n");
+			return CMD_WARNING;
+		}
+
+	if (sid_ctx)
+		do_show_srv6_sid_specific(vty, uj ? &json : NULL, locator, sid_ctx);
+	else
+		do_show_srv6_sid_all(vty, uj ? &json : NULL, locator);
+
+	if (uj)
+		vty_json(vty, json);
+
+	return CMD_SUCCESS;
+}
+
 DEFUN_NOSH (segment_routing,
             segment_routing_cmd,
             "segment-routing",
@@ -357,6 +758,7 @@ DEFUN (no_srv6_locator,
 	struct listnode *node, *nnode;
 	struct zebra_srv6_sid_ctx *ctx;
 	struct srv6_locator *locator = zebra_srv6_locator_lookup(argv[2]->arg);
+
 	if (!locator) {
 		vty_out(vty, "%% Can't find SRv6 locator\n");
 		return CMD_WARNING_CONFIG_FAILED;
@@ -390,7 +792,7 @@ DEFUN (no_srv6_locator,
 DEFPY (locator_prefix,
        locator_prefix_cmd,
        "prefix X:X::X:X/M$prefix [block-len (16-64)$block_bit_len]  \
-	        [node-len (16-64)$node_bit_len] [func-bits (0-64)$func_bit_len]",
+	        [node-len (0-64)$node_bit_len] [func-bits (0-64)$func_bit_len]",
        "Configure SRv6 locator prefix\n"
        "Specify SRv6 locator prefix\n"
        "Configure SRv6 locator block length in bits\n"
@@ -405,9 +807,16 @@ DEFPY (locator_prefix,
 	struct listnode *node = NULL;
 	uint8_t expected_prefixlen;
 	struct srv6_sid_format *format;
+	int idx = 0;
+	bool node_bit_not_conf = false;
 
 	locator->prefix = *prefix;
-	func_bit_len = func_bit_len ?: ZEBRA_SRV6_FUNCTION_LENGTH;
+	/* Only set default if func_bit_len was not provided in command */
+	if (func_bit_len == 0 && !argv_find(argv, argc, "func-bits", &idx))
+		func_bit_len = ZEBRA_SRV6_FUNCTION_LENGTH;
+
+	if (node_bit_len == 0 && !argv_find(argv, argc, "node-len", &idx))
+		node_bit_not_conf = true;
 
 	expected_prefixlen = prefix->prefixlen;
 	format = locator->sid_format;
@@ -431,13 +840,13 @@ DEFPY (locator_prefix,
 	}
 
 	/* Resolve optional arguments */
-	if (block_bit_len == 0 && node_bit_len == 0) {
+	if (block_bit_len == 0 && node_bit_not_conf) {
 		block_bit_len = prefix->prefixlen -
 				ZEBRA_SRV6_LOCATOR_NODE_LENGTH;
 		node_bit_len = ZEBRA_SRV6_LOCATOR_NODE_LENGTH;
 	} else if (block_bit_len == 0) {
 		block_bit_len = prefix->prefixlen - node_bit_len;
-	} else if (node_bit_len == 0) {
+	} else if (node_bit_not_conf) {
 		node_bit_len = prefix->prefixlen - block_bit_len;
 	} else {
 		if (block_bit_len + node_bit_len != prefix->prefixlen) {
@@ -484,14 +893,10 @@ DEFPY (locator_prefix,
 
 			if (memcmp(&chunk->prefix.prefix, zero, 16) == 0) {
 				struct zserv *client;
-				struct listnode *client_node;
 
 				chunk->prefix = *prefix;
-				for (ALL_LIST_ELEMENTS_RO(zrouter.client_list,
-							  client_node,
-							  client)) {
+				frr_each (zserv_client_list, &zrouter.client_list, client) {
 					struct srv6_locator *tmp;
-
 					if (client->proto != chunk->proto)
 						continue;
 
@@ -543,6 +948,38 @@ DEFPY (locator_behavior,
 	return CMD_SUCCESS;
 }
 
+DEFPY (locator_flavor_psp,
+       locator_flavor_psp_cmd,
+       "[no] flavor psp",
+       NO_STR
+       "Configure SRv6 flavors\n"
+       "Specify Penultimate Segment Popping flavor\n")
+{
+	VTY_DECLVAR_CONTEXT(srv6_locator, locator);
+
+	if (no && !CHECK_FLAG(locator->flags, SRV6_LOCATOR_PSP))
+		/* SRv6 locator PSP flag already unset, nothing to do */
+		return CMD_SUCCESS;
+
+	if (!no && CHECK_FLAG(locator->flags, SRV6_LOCATOR_PSP))
+		/* SRv6 locator PSP flag already set, nothing to do */
+		return CMD_SUCCESS;
+
+	/* Remove old locator from zclients */
+	zebra_notify_srv6_locator_delete(locator);
+
+	/* Set/Unset the SRV6_LOCATOR_USID */
+	if (no)
+		UNSET_FLAG(locator->flags, SRV6_LOCATOR_PSP);
+	else
+		SET_FLAG(locator->flags, SRV6_LOCATOR_PSP);
+
+	/* Notify the new locator to zclients */
+	zebra_srv6_locator_add(locator);
+
+	return CMD_SUCCESS;
+}
+
 DEFPY(locator_sid_format,
       locator_sid_format_cmd,
       "format <usid-f3216|uncompressed-f4024>$format",
@@ -941,6 +1378,60 @@ DEFPY(no_srv6_sid_format_explicit,
 	return CMD_SUCCESS;
 }
 
+/* Helper function to check if a SID format is using the default config */
+static bool has_default_sid_format_config(struct srv6_sid_format *format)
+{
+	bool has_default_config = true;
+
+	switch (format->type) {
+	case SRV6_SID_FORMAT_TYPE_USID:
+		if (format->config.usid.lib_start != SRV6_SID_FORMAT_USID_F3216_LIB_START)
+			has_default_config = false;
+
+		if (format->config.usid.elib_start != SRV6_SID_FORMAT_USID_F3216_ELIB_START)
+			has_default_config = false;
+
+		if (format->config.usid.elib_end != SRV6_SID_FORMAT_USID_F3216_ELIB_END)
+			has_default_config = false;
+
+		if (format->config.usid.wlib_start != SRV6_SID_FORMAT_USID_F3216_WLIB_START)
+			has_default_config = false;
+
+		if (format->config.usid.wlib_end != SRV6_SID_FORMAT_USID_F3216_WLIB_END)
+			has_default_config = false;
+
+		if (format->config.usid.ewlib_start != SRV6_SID_FORMAT_USID_F3216_EWLIB_START)
+			has_default_config = false;
+
+		break;
+
+	case SRV6_SID_FORMAT_TYPE_UNCOMPRESSED:
+		if (format->config.uncompressed.explicit_start !=
+		    SRV6_SID_FORMAT_UNCOMPRESSED_F4024_EXPLICIT_RANGE_START)
+			has_default_config = false;
+
+		break;
+
+	case SRV6_SID_FORMAT_TYPE_UNSPEC:
+		break;
+	}
+
+	return has_default_config;
+}
+
+/* Helper function to check if all SID formats are using the default config */
+static bool has_default_sid_format_config_all(void)
+{
+	struct zebra_srv6 *srv6 = zebra_srv6_get_default();
+	struct listnode *node;
+	struct srv6_sid_format *format;
+
+	for (ALL_LIST_ELEMENTS_RO(srv6->sid_formats, node, format))
+		if (!has_default_sid_format_config(format))
+			return false;
+
+	return true;
+}
 static int zebra_sr_config(struct vty *vty)
 {
 	struct zebra_srv6 *srv6 = zebra_srv6_get_default();
@@ -973,21 +1464,25 @@ static int zebra_sr_config(struct vty *vty)
 			vty_out(vty, "   locator %s\n", locator->name);
 			vty_out(vty, "    prefix %s/%u", str,
 				locator->prefix.prefixlen);
-			if (locator->block_bits_length)
+			if (locator->block_bits_length !=
+			    locator->prefix.prefixlen - ZEBRA_SRV6_LOCATOR_NODE_LENGTH)
 				vty_out(vty, " block-len %u",
 					locator->block_bits_length);
-			if (locator->node_bits_length)
+			if (locator->node_bits_length != ZEBRA_SRV6_LOCATOR_NODE_LENGTH)
 				vty_out(vty, " node-len %u",
 					locator->node_bits_length);
-			if (locator->function_bits_length)
-				vty_out(vty, " func-bits %u",
-					locator->function_bits_length);
+
+			if (locator->function_bits_length != ZEBRA_SRV6_FUNCTION_LENGTH)
+				vty_out(vty, " func-bits %u", locator->function_bits_length);
+
 			if (locator->argument_bits_length)
 				vty_out(vty, " arg-len %u",
 					locator->argument_bits_length);
 			vty_out(vty, "\n");
 			if (CHECK_FLAG(locator->flags, SRV6_LOCATOR_USID))
 				vty_out(vty, "    behavior usid\n");
+			if (CHECK_FLAG(locator->flags, SRV6_LOCATOR_PSP))
+				vty_out(vty, "    flavor psp\n");
 			if (locator->sid_format) {
 				format = locator->sid_format;
 				vty_out(vty, "    format %s\n", format->name);
@@ -997,50 +1492,55 @@ static int zebra_sr_config(struct vty *vty)
 		}
 		vty_out(vty, "  exit\n");
 		vty_out(vty, "  !\n");
-		vty_out(vty, "  formats\n");
-		for (ALL_LIST_ELEMENTS_RO(srv6->sid_formats, node, format)) {
-			if (format->type == SRV6_SID_FORMAT_TYPE_UNCOMPRESSED) {
-				vty_out(vty, "   format %s\n", format->name);
-				if (format->config.uncompressed.explicit_start !=
-				    SRV6_SID_FORMAT_UNCOMPRESSED_F4024_EXPLICIT_RANGE_START)
-					vty_out(vty, "    explicit start %u\n",
-						format->config.uncompressed
-							.explicit_start);
+
+		if (!has_default_sid_format_config_all()) {
+			vty_out(vty, "  formats\n");
+			for (ALL_LIST_ELEMENTS_RO(srv6->sid_formats, node, format)) {
+				if (has_default_sid_format_config(format))
+					/* This SID format is using the default config, skipping */
+					continue;
+
+				if (format->type == SRV6_SID_FORMAT_TYPE_UNCOMPRESSED) {
+					vty_out(vty, "   format %s\n", format->name);
+					if (format->config.uncompressed.explicit_start !=
+					    SRV6_SID_FORMAT_UNCOMPRESSED_F4024_EXPLICIT_RANGE_START)
+						vty_out(vty, "    explicit start %u\n",
+							format->config.uncompressed.explicit_start);
+				}
+				if (format->type == SRV6_SID_FORMAT_TYPE_USID) {
+					vty_out(vty, "   format %s\n", format->name);
+					if (format->config.usid.lib_start !=
+					    SRV6_SID_FORMAT_USID_F3216_LIB_START)
+						vty_out(vty, "    local-id-block start %u\n",
+							format->config.usid.lib_start);
+					if (format->config.usid.elib_start !=
+						    SRV6_SID_FORMAT_USID_F3216_ELIB_START ||
+					    format->config.usid.elib_end !=
+						    SRV6_SID_FORMAT_USID_F3216_ELIB_END)
+						vty_out(vty,
+							"    local-id-block explicit start %u end %u\n",
+							format->config.usid.elib_start,
+							format->config.usid.elib_end);
+					if (format->config.usid.wlib_start !=
+						    SRV6_SID_FORMAT_USID_F3216_WLIB_START ||
+					    format->config.usid.wlib_end !=
+						    SRV6_SID_FORMAT_USID_F3216_WLIB_END)
+						vty_out(vty,
+							"    wide-local-id-block start %u end %u\n",
+							format->config.usid.wlib_start,
+							format->config.usid.wlib_end);
+					if (format->config.usid.ewlib_start !=
+					    SRV6_SID_FORMAT_USID_F3216_EWLIB_START)
+						vty_out(vty,
+							"    wide-local-id-block explicit start %u\n",
+							format->config.usid.ewlib_start);
+				}
+				vty_out(vty, "   exit\n");
+				vty_out(vty, "   !\n");
 			}
-			if (format->type == SRV6_SID_FORMAT_TYPE_USID) {
-				vty_out(vty, "   format %s\n", format->name);
-				if (format->config.usid.lib_start !=
-				    SRV6_SID_FORMAT_USID_F3216_LIB_START)
-					vty_out(vty,
-						"    local-id-block start %u\n",
-						format->config.usid.lib_start);
-				if (format->config.usid.elib_start !=
-					    SRV6_SID_FORMAT_USID_F3216_ELIB_START ||
-				    format->config.usid.elib_end !=
-					    SRV6_SID_FORMAT_USID_F3216_ELIB_END)
-					vty_out(vty,
-						"    local-id-block explicit start %u end %u\n",
-						format->config.usid.elib_start,
-						format->config.usid.elib_end);
-				if (format->config.usid.wlib_start !=
-					    SRV6_SID_FORMAT_USID_F3216_WLIB_START ||
-				    format->config.usid.wlib_end !=
-					    SRV6_SID_FORMAT_USID_F3216_WLIB_END)
-					vty_out(vty,
-						"    wide-local-id-block start %u end %u\n",
-						format->config.usid.wlib_start,
-						format->config.usid.wlib_end);
-				if (format->config.usid.ewlib_start !=
-				    SRV6_SID_FORMAT_USID_F3216_EWLIB_START)
-					vty_out(vty,
-						"    wide-local-id-block explicit start %u\n",
-						format->config.usid.ewlib_start);
-			}
-			vty_out(vty, "   exit\n");
-			vty_out(vty, "   !\n");
+			vty_out(vty, "  exit\n");
+			vty_out(vty, "  !\n");
 		}
-		vty_out(vty, "  exit\n");
-		vty_out(vty, "  !\n");
 		vty_out(vty, " exit\n");
 		vty_out(vty, " !\n");
 	}
@@ -1091,6 +1591,7 @@ void zebra_srv6_vty_init(void)
 	/* Command for configuration */
 	install_element(SRV6_LOC_NODE, &locator_prefix_cmd);
 	install_element(SRV6_LOC_NODE, &locator_behavior_cmd);
+	install_element(SRV6_LOC_NODE, &locator_flavor_psp_cmd);
 	install_element(SRV6_LOC_NODE, &locator_sid_format_cmd);
 	install_element(SRV6_LOC_NODE, &no_locator_sid_format_cmd);
 	install_element(SRV6_ENCAP_NODE, &srv6_src_addr_cmd);
@@ -1120,4 +1621,5 @@ void zebra_srv6_vty_init(void)
 	install_element(VIEW_NODE, &show_srv6_locator_cmd);
 	install_element(VIEW_NODE, &show_srv6_locator_detail_cmd);
 	install_element(VIEW_NODE, &show_srv6_manager_cmd);
+	install_element(VIEW_NODE, &show_srv6_sid_cmd);
 }
